/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */

package com.sun.jini.admin;

import java.rmi.RemoteException;
import java.io.IOException;

/**
 * Admin interface for controlling the location of a service's persistent
 * storage.  Administrable services are encouraged to have their admin
 * object implement this interface.
 *
 * @see net.jini.admin.Administrable#getAdmin
 */
public interface StorageLocationAdmin {
    /**
     * Returns the location of the service's persistent storage.
     * Typically returns a directory pathname unless the service
     * specifies otherwise.
     */
    String getStorageLocation() throws RemoteException;

    /**
     * Sets the location of the service's persistent storage,
     * moving all current persistent storage from the current
     * location to the specified new location.
     *
     * @exception IOException if moving the persistent storage fails
     */
    void setStorageLocation(String location)
	throws IOException, RemoteException;
}
