/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.example.books;

import java.io.*;
import java.util.Random;
import java.util.Vector;
import java.rmi.RemoteException;

import net.jini.space.*;
import net.jini.core.transaction.*;
import net.jini.core.lease.Lease;

import java.net.URL;
/**
 * A "Blast a bunch of entries" object.
 */
class Blaster {
    JavaSpace space;
    BookBuyer buyer;

    private static Random rand = new Random();
    private static Vector words;

    static {				// set the value of "words"
	FileReader fin = null;
	BufferedReader in = null;
	try {
	    String dict = "com/sun/jini/example/books/resource/words";
	    System.out.print("Reading resource " + dict);

	    ClassLoader cls = Blaster.class.getClassLoader();
	    URL res = cls.getResource(dict);

	    Reader r = new BufferedReader(new InputStreamReader(res.openStream()));
	    System.out.flush();
	    in = new BufferedReader(r);
	    words = new Vector(26500);
	    String w;
//FileWriter o = new FileWriter("t");
	    while ((w = in.readLine()) != null) {
//o.write(w);
//o.write("\n");
//o.flush();
		words.addElement(w);
	    }
	    System.out.println(" " + words.size() + " words");
	} catch (EOFException eof) {
	    ; // normal end of input loop (ugly design)
	} catch (Throwable e) {	// can't continue if *anything* happens
	    e.printStackTrace();
	    System.exit(-1);
	}
	try {
	    if (fin != null)
		fin.close();
	} catch (IOException e) {
	    ; // oh, well, I tried to close it
	}
    }

    Blaster(BookBuyer buyer) {
	this.buyer = buyer;
	space = buyer.space();
    }


    void blast(int blastCnt, Transaction txn) throws RemoteException {
	RFB rfb = new RFB();
	FullRFB full = new FullRFB();
	RFB writing;
	long leaseTime = buyer.leaseTime();

	try {
	    while (blastCnt-- > 0) {
		writing = (Math.random() < 0.3333 ? rfb : full);
		writing.orderNumber = new Integer(buyer.nextOrderNumber());
		writing.title = newTitle();
		writing.count = new Integer(randInt() & 0xfff);
		if (writing == full)
		    full.isbn = newISBN();
		Lease lease = space.write(writing, txn, leaseTime);
		if (leaseTime != Lease.FOREVER)
		    buyer.addLease(writing.toString(), lease);
	    }
	} catch (RemoteException e) {
	    e.printStackTrace();
	} catch (TransactionException e) {
	    e.printStackTrace();
	}
    }

    private String newTitle() {
	return (String) words.elementAt(randInt() % words.size()) + " "
	     + (String) words.elementAt(randInt() % words.size());
    }

    private String newISBN() {
	return (randInt() % 10)   + "-" +
	       (randInt() % 1000) + "-" +
	       (randInt() % 100000) + "-" +
	       ((randInt() & 0x01f) > 20 ? randInt() % 10 + "" :
					   (char) (randInt() % 26 + 'A') + "");
    }

    private int randInt() {
	return rand.nextInt() & 0x7fffffff;
    }
}
