/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.example.books;

import net.jini.space.*;
import net.jini.core.lease.*;

import java.applet.*;
import java.awt.*;
import java.rmi.*;

/**
 * A book buyer applet that lets one place requests for bids on given books
 * If an ISBN is specified, the objects written are <code>FullRFB</code>
 * objects; otherwise they are simply <code>RFB</code> objects.
 * @see RFBApplet
 * @see BookSeller
 */
public class BookBuyer extends RFBApplet {
    private static int nextOrder = 1000;	// the next order number
    private static final boolean DEBUG = false;

    public BookBuyer() throws RemoteException { }

    /**
     * Initialize the extra GUI stuff that we need to be a buyer.
     * Invokes <code>super.init()</code>
     */
    public void init() {
	super.init();
	setOrderNumber();
	setBackground(new Color(0x11, 0x44, 0x8e));
	setForeground(new Color(0xff, 0xff, 0xff));

	buttonPanel().add(new WriteButton(this));
	buttonPanel().add(new BlastButton(this));

	setLeaseList();
    }

    /**
     * Write the record into the space.
     */
    void write() {
	boolean full = !isbn().getText().equals("");
	RFB rfb = (full ? new FullRFB() : new RFB());
	rfb.orderNumber = new Integer(order().getText());
	rfb.title = title().getText();
	rfb.count = new Integer(count().getText());
	if (full)
	    ((FullRFB) rfb).isbn = isbn().getText();
	try {
	    Lease lease = space().write(rfb, currentTxn(), getLeaseValue());
	    getLeaseList().addLease(rfb.toString(), lease);
	} catch (Exception e) {
	    e.printStackTrace();
	}
	setOrderNumber();
    }

    // used by Blaster
    void addLease(String str, Lease lease) {
	getLeaseList().addLease(str, lease);
    }

    // used by Blaster
    long leaseTime() {
	return getLeaseValue();
    }

    // inherit doc comment
    synchronized void setOrderNumber() {
	order().setText(nextOrderNumber() + "");
	if (DEBUG)
	  System.out.println("BookBuyer: setOrderNumber: number= " +
			     order().getText());
    }

    synchronized static int nextOrderNumber() {
	return nextOrder++;
    }

    // inherit doc comment
    void clear() {
	super.clear();
	count().setText("1");
    }

    public static void main(String[] args) throws Throwable {
	main(new BookBuyer());
    }
}
