/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.example.books;

import net.jini.space.*;
import net.jini.core.event.*;
import net.jini.core.lease.*;
import com.sun.jini.example.spaceApplet.*;

import java.awt.*;
import java.rmi.*;

import java.util.Map;
import java.util.ArrayList;
import java.util.Iterator;

class LeaseList extends Panel {
    List	list;
    RFBApplet	applet;
    JavaSpace	space;
    ArrayList	leases;
    TextField	leaseField;

    private static final boolean DEBUG = false;

    LeaseList(int cnt, RFBApplet applet) {
	Panel controls = new Panel();
	setLayout(new BorderLayout());
	add(controls, "North");

	GridBagConstraints c = new GridBagConstraints();

	controls.add(new TimedButton("Cancel", applet) {
	    public void pushed() { cancel(); }
	}, c);
	controls.add(new TimedButton("Renew", applet) {
	    public void pushed() { renew(); }
	}, c);
	controls.add(new Label("Time:"));
	controls.add(leaseField = new TextField("FOREVER"));

	list = new List(cnt, true);
	this.applet = applet;
	space = applet.space();
	leases = new ArrayList();
	add(list, "Center");
    }

    void addLease(String label, Lease lease) {
System.out.println("adding " + label + " (" + lease + ")");
	if (label.startsWith("-1"))
	    new Exception().printStackTrace();
	list.add(label);
	leases.add(lease);
    }

    private Lease[] selectedLeases() {
	int[] indices = list.getSelectedIndexes();
	if (indices == null || indices.length == 0)	// which will it return?
	    return null;
	Lease[] ls = new Lease[indices.length];
	for (int i = 0; i < ls.length; i++)
	    ls[i] = (Lease) leases.get(i);
	return ls;
    }

    /**
     * Return a <code>LeaseMap</code> for the leases in the given array.
     * This assumes that all the leases are batchable together, which is
     * always true in our example, but not true generally.
     */
    private LeaseMap leaseMap(Lease[] ls, long time) {
	Long t = new Long(time);
	LeaseMap map = ls[0].createLeaseMap(time);
	for (int i = 0; i < ls.length; i++)
	    map.put(ls[i], t);
	return map;
    }

    /**
     * Remove the leases at the specified index.  This is the basic method;
     * <code>RemoveLease</code> and <code>RemoveLeases</code> are written in
     * terms of this one.
     */
    public void remove(int i) {
	list.remove(i);
	leases.remove(i);
    }

    /**
     * Remove the specified from the lease.
     */
    void removeLease(Lease lease) {
	remove(leases.indexOf(lease));
    }

    /**
     * Remove all the leases in this map from the GUI.
     */
    void removeLeases(Map map) {
	Iterator it = map.keySet().iterator();
	while (it.hasNext())
	    removeLease((Lease) it.next());
    }

    /**
     * Implement the cancel button.
     */
    private void cancel() {
	Lease[] ls = selectedLeases();
	LeaseMap map = null;
	if (ls == null)
	    return;
	try {
	    if (ls.length == 1) {
		ls[0].cancel();
		removeLease(ls[0]);
	    } else {
		map = leaseMap(ls, 0);
		map.cancelAll();
		removeLeases(map);
	    }
	} catch (RemoteException e) {
	    applet.show(e);
	} catch (UnknownLeaseException e) {
	    removeLease(ls[0]);
	    applet.show(e);
	} catch (LeaseMapException e) {
	    removeLeases(e.exceptionMap);
	    removeLeases(map);			// all these are removed, too
	    applet.show(e);
	}
    }

    /**
     * Implement the renew button.
     */
    private void renew() {
	Lease[] ls = selectedLeases();
	if (ls == null)
	    return;
	try {
	    if (ls.length == 1)
		ls[0].renew(leaseTime());
	    else
		leaseMap(ls, leaseTime()).renewAll();
	} catch (RemoteException e) {
	    applet.show(e);
	} catch (UnknownLeaseException e) {
	    removeLease(ls[0]);
	    applet.show(e);
	} catch (LeaseDeniedException e) {
	    removeLease(ls[0]);
	    applet.show(e);
	} catch (LeaseMapException e) {
	    removeLeases(e.exceptionMap);
	    applet.show(e);
	}
    }

    /**
     * Return the lease time the user has put into the field.
     */
    long leaseTime() {
	String timeStr = leaseField.getText();
	String showStr = timeStr;
	char firstCh;
	if (timeStr.length() > 0)
	    firstCh = Character.toUpperCase(timeStr.charAt(0));
	else
	    firstCh = 'A';	// if nothing is present, use ANY;

	long lease;
	if (firstCh == 'F') {
	    showStr = "FOREVER";
	    lease = Lease.FOREVER;
	} else if (firstCh == 'A') {
	    showStr = "ANY";
	    lease  = Lease.ANY;
	} else {
	    try {
		lease = Long.parseLong(timeStr);
	    } catch (NumberFormatException e) {
		applet.show(e);
		showStr = "ANY";
		lease = Lease.ANY;
	    }
	}
	if (!timeStr.equals(showStr))
	    leaseField.setText(showStr);

        if (DEBUG)
	   System.out.println("LeaseList: leaseTime = " + lease);

	return lease;
    }
}
