/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.example.books;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.AbstractButton;
import javax.swing.JButton;
import java.awt.GridLayout;
import java.awt.event.WindowListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.Font;

public class RunitGUI
{
    
    private boolean    rmiBinderFlag =  false;
    private String     binderName;
    private JPanel     panel;
    private GridLayout classesGUILayout;

    private JButton    startBuyer,stopBuyer;
    private JButton    startSeller,stopSeller;
    private JButton    startWatcher,stopWatcher;

    private RegistryGUI   registryGUIRef;
    private SpacesGUI     spacesGUIRef;

    private RunBuyer       buyerProcess;
    private RunSeller      sellerProcess;
    private RunWatcher     watcherProcess;
    
    public RunitGUI( JPanel panel, String text,
		     RegistryGUI registryGUI,
		     SpacesGUI    spacesGUI) {

	//Save references to the GUI components for later
	//retreival of panel sub-components
	registryGUIRef   = registryGUI;
        spacesGUIRef      = spacesGUI;

	//Setup the Look and Feel for this Panel
	classesGUILayout = (new GridLayout(0, 2));
	classesGUILayout.setHgap(3);
	classesGUILayout.setVgap(3);

	panel.setLayout(classesGUILayout);
	//panel.setFont(new Font("Dialog", Font.PLAIN, 24));

	//Populate the GUI Piecces

	startBuyer = new JButton("Start Buyer");
	stopBuyer  = new JButton("Stop Buyer");

	startSeller= new JButton("Start Seller");
	stopSeller = new JButton("Stop Seller");

	startWatcher = new JButton("Start Watcher");
	stopWatcher  = new JButton("Stop Watcher");

	//Add Callbacks for Mouse Events
	SymMouse aSymMouse = new SymMouse();
	startBuyer.addMouseListener(aSymMouse);
	stopBuyer.addMouseListener(aSymMouse);
	startSeller.addMouseListener(aSymMouse);
	stopSeller.addMouseListener(aSymMouse);
	startWatcher.addMouseListener(aSymMouse);
	stopWatcher.addMouseListener(aSymMouse);

	//Populate the panel
	panel.add( new JLabel(" "));
	panel.add( new JLabel(" "));

	panel.add(startBuyer);
	panel.add(stopBuyer);

	panel.add(startSeller);
	panel.add(stopSeller);

	panel.add(startWatcher);
	panel.add(stopWatcher);

	panel.add( new JLabel(" "));
	panel.add( new JLabel(" "));
    }//end-constructor
    
    class SymMouse extends java.awt.event.MouseAdapter {
	
	public void mouseClicked(java.awt.event.MouseEvent event) {
	    
	    Object object = event.getSource();

	    if (object == startBuyer) {
		System.err.println("Starting Buyer");
		buyerProcess = new 
		    RunBuyer(spacesGUIRef.getPolicyFile(),
			      spacesGUIRef.getSpaceName(),
			      registryGUIRef.getCurrentRegistryType(),
			      registryGUIRef.getLookupGroups(),
			      spacesGUIRef.getJarTop(),
			      spacesGUIRef.getURLTop(),
			      registryGUIRef.getRegistryPort());

	    }

	    if (object == stopBuyer) {
		System.err.println("Stopping Buyer");
		buyerProcess.destroy();
	    }

	    if (object == startSeller) {
		System.err.println("Starting Seller");
		sellerProcess = new 
		    RunSeller(spacesGUIRef.getPolicyFile(),
			      spacesGUIRef.getSpaceName(),
			      registryGUIRef.getCurrentRegistryType(),
			      registryGUIRef.getLookupGroups(),
			      spacesGUIRef.getJarTop(),
			      spacesGUIRef.getURLTop(),
			      registryGUIRef.getRegistryPort());
	    }

	    if (object == stopSeller) {
		System.err.println("Stopping Seller");
		sellerProcess.destroy();

	    }

	    if (object == startWatcher) {
		System.err.println("Starting Watcher");
		watcherProcess = new 
		    RunWatcher(spacesGUIRef.getPolicyFile(),
			      spacesGUIRef.getSpaceName(),
			      registryGUIRef.getCurrentRegistryType(),
			      registryGUIRef.getLookupGroups(),
			      spacesGUIRef.getJarTop(),
			      spacesGUIRef.getURLTop(),
			      registryGUIRef.getRegistryPort());

	    }

	    if (object == stopWatcher) {
		System.err.println("Stopping Watcher");
		watcherProcess.destroy();
	    }
				
	}//end-mouseClicked
    }//end-SymMouse

}//end-class



