/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.example.browser;

import net.jini.core.lookup.*;
import net.jini.core.discovery.LookupLocator;
import net.jini.discovery.*;
import net.jini.core.lease.*;
import net.jini.core.event.*;
import net.jini.core.entry.Entry;
import net.jini.lease.LeaseRenewalManager;
import net.jini.lease.LeaseRenewalEvent;
import net.jini.lease.LeaseListener;
import java.rmi.RMISecurityManager;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import javax.swing.JTextArea;
import javax.swing.JScrollPane;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JCheckBoxMenuItem;
import javax.swing.JRadioButtonMenuItem;
import javax.swing.JMenuBar;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.event.MenuListener;
import javax.swing.event.MenuEvent;

import java.lang.reflect.Field;
import java.lang.reflect.Modifier;
import java.lang.reflect.Constructor;
import java.util.Vector;
import java.util.StringTokenizer;
import net.jini.admin.Administrable;
import net.jini.admin.JoinAdmin;
import net.jini.lookup.entry.ServiceControlled;
import java.rmi.RemoteException;
import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import javax.swing.JList;
import javax.swing.ImageIcon;
import javax.swing.Icon;
import javax.swing.JLabel;
import javax.swing.ListCellRenderer;
import javax.swing.JPanel;
import javax.swing.DefaultListModel;
import javax.swing.UIManager;
import javax.swing.BorderFactory;
import javax.swing.JPopupMenu;
import javax.swing.event.PopupMenuListener;
import javax.swing.event.PopupMenuEvent;
import javax.swing.border.TitledBorder;
import java.util.HashSet;
import java.util.Iterator;

/* This is a bad example of user interface design.  It was a quick-and-dirty
 * hack and an experiment in on-the-fly menu construction, and it's still
 * here because we haven't had time to do anything better.
 */
public class Browser extends JFrame {

    /**
     * @serial
     */
    private String[] origGroups;

    /**
     * @serial
     */
    private LookupDiscovery disco;

    /**
     * @serial
     */
    private ServiceRegistrar lookup;

    /**
     * @serial
     */
    private ActionListener exiter;

    /**
     * @serial
     */
    private ServiceTemplate tmpl;

    /**
     * @serial
     */
    private Listener listen;

    /**
     * @serial
     */
    private LookupListener adder;

    /**
     * @serial
     */
    private Lease elease;

    /**
     * @serial
     */
    private LeaseRenewalManager leaseMgr;

    /**
     * @serial
     */
    private LeaseListener lnotify;

    /**
     * @serial
     */
    private JTextArea text;

    /**
     * @serial
     */
    private JMenu registrars;

    /**
     * @serial
     */
    private JCheckBoxMenuItem esuper;

    /**
     * @serial
     */
    private JCheckBoxMenuItem ssuper;

    /**
     * @serial
     */
    private JCheckBoxMenuItem sclass;

    /**
     * For Admin mode
     * @serial
     */
    private boolean isAdmin;

    /**
     * @serial
     */
    private JList list;

    /**
     * @serial
     */
    private DefaultListModel listModel;

    /**
     * @serial
     */
    private DefaultListModel dummyModel = new DefaultListModel();

    /**
     * @serial
     */
    private JScrollPane listScrollPane;

    public Browser(ActionListener exiter) throws java.io.IOException {
	this(exiter, new LeaseRenewalManager(), false);
    }

    public Browser(ActionListener exiter, LeaseRenewalManager leaseMgr)
	throws java.io.IOException
    {
        this(exiter, leaseMgr, false);
    }

    public Browser(ActionListener exiter, boolean isAdmin)
	throws java.io.IOException
    {
        this(exiter, null, isAdmin);
    }

    public Browser(ActionListener exiter,
		   LeaseRenewalManager leaseMgr,
		   boolean isAdmin)
	throws java.io.IOException
    {
	this(exiter, leaseMgr, isAdmin, null);
    }

    public Browser(ActionListener exiter,
		   LeaseRenewalManager leaseMgr,
		   boolean isAdmin,
		   String[] groups)
	throws java.io.IOException
    {
	this.exiter = exiter;
	if (leaseMgr == null)
	    leaseMgr = new LeaseRenewalManager();
	this.leaseMgr = leaseMgr;
	this.isAdmin = isAdmin;
	listen = new Listener();
	origGroups = groups;
	disco = new LookupDiscovery(groups);
    }

    public void init() {
	tmpl = new ServiceTemplate(null, new Class[0], new Entry[0]);
	setTitle("Lookup");
	JMenuBar bar = new JMenuBar();
	JMenu file = new JMenu("File");
	JMenuItem allfind = new JMenuItem("Find All");
	allfind.addActionListener(new AllFind());
	file.add(allfind);
	JMenuItem pubfind = new JMenuItem("Find Public");
	pubfind.addActionListener(new PubFind());
	file.add(pubfind);
	JMenuItem multifind = new JMenuItem("Find Groups...");
	multifind.addActionListener(new MultiFind());
	file.add(multifind);
	JMenuItem unifind = new JMenuItem("Find One...");
	unifind.addActionListener(new UniFind());
	file.add(unifind);
	if(! isAdmin){
	    JMenuItem show = new JMenuItem("Show Matches");
	    show.addActionListener(new Show());
	    file.add(show);
	}
	JMenuItem reset = new JMenuItem("Reset");
	reset.addActionListener(new Reset());
	file.add(reset);
	JMenuItem exit = new JMenuItem("Exit");
	exit.addActionListener(exiter);
	file.add(exit);
	bar.add(file);
	registrars = new JMenu("Registrar");
	addNone(registrars);
	bar.add(registrars);
	JMenu options = new JMenu("Options");
	esuper = new JCheckBoxMenuItem("Attribute supertypes", false);
	options.add(esuper);
	ssuper = new JCheckBoxMenuItem("Service supertypes", false);
	options.add(ssuper);
	sclass = new JCheckBoxMenuItem("Service classes", false);
	options.add(sclass);
	bar.add(options);
	JMenu services = new JMenu("Services");
	services.addMenuListener(new Services(services));
	bar.add(services);
	JMenu attrs = new JMenu("Attributes");
	attrs.addMenuListener(new Entries(attrs));
	bar.add(attrs);
	setJMenuBar(bar);

	getContentPane().setLayout(new BorderLayout());
	int textRows = 8;
	if(isAdmin){
	  textRows = 4;
	    JPanel bpanel = new JPanel();
	    bpanel.setLayout(new BorderLayout());

	    TitledBorder border = BorderFactory.createTitledBorder("Matching Services");
	    border.setTitlePosition(TitledBorder.TOP);
	    border.setTitleJustification(TitledBorder.LEFT);
	    bpanel.setBorder(border);
      
	    listModel = new DefaultListModel();
	    list = new JList(listModel);
	    list.setFixedCellHeight(20);
	    list.setCellRenderer(new ServiceItemRenderer());
	    list.addMouseListener(new MouseReceiver(serviceListPopup()));
	    listScrollPane = new JScrollPane(list);
	    bpanel.add(listScrollPane, "Center");
	    getContentPane().add(bpanel, "South");
	}
	text = new JTextArea(genText(false), textRows, 40);
	JScrollPane scroll = new JScrollPane(text);
	getContentPane().add(scroll, "Center");

	validate();
	pack();
	show();
	adder = new LookupListener();
	lnotify = new LeaseNotify();
	disco.addDiscoveryListener(adder);
    }

    protected ServiceListPopup serviceListPopup() {
	return new ServiceListPopup();
    }

    private static String typeName(Class type) {
	String name = type.getName();
	int i = name.lastIndexOf('.');
	if (i >= 0)
	    name = name.substring(i + 1);
	return name;
    }

    private void setText(boolean match) {
	text.setText(genText(match));
    }

    private String genText(boolean match) {
	StringBuffer buf = new StringBuffer();
	if (tmpl.serviceTypes.length > 0) {
	    for (int i = 0; i < tmpl.serviceTypes.length; i++) {
		buf.append(tmpl.serviceTypes[i].getName());
		buf.append("\n");
	    }
	}
	if (tmpl.attributeSetTemplates.length > 0)
	    genEntries(buf, tmpl.attributeSetTemplates, false);
	genMatches(buf, match);
	return buf.toString();
    }

    private void genEntries(StringBuffer buf,
			    Entry[] entries,
			    boolean showNulls)
    {
	for (int i = 0; i < entries.length; i++) {
	    Entry ent = entries[i];
	    if (ent == null) {
		buf.append("null\n");
		continue;
	    }
	    buf.append(typeName(ent.getClass()));
	    buf.append(": ");
	    try {
		Field[] fields = ent.getClass().getFields();
		for (int j = 0; j < fields.length; j++) {
		    if (!valid(fields[j]))
			continue;
		    Object val = fields[j].get(ent);
		    if (val != null || showNulls) {
			buf.append(fields[j].getName());
			buf.append("=");
			buf.append(val);
			buf.append(" ");
		    }
		}
	    } catch (Exception e) {
	    }
	    buf.append("\n");
	}
    }

    private static boolean valid(Field f) {
	return (f.getModifiers() & (Modifier.STATIC|Modifier.FINAL)) == 0;
    }

    private void genMatches(StringBuffer buf, boolean match) {
        if(isAdmin) {
	    list.setModel(dummyModel);	// to keep away from Swing's bug
	    
	    listModel.removeAllElements();
	    list.clearSelection();
	    list.ensureIndexIsVisible(0);
	    list.repaint();
	    list.revalidate();
	    listScrollPane.validate();
	}
	if (lookup == null) {
	    if (disco != null) {
		String[] groups = disco.getGroups();
		if (groups == null) {
		    buf.append("Groups: <all>\n");
		} else {
		    buf.append("Groups:");
		    for (int i = 0; i < groups.length; i++) {
			String group = groups[i];
			if (group.length() == 0)
			    group = "public";
			buf.append(" ");
			buf.append(group);
		    }
		    buf.append("\n");
		}
	    }
	    if (!(registrars.getMenuComponent(0) instanceof
		  JRadioButtonMenuItem))
	    {
		buf.append("No registrars to select");
		return;
	    }
	    int n = registrars.getMenuComponentCount();
	    if (n == 1) {
		buf.append("1 registrar, not selected");
	    } else {
		buf.append(n);
		buf.append(" registrars, none selected");
	    }
	    return;
	}
	ServiceMatches matches;
	try {
	    matches = lookup.lookup(tmpl, (match || isAdmin) ? 1000 : 0);
	} catch (java.rmi.NoSuchObjectException e) {
	    buf.append(e.toString());
	    return;
	} catch (java.rmi.ConnectException e) {
	    buf.append(e.toString());
	    return;
	} catch (Exception e) {
	    e.printStackTrace(System.out);
	    return;
	}

	if(isAdmin){
	    for (int i = 0; i < matches.items.length; i++)
	        listModel.addElement(new ServiceListItem(matches.items[i]));
	    list.setModel(listModel);
	    list.clearSelection();
	    list.ensureIndexIsVisible(0);
	    list.repaint();
	    list.revalidate();
	    listScrollPane.validate();
	}

	if (!match &&
	    tmpl.serviceTypes.length == 0 &&
	    tmpl.attributeSetTemplates.length == 0)
	{
	    buf.append("Total services registered: ");
	    buf.append(matches.totalMatches);
	    return;
	}
	buf.append("\nMatching services: ");
	buf.append(matches.totalMatches);

	if(! isAdmin){
	    if (!match)
	        return;
	    buf.append("\n\n");
	    for (int i = 0; i < matches.items.length; i++) {
	        ServiceItem item = matches.items[i];
		buf.append("Service ID: ");
		buf.append(item.serviceID);
		buf.append("\n");
		buf.append("Service instance: ");
		buf.append(item.service);
		buf.append("\n");
		genEntries(buf, item.attributeSets, true);
		buf.append("\n");
	    }
	}
    }

    private static void addNone(JMenu menu) {
	JMenuItem item = new JMenuItem("(none)");
	item.setEnabled(false);
	menu.add(item);
    }

    private void addOne(ServiceRegistrar registrar) {
	LookupLocator loc;
	try {
	    loc = registrar.getLocator();
	} catch (Exception e) {
	    System.out.println(e);
	    return;
	}
	String host = loc.getHost();
	if (loc.getPort() != Constants.discoveryPort)
	    host += ":" + loc.getPort();
	JRadioButtonMenuItem reg = new JRadioButtonMenuItem(host);
	if (registrar.equals(lookup))
	    reg.setSelected(true);
	reg.addActionListener(new Lookup(registrar));
	synchronized (registrars) {
	    if (!(registrars.getMenuComponent(0)
		  instanceof JRadioButtonMenuItem))
		registrars.removeAll();
	    registrars.add(reg);
	}
	if (lookup == null)
	    setText(false);
    }

    private class Services implements MenuListener {
	private JMenu menu;

	public Services(JMenu menu) {
	    this.menu = menu;
	}

	public void menuSelected(MenuEvent ev) {
	    if (lookup == null) {
		addNone(menu);
		return;
	    }
	    Vector all = new Vector();
	    Class[] types = tmpl.serviceTypes;
	    for (int i = 0; i < types.length; i++) {
		all.addElement(types[i]);
		JCheckBoxMenuItem item =
		    new JCheckBoxMenuItem(types[i].getName(), true);
		    item.addActionListener(new Service(types[i], i));
		    menu.add(item);
	    }
	    try {
		types = lookup.getServiceTypes(tmpl, "");
	    } catch (Exception e) {
		failure(e);
		return;
	    }
	    if (types == null) {
		if (all.isEmpty())
		    addNone(menu);
		return;
	    }
	    for (int i = 0; i < types.length; i++) {
		Class[] stypes;
		if (types[i] == null) {
		    all.addElement(new JMenuItem("null"));
		    continue;
		}
		if (types[i].isInterface() || sclass.getState()) {
		    stypes = new Class[1];
		    stypes[0] = types[i];
		} else {
		    stypes = types[i].getInterfaces();
		}
		for (int j = 0; j < stypes.length; j++) {
		    addType(stypes[j], all);
		}
	    }
	}

	private void addType(Class type, Vector all) {
	    if (all.contains(type))
		return;
	    all.addElement(type);
	    JCheckBoxMenuItem item =
		new JCheckBoxMenuItem(type.getName(), false);
	    item.addActionListener(new Service(type, -1));
	    menu.add(item);
	    if (!ssuper.getState())
		return;
	    if (sclass.getState() && type.getSuperclass() != null)
		addType(type.getSuperclass(), all);
	    Class[] stypes = type.getInterfaces();
	    for (int i = 0; i < stypes.length; i++) {
		addType(stypes[i], all);
	    }
	}

	public void menuDeselected(MenuEvent ev) {
	    menu.removeAll();
	}

	public void menuCanceled(MenuEvent ev) {
	    menu.removeAll();
	}
    }

    private class Show implements ActionListener {

	public void actionPerformed(ActionEvent ev) {
	    setText(true);
	}
    }

    private void resetTmpl() {
	tmpl.serviceTypes = new Class[0];
	tmpl.attributeSetTemplates = new Entry[0];
	update();
    }

    private void reset() {
	ssuper.setState(false);
	esuper.setState(false);
	sclass.setState(false);
	resetTmpl();
    }

    private class Reset implements ActionListener {

	public void actionPerformed(ActionEvent ev) {
	    reset();
	}
    }

    private class Service implements ActionListener {
	private Class type;
	private int index;

	public Service(Class type, int index) {
	    this.type = type;
	    this.index = index;
	}

	public void actionPerformed(ActionEvent ev) {
	    int z = tmpl.serviceTypes.length;
	    Class[] newTypes;
	    if (index < 0) {
		newTypes = new Class[z + 1];
		System.arraycopy(tmpl.serviceTypes, 0, newTypes, 0, z);
		newTypes[z] = type;
	    } else {
		newTypes = new Class[z - 1];
		System.arraycopy(tmpl.serviceTypes, 0,
				 newTypes, 0, index);
		System.arraycopy(tmpl.serviceTypes, index + 1,
				 newTypes, index, z - index - 1);
	    }
	    tmpl.serviceTypes = newTypes;
	    update();
	}
    }

    private class Entries implements MenuListener {
	private JMenu menu;

	public Entries(JMenu menu) {
	    this.menu = menu;
	}

	public void menuSelected(MenuEvent ev) {
	    if (lookup == null) {
		addNone(menu);
		return;
	    }
	    Entry[] attrs = tmpl.attributeSetTemplates;
	    for (int i = 0; i < attrs.length; i++) {
		Class type = attrs[i].getClass();
		JMenu item = new JMenu(typeName(type));
		item.addMenuListener(new Fields(item, i));
		menu.add(item);
	    }
	    Class[] types;
	    try {
		types = lookup.getEntryClasses(tmpl);
	    } catch (Exception e) {
		failure(e);
		return;
	    }
	    if (types == null) {
		if (attrs.length == 0)
		    addNone(menu);
		return;
	    }
	    Vector all = new Vector();
	    for (int i = 0; i < types.length; i++) {
		if (types[i] == null)
		    menu.add(new JMenuItem("null"));
		else
		    addType(types[i], all);
	    }
	}

	private void addType(Class type, Vector all) {
	    if (all.contains(type))
		return;
	    all.addElement(type);
	    JCheckBoxMenuItem item =
		new JCheckBoxMenuItem(typeName(type), false);
	    item.addActionListener(new AttrSet(type));
	    menu.add(item);
	    if (esuper.getState() &&
		Entry.class.isAssignableFrom(type.getSuperclass()))
		addType(type.getSuperclass(), all);
	}

	public void menuDeselected(MenuEvent ev) {
	    menu.removeAll();
	}

	public void menuCanceled(MenuEvent ev) {
	    menu.removeAll();
	}
    }

    private class AttrSet implements ActionListener {
	private Class type;

	public AttrSet(Class type) {
	    this.type = type;
	}

	public void actionPerformed(ActionEvent ev) {
	    Entry ent;
	    try {
		ent = (Entry)type.newInstance();
	    } catch (Exception e) {
		e.printStackTrace(System.out);
		return;
	    }
	    int z = tmpl.attributeSetTemplates.length;
	    Entry[] newSets = new Entry[z + 1];
	    System.arraycopy(tmpl.attributeSetTemplates, 0, newSets, 0, z);
	    newSets[z] = ent;
	    tmpl.attributeSetTemplates = newSets;
	    update();
	}
    }

    private class Fields implements MenuListener {
	private JMenu menu;
	private int index;

	public Fields(JMenu menu, int index) {
	    this.menu = menu;
	    this.index = index;
	}

	public void menuSelected(MenuEvent ev) {
	    JRadioButtonMenuItem match = new JRadioButtonMenuItem("(match)");
	    match.setSelected(true);
	    match.addActionListener(new Unmatch(index));
	    menu.add(match);
	    Entry ent = tmpl.attributeSetTemplates[index];
	    Field[] fields = ent.getClass().getFields();
	    for (int i = 0; i < fields.length; i++) {
		Field field = fields[i];
		if (!valid(field))
		    continue;
		try {
		    if (field.get(ent) != null) {
			JCheckBoxMenuItem item =
			    new JCheckBoxMenuItem(field.getName(), true);
			item.addActionListener(new Value(index, field, null));
			menu.add(item);
		    } else {
			JMenu item = new JMenu(field.getName());
			item.addMenuListener(new Values(item, index, field));
			menu.add(item);
		    }
		} catch (Exception e) {
		}
	    }
	}

	public void menuDeselected(MenuEvent ev) {
	    menu.removeAll();
	}

	public void menuCanceled(MenuEvent ev) {
	    menu.removeAll();
	}
    }

    private class Unmatch implements ActionListener {
	private int index;

	public Unmatch(int index) {
	    this.index = index;
	}

	public void actionPerformed(ActionEvent ev) {
	    int z = tmpl.attributeSetTemplates.length;
	    Entry[] newSets = new Entry[z - 1];
	    System.arraycopy(tmpl.attributeSetTemplates, 0,
			     newSets, 0, index);
	    System.arraycopy(tmpl.attributeSetTemplates, index + 1,
			     newSets, index, z - index - 1);
	    tmpl.attributeSetTemplates = newSets;
	    update();
	}
    }

    private class Values implements MenuListener {
	private JMenu menu;
	private int index;
	private Field field;

	public Values(JMenu menu, int index, Field field) {
	    this.menu = menu;
	    this.index = index;
	    this.field = field;
	}

	public void menuSelected(MenuEvent ev) {
	    Object[] values;
	    try {
		values = lookup.getFieldValues(tmpl, index, field.getName());
	    } catch (Exception e) {
		failure(e);
		return;
	    }
	    if (values == null) {
		addNone(menu);
		return;
	    }
	    for (int i = 0; i < values.length; i++) {
		JMenuItem item = new JMenuItem(values[i].toString());
		item.addActionListener(new Value(index, field, values[i]));
		menu.add(item);
	    }
	}

	public void menuDeselected(MenuEvent ev) {
	    menu.removeAll();
	}

	public void menuCanceled(MenuEvent ev) {
	    menu.removeAll();
	}
    }

    private class Value implements ActionListener {
	private int index;
	private Field field;
	private Object value;

	public Value(int index, Field field, Object value) {
	    this.index = index;
	    this.field = field;
	    this.value = value;
	}

	public void actionPerformed(ActionEvent ev) {
	    try {
		field.set(tmpl.attributeSetTemplates[index], value);
	    } catch (Exception e) {
		e.printStackTrace(System.out);
	    }
	    update();
	}
    }

    private class Listener extends java.rmi.server.UnicastRemoteObject
		implements RemoteEventListener
    {
	public Listener() throws java.rmi.RemoteException {
	    super();
	}

	public void notify(RemoteEvent ev) {
	    setText(false);
	}
    }

    private class LookupListener implements DiscoveryListener {

	public void discovered(DiscoveryEvent e) {
	    ServiceRegistrar[] newregs = e.getRegistrars();
	    for (int i = 0; i < newregs.length; i++) {
		addOne(newregs[i]);
	    }
	}

	public void discarded(DiscoveryEvent e) {
	}
    }

    private void setGroups(String[] groups) {
	lookup = null;
	synchronized (registrars) {
	    registrars.removeAll();
	    addNone(registrars);
	}
	resetTmpl();
	try {
	    if (disco == null) {
		disco = new LookupDiscovery(groups);
	    } else {
		disco.removeDiscoveryListener(adder);
		disco.setGroups(groups);
	    }
	    disco.addDiscoveryListener(adder);
	} catch (Exception e) {
	    e.printStackTrace(System.out);
	}
	update();
    }

    private class AllFind implements ActionListener {

	public void actionPerformed(ActionEvent ev) {
	    setGroups(null);
	}
    }

    private class PubFind implements ActionListener {

	public void actionPerformed(ActionEvent ev) {
	    setGroups(new String[]{""});
	}
    }

    private class MultiFind implements ActionListener {

	public void actionPerformed(ActionEvent ev) {
	    String names = JOptionPane.showInputDialog(Browser.this,
						       "Enter group names");
	    if (names == null)
		return;
	    setGroups(parseGroups(names));
	}
    }

    private class UniFind implements ActionListener {

	public void actionPerformed(ActionEvent ev) {
	    String host = JOptionPane.showInputDialog(Browser.this,
						      "Enter host[:port]");
	    if (host == null)
		return;
	    try {
		lookup = new LookupLocator("jini://" + host).getRegistrar();
	    } catch (Exception e) {
		JOptionPane.showMessageDialog(Browser.this, e, "Error",
					      JOptionPane.ERROR_MESSAGE);
		return;
	    }
	    if (disco != null) {
		disco.terminate();
		disco = null;
	    }
	    synchronized (registrars) {
		registrars.removeAll();
		addNone(registrars);
	    }
	    addOne(lookup);
	    resetTmpl();
	    update();
	}
    }

    private class Lookup implements ActionListener {
	private ServiceRegistrar registrar;

	public Lookup(ServiceRegistrar registrar) {
	    this.registrar = registrar;
	}

	public void actionPerformed(ActionEvent ev) {
	    if (lookup == registrar)
		lookup = null;
	    else
		lookup = registrar;
	    for (int i = 0; i < registrars.getMenuComponentCount(); i++) {
		JMenuItem item = (JMenuItem)registrars.getMenuComponent(i);
		if (item != ev.getSource())
		    item.setSelected(false);
	    }
	    resetTmpl();
	}
    }

    private static class LeaseNotify implements LeaseListener {
	public void notify(LeaseRenewalEvent ev) {
	    if (ev.getException() != null)
		ev.getException().printStackTrace(System.out);
	    else
		System.out.println("event lease failed to renew");
	}
    }

    private static String[] parseGroups(String names) {
	StringTokenizer st = new StringTokenizer(names, " \t\n\r\f,");
	String[] groups = new String[st.countTokens()];
	for (int i = 0; st.hasMoreTokens(); i++) {
	    groups[i] = st.nextToken();
	    if (groups[i].equalsIgnoreCase("public"))
		groups[i] = "";
	}
	return groups;
    }

    private void update() {
	setText(false);
	if (elease != null) {
	    try {
		leaseMgr.cancel(elease);
	    } catch (Exception e) {
		e.printStackTrace(System.out);
	    }
	    elease = null;
	}
	if (lookup == null)
	    return;
	try {
	    elease = lookup.notify(tmpl,
				   ServiceRegistrar.TRANSITION_MATCH_NOMATCH |
				   ServiceRegistrar.TRANSITION_NOMATCH_MATCH |
				   ServiceRegistrar.TRANSITION_MATCH_MATCH,
				   listen, null, Lease.ANY).getLease();
	    leaseMgr.renewUntil(elease, Lease.ANY, lnotify);
	} catch (Exception e) {
	    failure(e);
	}
    }

    private void failure(Exception e) {
	if (e instanceof java.rmi.ConnectException ||
	    e instanceof java.rmi.NoSuchObjectException)
	    System.out.println(e);
	else
	    e.printStackTrace(System.out);
	if (elease != null) {
	    try {
		leaseMgr.cancel(elease);
	    } catch (Exception ex) {
	    }
	    elease = null;
	}
	if (disco != null) {
	    disco.discard(lookup);
	    setGroups(disco.getGroups());
	} else {
	    setGroups(origGroups);
	}
    }

    // admin mode
    private static boolean isAdministrable(ServiceItem item) {
        return (item.service instanceof Administrable) ? true : false;
    }

    private static boolean isJoinAdministrable(ServiceItem item) throws RemoteException {
        if(! isAdministrable(item))
	    return false;

	return (((Administrable) item.service).getAdmin() instanceof JoinAdmin) ? true : false;
    }

    class ServiceItemRenderer implements ListCellRenderer {
        private JLabel label;

        public ServiceItemRenderer() {
	    label = new JLabel();
	    label.setOpaque(true);
	}
    
        public Component getListCellRendererComponent(JList list,
						      Object value,
						      int index,
						      boolean isSelected,
						      boolean cellHasFocus){
	    ServiceListItem item = null;
	    if(value instanceof ServiceListItem)
	        item = (ServiceListItem) value;

	    label.setFont(list.getFont());

	    if(isSelected){
	        label.setBackground(list.getSelectionBackground());
		label.setForeground(list.getSelectionForeground());
	    } else {
	        label.setBackground(list.getBackground());
		label.setForeground(list.getForeground());
	    }
	    if(item != null){
	        label.setIcon(item.getIcon());	// accessible check has done in this method
	        label.setText(item.getTitle());
	    } else
	        label.setText(value.toString());

	    return label;
	}
    }

    private static Icon[] icons = new Icon[3];
    static {
        icons[0] = MetalIcons.getBlueFolderIcon();	// Administrable Service, Controllable Attribute
	icons[1] = MetalIcons.getGrayFolderIcon();	// Non-administrable Service
	icons[2] = MetalIcons.getUnusableFolderIcon();	// "Connection Refused" Service
    }

    public class ServiceListItem {
        private ServiceItem item;
        private boolean isAccessible = true;

        public ServiceListItem(ServiceItem item) {
	    this.item = item;
	    if(item.service == null) {
	        System.out.println("[ServiceListItem#<init>]: item.service == null");
	        isAccessible = false;
	    }
	}

        public String getTitle() {
	    if(item.service == null)
	        return "Unknown service";

	    HashSet set = new HashSet();
	    Class[] infs = item.service.getClass().getInterfaces();
	    for(int j = 0; j < infs.length; j++)
	        set.add(infs[j].getName());

	    // remove known interfaces
	    set.remove("net.jini.admin.Administrable");
	    set.remove("java.rmi.Remote");
	    set.remove("java.io.Serializable");

	    String title = null;
	    if(set.size() == 1) {
	        Iterator iter = set.iterator();
	        title = (String) iter.next();
	    } else
	        title = item.service.getClass().getName();

	    if(! isAccessible)
	        title += " (Stale service)";
	    return title;
	}
      
        public boolean isAccessible() {
	    return isAccessible;
	}

        public ServiceItem getServiceItem() {
	    return item;
	}

        public Entry[] getAttributes() {
	    return item.attributeSets;
	}

        private boolean isControllable(int i){
	  return (item.attributeSets[i] instanceof ServiceControlled) ? false : true;
	}

        public Icon getIcon() {
	    try {
	        if(! isAccessible)
		    return icons[2];
	        else if(isJoinAdministrable(item))
		    return icons[0];
		else
		    return icons[1];
	    } catch(RemoteException e){
	        isAccessible = false;
		return icons[2];
	    }
	}

        public String toString() {
	    return isAccessible ? item.service.getClass().getName() : "Unknown service";
	}
    }

    class MouseReceiver extends MouseAdapter {
        private ServiceListPopup popup;

        public MouseReceiver(ServiceListPopup popup) {
	  this.popup = popup;
	}

        public void mouseClicked(MouseEvent ev) {
	    if(ev.getClickCount() >= 2){
	        ServiceItem item = getTargetItem(ev);
		if(item == null)
		    return;

		try {
		    if(getTargetListItem(ev).isAccessible())
		        if(isJoinAdministrable(item))
			    new ServiceEditor(item, lookup, Browser.this).setVisible(true);
			else
			    new ServiceBrowser(item, lookup, Browser.this).setVisible(true);
		} catch(RemoteException e){
		    // Could not access the service
		}
	    }
	}

        public void mouseReleased(MouseEvent ev) {
	    if(ev.isPopupTrigger() && (getTargetListItem(ev) != null)) {
	        popup.setServiceItem(getTargetListItem(ev));
	        popup.show(ev.getComponent(), ev.getX(), ev.getY());
	    }
	}

        public void mousePressed(MouseEvent ev) {
	    if(ev.isPopupTrigger() && (getTargetListItem(ev) != null)) {
	        popup.setServiceItem(getTargetListItem(ev));
	        popup.show(ev.getComponent(), ev.getX(), ev.getY());
	    }
	}

        private ServiceListItem getTargetListItem(MouseEvent ev) {
	    int index = list.locationToIndex(ev.getPoint());
	    if(index >= 0)
	        return (ServiceListItem) listModel.getElementAt(index);
	    else
	        return null;
	}

        private ServiceItem getTargetItem(MouseEvent ev) {
	    //return ((ServiceListItem) listModel.getElementAt(list.locationToIndex(ev.getPoint()))).getServiceItem();
	    ServiceListItem listItem = getTargetListItem(ev);
	    if(listItem != null)
	        return listItem.getServiceItem();
	    else
	        return null;
	}
    }

    public class ServiceListPopup extends JPopupMenu implements ActionListener, PopupMenuListener {
        protected JMenuItem infoItem;
        protected JMenuItem browseItem;
        protected JMenuItem adminItem;
        protected ServiceItem item;

        public ServiceListPopup() {
	    super();

	    infoItem = new JMenuItem("Show Info");
	    infoItem.addActionListener(this);
	    infoItem.setActionCommand("showInfo");
	    add(infoItem);

	    browseItem = new JMenuItem("Browse Service");
	    browseItem.addActionListener(this);
	    browseItem.setActionCommand("browseService");
	    add(browseItem);

	    adminItem = new JMenuItem("Admin Service");
	    adminItem.addActionListener(this);
	    adminItem.setActionCommand("adminService");
	    add(adminItem);

	    addPopupMenuListener(this);
	    setOpaque(true);
	    setLightWeightPopupEnabled(true);
	}

        public void setServiceItem(ServiceListItem listItem) {
	    item = listItem.getServiceItem();
	    try {
	        if(! listItem.isAccessible()){
		    infoItem.setEnabled(false);
		    browseItem.setEnabled(false);
		    adminItem.setEnabled(false);
		} else {
		    infoItem.setEnabled(true);
		    browseItem.setEnabled(true);

		    if(isJoinAdministrable(item))
		        adminItem.setEnabled(true);
		    else
		        adminItem.setEnabled(false);
		}
	    } catch(RemoteException e){
	        infoItem.setEnabled(false);
	        browseItem.setEnabled(false);
	        adminItem.setEnabled(false);
	    }
	}

        public void actionPerformed(ActionEvent ev) {
	    String command = ev.getActionCommand();

	    //System.out.println("[popup]: " + command);
	    if(command.equals("showInfo")){
	        Class[] infs = item.service.getClass().getInterfaces();
		String[] msg = new String[3 + infs.length];
		msg[0] = "ServiceID: " + item.serviceID;
		msg[1] = "Service Instance: " + item.service.getClass().getName();
		if(infs.length == 1)
		    msg[2] = "Implemented Interface:";
		else
		    msg[2] = "Implemented Interfaces:";
		for(int i = 0; i < infs.length; i++)
		    msg[3 + i] = infs[i].getName();

		JOptionPane.showMessageDialog(Browser.this,
					      msg,
					      "ServiceItem Information",
					      JOptionPane.INFORMATION_MESSAGE);
	    } else if(command.equals("browseService")){
	        new ServiceBrowser(item, lookup, Browser.this).setVisible(true);
	    } else if(command.equals("adminService")){
	        new ServiceEditor(item, lookup, Browser.this).setVisible(true);
	    }
	}

        public void popupMenuWillBecomeVisible(PopupMenuEvent ev) {
	    //System.out.println("[popup will become visible]");
	}

        public void popupMenuWillBecomeInvisible(PopupMenuEvent ev) {
	    //System.out.println("[popup will become invisible]");
	}

        public void popupMenuCanceled(PopupMenuEvent ev) {
	    //System.out.println("[popup canceled]");
	}
    }

    private static class Exit implements ActionListener {

	public void actionPerformed(ActionEvent ev) {
	    System.exit(0);
	}
    }

    public static void main (String[] args, Class bclass) {
        boolean isAdmin = false;
	String[] groups = null;
	for (int i = 0; i < args.length; i++) {
	    if (args[i].equals("-admin")) {
	        isAdmin = true;
	    } else if (args[i].equals("-help")) {
		System.out.println("Browser [-admin] [group,...]");
		return;
	    } else {
		groups = parseGroups(args[i]);
	    }
	}

	System.setSecurityManager(new RMISecurityManager());
	try {
	    Constructor cons =
		bclass.getConstructor(new Class[]{ActionListener.class,
						  LeaseRenewalManager.class,
						  boolean.class,
						  String[].class});
	    Browser browser =
		(Browser)cons.newInstance(new Object[]{new Exit(), null,
						       new Boolean(isAdmin),
						       groups});
	    browser.init();
	} catch (Exception e) {
	    e.printStackTrace(System.out);
	}
    }

    public static void main (String[] args) {
	main(args, Browser.class);
    }
}
