/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.example.genericWorker;

import net.jini.core.entry.*;
import com.sun.jini.collection.WeakTable;

/**
 * Parent class for generic tasks.  A worker thread will repeatedly
 * take a <code>TaskEntry</code> object from the space, invoke its
 * <code>execute</code> method, and write the returned
 * <code>Entry</code> into the space with the lease time returned by
 * <code>responseLeaseTime</code>.  If <code>execute</code> returns
 * <code>null</code>, no value will be written -- it is assumed that a
 * <code>null</code> return value means that the task requires no
 * response in the space.
 *
 * @see GenericWorker
 */

public class TaskEntry implements Entry {
    /**
     * Execute whatever task the entry requires.  If an entry is
     * returned, it is written back into the space.  If
     * <code>null</code> is returned, nothing is written into the
     * space.  If a subclass of <code>TaskEntry</code> has not
     * overridden this method, a <code>RuntimeException</code> will be
     * generated when <code>execute</code> is invoked.
     * <p>
     * The <code>WeakTable</code> entry can be used to store cached
     * state between one entry's use of <code>execute</code> and
     * another of a related entry.  Weak references are stored, so
     * <code>execute</code> must be able to execute properly if the
     * holder doesn't have a value for the key.
     */
    public Entry execute(WeakTable cache) {
	throw new RuntimeException("TaskEntry.execute() not implemented");
    }

    /**
     * The lease time that will be requested for any response from
     * <code>execute</code>.  The default is for ten minutes.
     */
    public long responseLeaseTime() {
	return 10 * 60 * 60 * 1000;	// 10 minutes
    }

    /**
     * Turn on verbose messages.  What that means is up to each task
     * type -- by default this method does nothing.
     */
    public void verbose(boolean showMessages) {
    }
}
