/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */


/*
 * The contents of this file are subject to the Sun Community Source
 * License, Jini Technology Core Platform, v.1.0 (the "License"); you
 * may not use this file except in compliance with the License. You may
 * obtain a copy of the License at http://www.sun.com/jini.  Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Jini Technology Core Platform code, version v.1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1998 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * Contributor(s): Eric Freeman
 */


package com.sun.jini.example.raytrace;

import com.sun.jini.example.genericWorker.*;
import net.jini.core.entry.*;
import com.sun.jini.collection.WeakTable;

import java.awt.*;
import java.awt.image.*;
import java.rmi.*;
import java.rmi.registry.*;
import java.rmi.server.*;
import java.net.*;


//
// Floating point color.
// The RGB values are normalized to 1.0
//

class color
{
    // from RGB to normalized color
    final double normFactor = 1.0 / 255.0;

    double red;
    double green;
    double blue;

    public color ()
    {
        Init (0, 0, 0);
    }

    public color (double red, double green, double blue)
    {
        Init (red, green, blue);
    }

    public color (color c)
    {
        Init (c);
    }

    public color (int rgb)
    {

        Init (((rgb >> 16) & 0xff) * normFactor,
              ((rgb >> 8)  & 0xff) * normFactor,
               (rgb & 0xff) * normFactor);
    }

    public final void Init (color c)
    {
        Init (c.red, c.green, c.blue);
    }

    public final void Init (double red, double green, double blue)
    {
        this.red = red;
        this.green = green;
        this.blue = blue;
    }

    public final void AttenuateBy (double a)
    {
        red *= a;
        green *= a;
        blue *= a;
    }

    public final void AttenuateBy (color c)
    {
        red *= c.red;
        green *= c.green;
        blue *= c.blue;
    }

    public final void CombineWith (color c)
    {
        red += c.red;
        green += c.green;
        blue += c.blue;
    }

    public final int Rgb ()
    {
        int r = (int)Math.ceil (red * 255);
        int g = (int)Math.ceil (green * 255);
        int b = (int)Math.ceil (blue * 255);

        return (255 << 24) |
               (r > 255? 255: r) << 16 |
               (g > 255? 255: g) << 8 |
               (b > 255? 255: b);
    }
}

//
// Optical properties of the object
//
class OpticalProp
{
    color  objColor;
    double refractiveness;
    double transparency;
    double reflectiveness;
    double diffusion;
    boolean isLuminous;


    OpticalProp (color c,
                 double refractiveness,
                 double transparency,
                 double reflectiveness,
                 double diffusion)
    {
        this.objColor = new color (c);
        this.refractiveness = refractiveness;
        this.transparency = transparency;
        this.reflectiveness = reflectiveness;
        this.diffusion = diffusion;
        this.isLuminous = false;
    }

    OpticalProp (OpticalProp prop)
    {
        this (prop.objColor,
              prop.refractiveness,
              prop.transparency,
              prop.reflectiveness,
              prop.diffusion);
    }

    public final color GetColor ()
    {
        return objColor;
    }

    public final void   MakeLuminous () { isLuminous = true; }
    public final boolean IsLuminous () { return isLuminous; }
    public final double Transparency () { return transparency; }
    public final double Refractiveness () { return refractiveness; }
    public final double Reflectiveness () { return reflectiveness; }
    public final double Diffusion () { return diffusion; }
}

//
// A three dimensional double precision point
//
class Point3
{
    protected double x;
    protected double y;
    protected double z;

    public Point3 (double x, double y, double z)
    {
        this.x = x;
        this.y = y;
        this.z = z;
    }

    public Point3 (Point3 p)
    {
        this (p.x, p.y, p.z);
    }

    final public double X () { return x; }
    final public double Y () { return y; }
    final public double Z () { return z; }

    // you can add a vector to a point
    final public void Add (Vector3 v)
    {
        x += v.X ();
        y += v.Y ();
        z += v.Z ();
    }
}

//
// A three dimensional double precision vector
//

class Vector3 extends Point3
{
    protected final double EPSILON = 0.001;

    // Create vector from two points:
    // the endpoint and the origin
    Vector3 (Point3 endpoint, Point3 origin)
    {
        super  (endpoint.X () - origin.X (),
                endpoint.Y () - origin.Y (),
                endpoint.Z () - origin.Z ());
    }

    Vector3 (double x, double y, double z)
    {
        super (x, y, z);
    }

    Vector3 (Vector3 v)
    {
        super (v.x, v.y, v.z);
    }

    boolean NonZero ()
    {
        return x != 0 | y != 0 | z != 0;
    }

    public final double Len ()
    {
        return Math.sqrt (x * x + y * y + z * z);
    }

    public final void ScaleBy (double a)
    {
        x *= a;
        y *= a;
        z *= a;
    }

    public final double dotProduct (Vector3 v)
    {
        return x * v.x + y * v.y + z * v.z;
    }

    public final void Normalize ()
    {
        double len = Len ();
        if (len > EPSILON)
        {
            double invLen = 1/len;
            ScaleBy (invLen);
        }
        else
        {
            x = 0;
            y = 0;
            z = 0;
        }
    }
}

//
// VisibleObject: the base class for all visible object
//
abstract class VisibleObject
{
    protected OpticalProp opticalProp;

    VisibleObject (OpticalProp prop)
    {
        this.opticalProp = prop;
    }
    public final boolean IsLuminous () { return opticalProp.IsLuminous (); }
    public final OpticalProp OpticalProperty () { return opticalProp; }
    public final color GetColor () { return opticalProp.GetColor (); }

    public abstract Vector3 Normal (Point3 intersect);
    public abstract boolean Intersect (Ray  ray);
    public color GetPixel (Point3 intersect)
    {
        return opticalProp.GetColor ();
    }

    public abstract Shader CreateShader (Ray ray);
}

//
// Plane tiled with a picture (texture mapping)
//
class Plane extends VisibleObject
{
    // geometry
    Point3    center;
    Vector3 normal;
    // the tile
    int pixels [];
    int width;
    int height;

    double  cos_inv;

    Plane (Point3 center, Vector3 perpendicular, OpticalProp prop,
           int pixels [], int width, int height)
    {
        super (prop);
        this.pixels = pixels;
        this.width = width;
        this.height = height;
        this.center = center;
        this.normal = new Vector3 (perpendicular);
        this.normal.Normalize ();
        this.cos_inv = 1.0 / this.normal.dotProduct (new Vector3 (0, 0, 1));
    }

    public color GetPixel (Point3 p)
    {
        // texture mapping
        // convert to local plane coordinates
        int x = (int)Math.ceil (p.X () * cos_inv);
        int y = (int)Math.ceil (p.Y ());
        // divide them modulo tile size
        x %= width;
        if (x < 0)
            x += width;
        y %= height;
        if (y < 0)
            y += height;
        // find the corresponding pixel in the tile
        int rgb = pixels [y * width + x];
        return new color (rgb);
    }

    public Vector3 Normal (Point3 intersect)
    {
        return normal;
    }

    public boolean Intersect (Ray  ray)
    {
        // ( org + t * dir - center ) * normal = 0

        // cosine of incidence angle
        double inc = ray.Direction().dotProduct (normal);
        if (inc >= .001)
            return false;

        Vector3 oc = new Vector3 (center, ray.Origin ());

        double t = oc.dotProduct (normal);
        t /= inc;
        if (t > .001)
        {
            ray.Record (t, this);
            return true;
        }
        return false;
    }

    public Shader CreateShader (Ray ray)
    {
        return new PlaneShader (ray, this);
    }
}

//
// A sphere
//
class Sphere extends VisibleObject
{
    Point3  center;
    double  radius;

    double  r2; // radius square

    Sphere (Point3 center, double radius, OpticalProp pr)
    {
        super (pr);
        this.center = center;
        this.radius = radius;
        this.r2 = radius * radius;
    }

    public final Point3 Center() { return center; }

    public Vector3 Normal (Point3 intersect)
    {
        Vector3 result = new Vector3 (intersect, center);
        result.Normalize();
        return result;
    }

    public boolean Intersect (Ray ray)
    {
        // ray = _origin + t * _direction;
        // Sphere: (v - _center )^2 = r * r;
        // intersection:
        // ( _origin - _center + t * _direction ) ^2 = r^2;
        // (_o-_c)^2 + 2 * t * ( (_o-_c) . _direction ) + t^2  = r^2;
        // _direction is normalized to 1
        // disc = 4 * ((_o-_c) . _direction ) - 4 * (_o-_c)^2
        // t = (- 2 * ((_o-_c) . _direction) + sqrt ( disc ) ) / 2

        // origin to center vector
        Vector3 oc = new Vector3 (center, ray.Origin());

        // square distance
        double oc2 = oc.dotProduct (oc);
        double dist2 = oc2 - r2;

        if ( dist2 <= .001 && dist2 >= -0.001 ) // on the surface
        {
            return false;
        }
        // closest approach along the ray
        double tca = oc.dotProduct (ray.Direction());

        if ( dist2 > 0.001 && tca < 0.0 ) // outside and pointing away
            return false;

        // half cord squared
        double hc2 = r2 - oc2 + tca * tca;

        if (hc2 < 0.0) // no real solution
            return false;

        if (dist2 > 0.001) // outside
            ray.Record (tca - Math.sqrt (hc2), this);
        else
            ray.Record (tca + Math.sqrt (hc2), this);

        return true;
    }

    public Shader CreateShader (Ray ray)
    {
        return new PhongShader (ray, this);
    }
}

//
// Generic shader
//
abstract class Shader
{
    Ray            incidentRay;
    OpticalProp    prop;
    int            depth;

    Shader (Ray incident, VisibleObject obj)
    {
        this.prop = obj.OpticalProperty ();
        this.incidentRay = incident;
        this.depth = incident.Depth ();
    }

    public abstract void GetColor (color color);
}

//
// Traditional Phong shading algorithm
//
class PhongShader extends Shader
{
    // Secondary rays
    Ray           reflectedRay;
    RefractedRay  refractedRay;
    ShadowRay     shadowRay;

    // geometry
    Point3  intersect;
    Vector3 normal;
    double  cosine;
    double  light_cos;

    PhongShader (Ray incident, VisibleObject obj)
    {
        super (incident, obj);

        this.intersect = incident.Intersect ();
        this.normal = obj.Normal (intersect);
        this.cosine = normal.dotProduct (incident.Direction ());
        // vector pointing towards light
        Vector3 light = new Vector3 (Painter.TheScene.LightPoint (), intersect);
        light.Normalize ();
        light_cos = light.dotProduct (normal);

        // Shadow ray towards the source of light
        if (prop.Diffusion () > 0.0 && light_cos > 0.0)
        {
            shadowRay = new ShadowRay (intersect, light, depth + 1);
        }

        // Reflection
        if (prop.Reflectiveness () > 0.0)
        {
            Vector3 ref  = Reflect ();
            reflectedRay = new Ray (intersect, ref, depth + 1);
        }

        // Refraction
        if (prop.Transparency () > 0.0)
        {
            Vector3 ref = Refract ();
            if (ref.NonZero())
                refractedRay = new RefractedRay (intersect, ref, depth+1, prop.Refractiveness ());
        }
    }

    public void GetColor (color final_color)
    {
        // Illumination model
        // I = Ia + kd ( N . L ) + ks S + kt T

        // Diffused light
        color diffuse = new color ();
        if (shadowRay != null)
        {
            shadowRay.Hit (diffuse);
            diffuse.AttenuateBy (shadowRay.Attenuation ());
            diffuse.AttenuateBy (light_cos);
        }

        diffuse.AttenuateBy (prop.Diffusion ());
        diffuse.CombineWith (Painter.TheScene.Ambient ());
        diffuse.AttenuateBy (prop.GetColor ());
        final_color.Init (diffuse);

        // Reflected light

        color specular = new color ();
        if (reflectedRay != null)
            reflectedRay.Hit (specular);
        specular.AttenuateBy (prop.Reflectiveness ());
        final_color.CombineWith (specular);

        // Refracted light

        color refr = new color ();
        if ( refractedRay != null )
            refractedRay.Hit (refr);
        refr.AttenuateBy (prop.Transparency ());
        final_color.CombineWith (refr);
    }

    Vector3 Reflect ()
    {
        // Angle of reflection = angle of incidence
           // refl = incid - 2 * ( normal . incid ) normal
        Vector3 result = new Vector3 (normal);
        result.ScaleBy (-2 * cosine);
        result.Add (incidentRay.Direction());
        return result;
    }

    Vector3 Refract ()
    {
        // Snell's law:   ki * sin ( angle(inc, n) ) = kr * sin ( angle(r * n) )
        // where ki and kr are refraction coefficients of the two media

        // all vectors normalized, sin calculated using cross product
        //    ki * cross( inc, n ) =  kr * cross ( r, n )
        //    r = a n + b inc
        //    cross ( ( a n + b inc ), n ) = b cross ( inc, n )
        //    (ki - b * kr ) cross ( inc, n ) = 0
        //    b = ki/kr
        //      r normalized
        //    r . r = 1
        //    ( a n + b inc )^2 = 1
        //    a^2 + 2 a b ( n . inc) + b^2 = 1
        //    a^2 + 2 b cos a + b^2 - 1 = 0
        //    disc = 4 (b^2 cos^2 - b^2 + 1)
        //

        // if cosine negative we are entering the object (from vacuum)
        // ki = 1, kr = refr, b = 1/refr
        // else ki = refr, kr = 1, b = refr

        double refr = 0.0;
        if (cosine >= 0.0)
            refr = prop.Refractiveness ();
        else if (prop.Refractiveness () > 0.0)
            refr = 1. / prop.Refractiveness ();

        // discriminant squared
        // if negative, the equation has no solution
        double disc_2 = refr * refr * (cosine * cosine - 1) + 1;
        if (disc_2 > 0.0)
        {
            double discr = Math.sqrt (disc_2);
            // a = - b cos - discr
            double alpha =  - refr * cosine;
            if (cosine < 0.0)
                alpha -= discr;
            else
                alpha += discr;

            // r = a n + b inc
            Vector3 result = new Vector3 (incidentRay.Direction ());
            result.ScaleBy (refr);

            Vector3 n = new Vector3 (normal);
            n.ScaleBy (alpha);
            result.Add (n);
            return result;
        }
        else // total internal reflection (fale)
            return new Vector3 (0, 0, 0);
    }

}

//
// Maps texture onto the plane
//
class PlaneShader extends PhongShader
{
    color  pixel;    // pixel at the point of incidence

    PlaneShader (Ray incident, VisibleObject obj)
    {
        super (incident, obj);
        this.pixel = obj.GetPixel (intersect);
    }

    public void GetColor (color c)
    {
        // first calculate the Phong shade
        // then attenuate it by the texture

        this.LocalGetColor (c);
        c.AttenuateBy (pixel);
    }

    public void LocalGetColor (color final_color)
    {
        // Illumination model
        // I = Ia + kd ( N . L ) + ks S + kt T

        // Diffused light
        color diffuse = new color ();
        if (shadowRay != null)
        {
            shadowRay.Hit (diffuse);
            diffuse.AttenuateBy (shadowRay.Attenuation ());
            diffuse.AttenuateBy (light_cos);
        }

        diffuse.AttenuateBy (prop.Diffusion ());
        diffuse.CombineWith (Painter.TheScene.Ambient ());
        diffuse.AttenuateBy (prop.GetColor ());
        final_color.Init (diffuse);

        // Reflected light

        color specular = new color ();
        if (reflectedRay != null)
            reflectedRay.Hit (specular);
        specular.AttenuateBy (prop.Reflectiveness ());
        final_color.CombineWith (specular);

        // Refracted light

        color refr = new color ();
        if ( refractedRay != null )
            refractedRay.Hit (refr);
        refr.AttenuateBy (prop.Transparency ());
        final_color.CombineWith (refr);
    }
};

//
// The light ray
//
class Ray
{
    final double HUGE_DIST = 1000000.0;

    Point3        origin;
    Vector3        dir;
    int            depth;
    // history of hits
    double        dist;
    VisibleObject object;

    public Ray (Point3 origin, Vector3 dir, int depth)
    {
        this.origin = origin;
        this.dir = new Vector3 (dir);
        this.dir.Normalize ();
        this.depth = depth;
        this.dist = HUGE_DIST;
    }

    public final int Depth () { return depth; }
    public final double Distance () { return dist; }
    public final Point3 Origin () { return origin; }
    public final Vector3 Direction () { return dir; }
    public final VisibleObject GetObject () { return object; }

    public void Hit (color shade)
    {
        // Go through all objects and try to intersect with them
        // It is a recursive procedure
        for (int i = 0; i < Painter.TheScene.ObjectCount (); i++)
        {
            Painter.TheScene.GetObject (i).Intersect (this);
        }
        DoShade (shade);
    }

    // Record a ray's intersection with an object
    public void Record (double distance, VisibleObject object)
    {
        // only record if the new object is closer
        if (distance < dist && distance > 0.001)
        {
            this.dist = distance;
            this.object = object;
        }
    }

    Point3 Intersect ()
    {
        // Find the point of intersection
        Vector3 tmp = new Vector3 (dir);
        tmp.ScaleBy (dist);
        Point3 intersect = new Point3 (origin);
        intersect.Add (tmp);
        return intersect;
    }

    void DoShade (color shade)
    {
        if (object == null || depth > Painter.TheScene.MaxDepth())
        {
            shade.Init (0, 0, 0);
            return;
        }

        if (object.OpticalProperty ().IsLuminous())
        {
            // It's the light source
            shade.Init (object.GetColor ());
            return;
        }

        // Ask the object for its shader
        Shader shader = object.CreateShader (this);
        //
        // RECURSION
        //
        shader.GetColor (shade);
    }
}

//
// Refracted ray
//
class RefractedRay extends Ray
{
    double refr;

    RefractedRay (Point3 org, Vector3 dir, int depth, double refr)
    {
        super (org, dir, depth);
        this.refr = refr;
    }
}

//
// Shadow ray points directly towards the light source
//
class ShadowRay extends Ray
{
    double attenuation;

    ShadowRay (Point3 origin, Vector3 dir, int depth)
    {
        super (origin, dir, depth);
        this.attenuation = 1.0;
    }

    public final double Attenuation () { return attenuation; }

    public void Record (double distance, VisibleObject object)
    {
      //  if (distance < dist && distance > Vector3.EPSILON)
        if (distance < dist && distance > 0.001)

        {
            if (object.OpticalProperty().Transparency () != 0.0)
            {   // pass through but record attenuation
                attenuation *= object.OpticalProperty().Transparency ();
            }
            else
            {
                this.dist = distance;
                this.object = object;
            }
        }
    }

    void DoShade (color c)
    {
        if (object != null && object.IsLuminous())
        {
            c.Init (object.GetColor());
        }
        else
        {
            c.Init (0, 0, 0);
        }
    }
}

//
// The engine that produces and traces the rays
//
class RayTracerEngine
{
    public void DoLine (int xStart, int y, int imagePixels [], int offset, int width, int off)
    {
        for (int x = 0; x < width; x++)
        {
            // Pixel on the viewing screen (at z = 0)
            Point3 pixel = new Point3 (xStart + x, y, 0);
            // vector from the eye towards the screen
            Vector3 direction = new Vector3 (pixel, Painter.TheScene.ViewPoint ());
            // The ray pointing in that direction
            Ray Root = new Ray (Painter.TheScene.ViewPoint(), direction, 0);
            color shade = new color ();
            // Let the ray recursively hit objects and return
            // the cumulative color
            Root.Hit (shade);
            // remember the color of this pixel
            imagePixels [off + offset + x] = shade.Rgb();
        }
    }
};

//
// The scene: contains all the objects,
// the point of view (the eye) and the
// single light source
//
class Scene
{
    final int MAX_OBJ = 6;

    Point3 viewPoint;
    Point3 lightPoint;
    color  ambient;
    int    maxDepth; // of recursion

    VisibleObject objects [];    // list of objects
    int objCount;

    public Scene (Image tile)
    {
        // Create a simple scene
        this.maxDepth = 4;
        this.objects = new VisibleObject [MAX_OBJ];
        this.objCount = 0;
        this.viewPoint = new Point3 (250, 150, 600);
        this.lightPoint = new Point3 (-500, -100, 200);
        ambient = new color (0.2, 0.2, 0.2);

        AddPlane (tile);
        AddSphere1 ();
        AddSphere2 ();
        AddSphere3 ();
        AddSphere4 ();
        //AddSphere5 ();
        AddLight ();
    }

    void AddPlane (Image tile)
    {
        Point3 center = new Point3 (500, 0, -400);
        Vector3 perp = new Vector3 (-1, 0, 2);
        color c = new color (1, 1, 1);

        double refr = 0;
        double transp = 0;
        double refl = 0;
        double diffusion = 1;

        OpticalProp prop = new OpticalProp (c, refr, transp, refl, diffusion);

        // prepare tile by grabbing pixels from the image
        int width = tile.getWidth (null);
        int height = tile.getHeight (null);
        int[] pixels = new int [width * height];
        PixelGrabber grabber = new PixelGrabber (
                                    tile,
                                    0, 0,
                                    width, height,
                                    pixels,
                                    0,
                                    width);
        try
        {
	    grabber.grabPixels ();
        }
        catch (InterruptedException e)
        {
        }

        objects [objCount++] = new Plane (center, perp, prop, pixels, width, height);
    }

    void AddSphere4 ()
    {
        Point3 center = new Point3 (259, 149, -100);
        double radius = 100;

        color c = new color (1, 1, 1);
        double refr = 1.5;
        double transp = 0.6;
        double refl = 0.2;
        double diffusion = 0;
        OpticalProp prop = new OpticalProp (c, refr, transp, refl, diffusion);
        objects [objCount++] = new Sphere (center, radius, prop);
    }


    void AddSphere1 ()
    {
        Point3 center = new Point3 (359, 249, -300);
        double radius = 200;

        color c = new color (1, 1, 1);
        double refr = 1.5;
        double transp = 0.6;
        double refl = 0.2;
        double diffusion = 0;
        OpticalProp prop = new OpticalProp (c, refr, transp, refl, diffusion);
        objects [objCount++] = new Sphere (center, radius, prop);
    }

    void AddSphere2 ()
    {
        Point3 center = new Point3 (250, 295, -50);
        double radius = 44;

        color c = new color (0.2, 0.12, 0.9);
        double refr = 0;
        double transp = 0;
        double refl = 0.1;
        double diffusion = 0.4;

        OpticalProp prop = new OpticalProp (c, refr, transp, refl, diffusion);
        objects [objCount++] = new Sphere (center, radius, prop);
    }

    void AddSphere3 ()
    {
        Point3 center = new Point3 (79, 295, -108);
        double radius = 84;

        color c = new color (0.4, 0.95, 0.3);
        double refr = 0;
        double transp = 0;
        double refl = 0.5;
        double diffusion = 0.4;

        OpticalProp prop = new OpticalProp (c, refr, transp, refl, diffusion);
        objects [objCount++] = new Sphere (center, radius, prop);
    }
    void AddSphere5 ()
    {
        Point3 center = new Point3 (49, 315, -158);
        double radius = 84;

        color c = new color (0.95, 0.1, 0.1);
        double refr = 0;
        double transp = 0;
        double refl = 0.5;
        double diffusion = 0.4;

        OpticalProp prop = new OpticalProp (c, refr, transp, refl, diffusion);
        objects [objCount++] = new Sphere (center, radius, prop);
    }





    void AddLight ()
    {
        Point3 center = new Point3 (lightPoint);
        double radius = 120;

        color c = new color (1.1, 1.1, 0.9);
        double refr = 0;
        double transp = 0;
        double refl = 0.5;
        double diffusion = 0.5;

        OpticalProp prop = new OpticalProp (c, refr, transp, refl, diffusion);
        prop.MakeLuminous ();
        objects [objCount++] = new Sphere (center, radius, prop);
    }

    public final int MaxDepth () { return maxDepth; }
    public final color Ambient () { return ambient; }
    public final int ObjectCount () { return objCount; }
    public final VisibleObject GetObject (int i) { return objects [i]; }

    final public Point3 ViewPoint() { return viewPoint;    }
    final public Point3 LightPoint() { return lightPoint; }
}

//
// The painter object asynchronously creates the ray-traced picture
// line-by-line
//
class Painter implements ImageProducer, Runnable
{
    public static Scene    TheScene;

    Thread             thread;
    RayTracerEngine        engine;
    ImageConsumer     consumer;
    int             imagePixels [];
    int             linesConsumed;
    int                linesProduced;
    int             maxX;
    int             maxY;

    public Painter (int MaxX, int MaxY, Image tile)
    {
        // initialize the scene
        this.TheScene = new Scene (tile);
        this.engine = new RayTracerEngine ();
        this.linesProduced = 0;
        this.linesConsumed = 0;
        this.maxX = MaxX;
        this.maxY = MaxY;
        this.imagePixels = new int [MaxX * MaxY];
        this.thread = new Thread (this, "Ray Tracing Thread");
        thread.start ();
    }

    public void run ()
    {
        for (int y = 0; y < maxY; y++)
        {
            // calculate a scan line
            //engine.DoLine (y, imagePixels, y * maxX, maxX);

            synchronized (this)
            {
                linesProduced++;
                if (consumer != null)
                {
                    // give pixels to the consumer
                    consumer.setPixels (
                            0,                 // starting x
                            linesConsumed,    // starting y
                            maxX,             // width
                            linesProduced - linesConsumed,     // height
                            ColorModel.getRGBdefault (),// color model
                            imagePixels,         // array of imagePixels
                            linesConsumed * maxX, // offset into array
                            maxX);            // line width
                    linesConsumed = linesProduced;
                    // Tell the consumer to update the display
                    consumer.imageComplete (ImageConsumer.SINGLEFRAMEDONE);
                }
            }
        }
        synchronized (this)
        {
            if (consumer != null)
                consumer.imageComplete (ImageConsumer.STATICIMAGEDONE);
        }
    }

    //
    // ImageProducer implementation
    //
    public synchronized void addConsumer (ImageConsumer ic)
    {
        consumer = ic;
        consumer.setColorModel (ColorModel.getRGBdefault ());
        consumer.setDimensions (maxX, maxY);
        consumer.setHints (ImageConsumer.COMPLETESCANLINES | ImageConsumer.RANDOMPIXELORDER);
    }

    public synchronized boolean isConsumer (ImageConsumer ic)
    {
        // if (consumer != null) throw;
        return ic == consumer;
    }

    public synchronized void removeConsumer (ImageConsumer ic)
    {
        //if (ic != consumer) throw;
        consumer = null;
    }

    public synchronized void requestTopDownLeftRightResend (ImageConsumer ic)
    {
        // if (ic != consumer) throw
        consumer.setPixels (0, 0, maxX, linesProduced, ColorModel.getRGBdefault (),
                            imagePixels, 0, maxX);
        linesConsumed = linesProduced;
        if (linesProduced == maxY)
        {
            consumer.imageComplete (ImageConsumer.STATICIMAGEDONE);
        }
    }

    public synchronized void startProduction (ImageConsumer ic)
    {
        addConsumer (ic);
        if (linesProduced == maxY)
        {
            requestTopDownLeftRightResend (consumer);
        }
    }
}

public class RenderTask extends TaskEntry {
    /**
     * @serial
     */
    public Long requestor;

    /**
     * @serial
     */
    public Integer startY;

    /**
     * @serial
     */
    public Integer stopY;

    /**
     * @serial
     */
    public Integer startX;

    /**
     * @serial
     */
    public Integer stopX;

    public Entry execute(WeakTable cache) {
	Toolkit tk = Toolkit.getDefaultToolkit();
        Image logo = tk.getImage(
	    "/vob/jive/src/com.sun.jini.example.raytrace/logo.gif");
	Canvas output = new Canvas();
        MediaTracker imageTracker = new MediaTracker(output);
        imageTracker.addImage(logo, 0);
        try {
            imageTracker.waitForID(0);
        } catch (InterruptedException e) {}
	if (Painter.TheScene == null)	// create it if needed
	    Painter.TheScene = new Scene(logo);
        RayTracerEngine engine = new RayTracerEngine();

	int pixels;
	int width = stopX.intValue() - startX.intValue() + 1;
	int height = stopY.intValue() - startY.intValue() + 1;
	pixels = width * height;
	int imagePixels[] = new int [pixels];

	int i = 0;
	for (int y = startY.intValue(); y <= stopY.intValue() ; y++) {
	    engine.DoLine(startX.intValue(), y, imagePixels, 0, width, i);
	    i += width;
	}

	RenderResult result = new RenderResult();
	result.requestor = requestor;
	result.startX = startX;
	result.startY = startY;
	result.points = imagePixels;
	return result;
    }

    public String toString() {
	return requestor + ": (" + startX + "," + startY + ") - " +
	       "(" + stopX + "," + stopY + ")";
    }
}
