/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.example.raytrace;

import com.sun.jini.example.books.RegistryGUI;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.AbstractButton;
import javax.swing.JButton;
import java.awt.GridLayout;
import java.awt.event.WindowListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.awt.Font;

public class RunitGUI
{
    
    private boolean    rmiBinderFlag =  false;
    private String     binderName;
    private JPanel     panel;
    private GridLayout classesGUILayout;

    private JButton    startRayTrace,stopRayTrace;
    private JButton    startWorker,stopWorker;

    private RegistryGUI registryGUIRef;
    private DataGUI     dataGUIRef;

    private RunRayTrace rayTraceProcess;
    private RunWorker   workerProcess;
    
    public RunitGUI( JPanel panel, String text,
		     RegistryGUI registryGUI,
		     DataGUI     dataGUI) {

	//Save references to the GUI components for later
	//retreival of panel sub-components
	registryGUIRef  = registryGUI;
        dataGUIRef      = dataGUI;

	//Setup the Look and Feel for this Panel
	classesGUILayout = (new GridLayout(0, 2));
	classesGUILayout.setHgap(3);
	classesGUILayout.setVgap(3);

	panel.setLayout(classesGUILayout);
	//panel.setFont(new Font("Dialog", Font.PLAIN, 24));

	//Populate the GUI Piecces

	startRayTrace = new JButton("Start Raytrace");
	stopRayTrace  = new JButton("Stop Raytrace");

	startWorker= new JButton("Start Worker");
	stopWorker = new JButton("Stop Worker");

	//Add Callbacks for Mouse Events
	SymMouse aSymMouse = new SymMouse();
	startRayTrace.addMouseListener(aSymMouse);
	stopRayTrace.addMouseListener(aSymMouse);
	startWorker.addMouseListener(aSymMouse);
	stopWorker.addMouseListener(aSymMouse);

	//Populate the panel
	panel.add( new JLabel(" "));
	panel.add( new JLabel(" "));

	panel.add(startRayTrace);
	panel.add(stopRayTrace);

	panel.add(startWorker);
	panel.add(stopWorker);

	panel.add( new JLabel(" "));
	panel.add( new JLabel(" "));
    }//end-constructor
    
    class SymMouse extends java.awt.event.MouseAdapter {
	
	public void mouseClicked(java.awt.event.MouseEvent event) {
	    
	    Object object = event.getSource();

	    if (object == startRayTrace) {
		System.err.println("Starting RayTrace");
		rayTraceProcess = new 
		    RunRayTrace(dataGUIRef.getPolicyFile(),
			      dataGUIRef.getSpaceName(),
			      registryGUIRef.getCurrentRegistryType(),
			      registryGUIRef.getRegistryPort(),
			      registryGUIRef.getLookupGroups(),
			      dataGUIRef.getJarTop(),
			      dataGUIRef.getURLTop(),
			      dataGUIRef.getTraceFile());

	    }

	    if (object == stopRayTrace) {
		System.err.println("Stopping RayTrace");
		rayTraceProcess.destroy();
	    }

	    if (object == startWorker) {
		System.err.println("Starting Worker");
		workerProcess = new 
		    RunWorker(dataGUIRef.getPolicyFile(),
			      dataGUIRef.getSpaceName(),
			      registryGUIRef.getCurrentRegistryType(),
			      registryGUIRef.getRegistryPort(),
			      registryGUIRef.getLookupGroups(),
			      dataGUIRef.getJarTop(),
			      dataGUIRef.getURLTop(),
			      dataGUIRef.getWorkerFile());
	    }

	    if (object == stopWorker) {
		System.err.println("Stopping Worker");
		workerProcess.destroy();

	    }

	}//end-mouseClicked
    }//end-SymMouse

}//end-class



