/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */


/*
 * The contents of this file are subject to the Sun Community Source
 * License, Jini Technology Core Platform, v.1.0 (the "License"); you
 * may not use this file except in compliance with the License. You may
 * obtain a copy of the License at http://www.sun.com/jini.  Software
 * distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 * License for the specific language governing rights and limitations
 * under the License.
 *
 * The Original Code is Jini Technology Core Platform code, version v.1.0.
 * The Developer of the Original Code is Sun Microsystems, Inc. Portions
 * created by Sun Microsystems, Inc. are Copyright (C) 1998 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * Contributor(s): Eric Freeman
 */


package com.sun.jini.example.raytrace;

import com.sun.jini.example.spaceApplet.*;
import net.jini.space.*;
import net.jini.core.entry.*;
import net.jini.core.transaction.*;
import net.jini.core.lease.*;

import java.rmi.*;
import java.awt.*;
import java.awt.event.*;
import java.awt.image.*;
import java.util.*;

/**
 * An applet that places encryption cracking requests into a JavaSpace
 * for workers to chew on.
 *
 * @see RenderTask
 * @see RenderResult
 * @see com.sun.jini.example.genericWorker.GenericWorker
 */
public class Trace extends SpaceApplet {
    /**
     * @serial
     */
    private TextField	xText, yText;	// chunk dimension typing fields

    /**
     * @serial
     */
    private int		xChunk, yChunk;	// image chunk sizes

    /**
     * @serial
     */
    private Label	msg;		// message display

    /**
     * @serial
     */
    private Component	display;	// where the image is shown

    /**
     * @serial
     */
    private int		numChunks;	// number of chunks in this job

    /**
     * @serial
     */
    private Long	taskID;		// the current task ID

    /**
     * @serial
     */
    private long	resultCnt;	// the number of results returned

    /**
     * @serial
     */
    private JavaSpace	space;		// the space we're using

    /**
     * @serial
     */
    private Random	rand = new Random();

    /**
     * @serial
     */
    private WriteThread	writeThread;	// the thread doing the writing

    /**
     * @serial
     */
    private TakeThread	takeThread;	// the thread doing the reading

    /**
     * @serial
     */
    private ImgPainter	painter;	// this is the creator of the image

    /**
     * @serial
     */
    private boolean	logoLoaded;	// has our logo been loaded yet?

    /**
     * @serial
     */
    private Image	logo;		// this will be tiled on the plane

    /**
     * @serial
     */
    private Image	picture;	// this is what we are creating

    private final static int MAX_X = 640;
    private final static int MAX_Y = 480;

    //!! These should be private, but a 1.2beta2 compiler bug prevents it
    final static int RANDOM = 0;
    final static int FORWARD = 1;
    final static int BACKWARD = 2;

    //!! see comment above GridElement class for a description
    private final static boolean BUG_WORKAROUND = true;

    public void init() {
	super.init();

	GridBagConstraints left = new GridBagConstraints();
	GridBagConstraints right = new GridBagConstraints();

	right.anchor = GridBagConstraints.WEST;
	right.fill = GridBagConstraints.HORIZONTAL;
	right.gridwidth = GridBagConstraints.REMAINDER;
	right.weightx = 1.0;
	right.weighty = 0.0;

        add(new Label("Chunk:", Label.RIGHT), left);
	add(xText = new TextField(4), left);
	add(new Label("x"), left);
	add(yText = new TextField(4), left);

	add(new StartButton(), left);
	add(new StopButton(), left);
	add(msg = new Label(), right);

	if (BUG_WORKAROUND) {
	    // without the bug workaround, display is not a container, so
	    // we store it as the Component it should be and then downcast
	    // it where we need to.
	    display = new Panel();
	    ((Container) display).setLayout(null);
	} else {
	    display = new TracedImage();
	}
	//display.setBackground(Color.black);
	add(display, right);
	display.setSize(MAX_X, MAX_Y);

	// First load the logo.
	// Notice: loading images is always asynchronous
	logoLoaded = false;
	logo = getImage(getDocumentBase(), "logo.gif");
	MediaTracker imageTracker = new MediaTracker(this);
	imageTracker.addImage(logo, 0);
	try {
	    imageTracker.waitForID(0);
	} catch (InterruptedException e) {
	    ;
	}
	logoLoaded = true;
    }

    // used for display
    private class TracedImage extends Component {
	public void paint(Graphics g) {
	    if (logoLoaded) {
		if (painter == null) {
		    painter = new ImgPainter(MAX_X, MAX_Y, numChunks);
		    picture = createImage(painter);
		}
		g.drawImage(picture, 0, 0, this); // observer
	    }
	}
	public Dimension getMinimumSize() { return getPreferredSize(); }
	public Dimension getPreferredSize() {
	    return new Dimension(MAX_X, MAX_Y);
	}
    }

    public void start() {
	super.start();
	space = space();
    }

    /** Start writing requests. */
    private class StartButton extends ActionButton {
	StartButton() { super("Start"); }
	public void actionPerformed(ActionEvent e) { startTrace(); }
    }

    /** Stop writing requests and clear out the queue -- cancel */
    private class StopButton extends ActionButton {
	StopButton() { super("Stop"); }
	public void actionPerformed(ActionEvent e) { stopTrace(); }
    }

    public void msg(String str) {
	msg.setText(str);
    }

    private void startTrace() {
	stopTrace();			// stop the current one
	taskID = new Long(rand.nextLong());

	try {
	    int nx = Integer.parseInt(xText.getText());
	    int ny = Integer.parseInt(yText.getText());
	    if (BUG_WORKAROUND) {
		if (nx != xChunk || ny != yChunk) {
		    Container d = (Container) display;
		    d.removeAll();
		    xChunk = nx;
		    yChunk = ny;
		    for (int x = 0; x < MAX_X; x += xChunk) {
			for (int y = 0; y < MAX_Y; y += yChunk) {
			    int width = Math.min(xChunk, MAX_X - x);
			    int height = Math.min(yChunk, MAX_Y - y);
			    d.add(new GridElement(x, y, width, height));
			}
		    }
		    d.validate();
		    d.repaint();
		}
	    }
	    xChunk = nx;
	    yChunk = ny;
	} catch (NumberFormatException e) {
	    msg("Bad chunk size");
	    return;
	}

	int xCnt = (MAX_X / xChunk) + (MAX_X % xChunk == 0 ? 0 : 1);
	int yCnt = (MAX_Y / yChunk) + (MAX_Y % yChunk == 0 ? 0 : 1);
	numChunks = xCnt * yCnt;
	msg("Writing...");
	writeThread = new WriteThread(xChunk, yChunk);
	writeThread.start();				// start writing tasks

	if (!BUG_WORKAROUND) {
	    // wait for the painter to be set by update()
	    while (painter == null) {
		try {
		    Thread.sleep(100);
		} catch (InterruptedException e) {
		    return;
		}
	    }
	}

	takeThread = new TakeThread();
	takeThread.start();				// start taking results
    }

    private synchronized void stopTrace() {
	if (writeThread != null)
	    writeThread.interrupt();			// stop writing new ones
	if (takeThread != null)
	    takeThread.interrupt();			// stop taking results
	writeThread = null;
	takeThread = null;
    }

    int getOrder() {
	String orderStr = getParameter("order", "random").toLowerCase();
	if (orderStr.equals("random"))
	    return RANDOM;
	else if (orderStr.equals("forward"))
	    return FORWARD;
	else if (orderStr.equals("backward"))
	    return BACKWARD;
	System.out.println("Unknown order:" + orderStr + " (using random)");
	return RANDOM;
    }

    String getParameter(String name, String defVal) {
	String val = getParameter(name);
	if (val == null)
	    return defVal;
	else
	    return val;
    }

    int getParameter(String name, int defVal) {
	String valStr = getParameter(name, "XXX");
	if (valStr == "XXX")
	    return defVal;
	else
	    return Integer.parseInt(valStr);
    }

    /**
     * A thread that keeps the number of requests high enough by
     * waiting until it gets a little low and then writing in more.
     */
    private class WriteThread extends Thread {
	private int	xChunk, yChunk;		// how big is each task?

	private WriteThread(int xChunk, int yChunk) {
	    this.xChunk = xChunk;
	    this.yChunk = yChunk;
	}

	// start up a thread to write tasks
	public void run() {
	    System.out.println("writing " + numChunks + " tasks");
	    Vector taskVector = new Vector();
	    System.out.println("Chunk is (" + xChunk + "x" + yChunk + ")");
	    for (int y = 0; y < MAX_Y; y+=yChunk) {
		for (int x = 0; x < MAX_X; x += xChunk) {
		    RenderTask task = new RenderTask();
		    task.requestor = taskID;
		    task.startX = new Integer(x);
		    task.startY = new Integer(y);
		    task.stopX = new Integer(
					Math.min(x + (xChunk - 1), MAX_X - 1));
		    task.stopY = new Integer(
					Math.min(y + (yChunk - 1), MAX_Y - 1));
		    taskVector.addElement(task);
		}
	    }

	    final RenderTask[] tasks = new RenderTask[taskVector.size()];
	    taskVector.copyInto(tasks);

	    switch (getOrder()) {
	      case RANDOM:
		System.out.println("random");
		Random rand = new Random();
		for (int i = tasks.length - 1; i > 0; i--) {
		    int s = (int) (rand.nextFloat() * (i + 1));
		    if (s == i)
			continue;
		    RenderTask t = tasks[s];
		    tasks[s] = tasks[i];
		    tasks[i] = t;
		}
		break;
	      case FORWARD:
		System.out.println("forward");
		for (int i = 0; i < tasks.length / 2; i++) {
		    int s = tasks.length - 1 - i;
		    RenderTask t = tasks[s];
		    tasks[s] = tasks[i];
		    tasks[i] = t;
		}
		break;
	      case BACKWARD:
		System.out.println("backward");
		break;	// fine as is
	    }
	    for (int i = 0; i < tasks.length; i++) {
		//System.out.println("task " + tasks[i]);

		try {
		    space().write(tasks[i], null, Lease.FOREVER);
		} catch (Exception e) {
		    e.printStackTrace();
		}
	    }
	    System.out.println("wrote " + tasks.length + " tasks");
	}
    }

    /**
     * Take back any pending requests so as not to waste worker's time
     * computing something we'll throw away.
     */
    private void takeBackRequests() {
	RenderTask tmpl = new RenderTask();
	tmpl.requestor = taskID;
	Entry ans = null;
	do {
	    try {
		ans = space().take(tmpl, null, JavaSpace.NO_WAIT);
	    } catch (UnusableEntryException e) {
		ans = tmpl;			// was bogus, but keep on taking
		continue;
	    } catch (RemoteException e) {
		break;
	    } catch (InterruptedException e) {
		break;
	    } catch (TransactionException e) {
		break;
	    }
	} while (ans != null);
    }

    /**
     * A thread that keeps looking for results.
     */
    private class TakeThread extends Thread {
	private boolean	shuttingDown = false;	// currently shutting down?

	public void run() {
	    long waitTime = 1000;		// see shutting down code below
	    resultCnt = 0;
	    RenderResult tmpl = new RenderResult();
	    tmpl.requestor = taskID;

	    int seen = 0;
	    while (seen < numChunks) {
		try {
		    Exception ex = null;	// exception generated (if any)
		    RenderResult result = null;	// result returned (if any)
		    try {
			result = (RenderResult)space.take(tmpl, null, waitTime);
		    } catch (UnusableEntryException e) {
			ex = e;
		    } catch (TransactionException e) {
			ex = e;
		    } catch (RemoteException e) {
			ex = e;
		    } finally {
			if (ex != null) {	// common code for exceptions
			    show(ex);
			    continue;
			}
		    }

		    if (shuttingDown) {		// if shutting down...
			if (result == null)	// ...stop when nothing left
			    return;
			else			// ...but if we got something
			    continue;		// ...keep looping
		    }

		    if (result == null)		// nothing this time
			continue;

		    int x =result.startX.intValue();
		    int y = result.startY.intValue();
		    msg(String.valueOf(++resultCnt) + " of " + numChunks);
		    if (BUG_WORKAROUND) {
			Container d = (Container) display;
			GridElement ge = (GridElement) d.getComponentAt(x, y);
			ge.pixels(result.points);
		    } else {
			painter.doBlock(x, y, xChunk, yChunk, result.points);
		    }
		    seen++;
		} catch (InterruptedException e) {
		    waitTime = JavaSpace.NO_WAIT;
		    shuttingDown = true;
		}
	    }
	    msg("done");
	    if (!BUG_WORKAROUND)
		painter.complete();
	}
    }


    /** Show an exception. */
    private void show(Throwable t) {
	t.printStackTrace(System.out);
	msg(t.toString());
    }

    //
    // The painter object asynchronously creates the ray-traced picture
    // line-by-line
    //
    private static class ImgPainter implements ImageProducer {
	Vector		consumers;
	int 		width, height;
	int[] 		imagePixels;
	int		blocksLeft;

	ImgPainter(int width, int height, int blocksExpected) {
	    // initialize the scene
	    this.width = width;
	    this.height = height;
	    imagePixels = new int[width * height];
	    consumers = new Vector();
	    blocksLeft = blocksExpected;
	}

	public synchronized void
	    doBlock(int x, int y, int width, int height, int[] imagePixels)
	{
	    for (int i = consumers.size() - 1; i >= 0; i--) {
		ImageConsumer ic = (ImageConsumer) consumers.elementAt(i);
		// give pixels to the consumer
		ic.setPixels(
			x, y, width, height,		// range
			ColorModel.getRGBdefault(),	// color model
			imagePixels, 			// array of imagePixels
			0,				// offset into array
			width);				// line width
	    }
	    --blocksLeft;
	}

	public synchronized void complete() {
	    for (int i = consumers.size() - 1; i >= 0; i--) {
		ImageConsumer ic = (ImageConsumer) consumers.elementAt(i);
		ic.imageComplete(ImageConsumer.STATICIMAGEDONE);
	    }
	}

	//
	// ImageProducer implementation
	//
	public synchronized void addConsumer(ImageConsumer ic) {
	    consumers.addElement(ic);
	    ic.setColorModel(ColorModel.getRGBdefault());
	    ic.setDimensions(width, height);
	}

	public synchronized boolean isConsumer(ImageConsumer ic) {
	    return consumers.contains(ic);
	}

	public synchronized void removeConsumer(ImageConsumer ic) {
	    consumers.remove(ic);
	}

	public synchronized void requestTopDownLeftRightResend(ImageConsumer ic)
	{
	    ic.setPixels(0, 0, width, 0, ColorModel.getRGBdefault(),
			 imagePixels, 0, width);

	    if (blocksLeft <= 0)
		ic.imageComplete(ImageConsumer.STATICIMAGEDONE);
	}

	public synchronized void startProduction(ImageConsumer ic) {
	    addConsumer(ic);	// harmless if redundant
	    if (blocksLeft <= 0)
		requestTopDownLeftRightResend(ic);
	}
    }

    //!! 1.2 incremental image bug workaround:
    //
    // In JDK1.2 Beta 2 incremental image update is *not* done by
    // Component, so the image producer ImgPainter will not work.
    // (Bugtraq bug #4098417).  As a workaround, we create a grid
    // of components that each display a part of the image.  This
    // is more expensive, so it would be nice to remove this when
    // the bug gets fixed.
    private static class GridElement extends Component implements ImageProducer
    {
	final int x, y;
	final int width, height;
	Image picture;
	int[] pixels;
	Vector consumers = new Vector();

	GridElement(int x, int y, int width, int height) {
	    this.x = x;
	    this.y = y;
	    this.width = width;
	    this.height = height;
	    setLocation(x, y);
	    setSize(width, height);
	    setVisible(true);
	}
	public void update(Graphics g) {
	    paint(g);
	}
	public void paint(Graphics g) {
	    if (picture == null)
		picture = createImage(this);
	    g.drawImage(picture, 0, 0, this); // observer
	}
	public Dimension getMinimumSize() { return getPreferredSize(); }
	public Dimension getPreferredSize() {
	    return new Dimension(MAX_X, MAX_Y);
	}

	public synchronized void pixels(int[] imagePixels) {
	    pixels = imagePixels;
	    for (int i = consumers.size() - 1; i >= 0; i--) {
		ImageConsumer ic = (ImageConsumer) consumers.elementAt(i);
		// give pixels to the consumer
		ic.setPixels(
			0, 0, width, height,		// range
			ColorModel.getRGBdefault(),	// color model
			imagePixels, 			// array of imagePixels
			0,				// offset into array
			width);				// line width
	    }
	    complete();
	    //repaint();
	}

	public synchronized void complete() {
	    for (int i = consumers.size() - 1; i >= 0; i--) {
		ImageConsumer ic = (ImageConsumer) consumers.elementAt(i);
		ic.imageComplete(ImageConsumer.STATICIMAGEDONE);
	    }
	}

	//
	// ImageProducer implementation
	//
	public synchronized void addConsumer(ImageConsumer ic) {
	    consumers.addElement(ic);
	    ic.setColorModel(ColorModel.getRGBdefault());
	    ic.setDimensions(width, height);
	    ic.setHints(ImageConsumer.RANDOMPIXELORDER);
	}

	public synchronized boolean isConsumer(ImageConsumer ic) {
	    return consumers.contains(ic);
	}

	public synchronized void removeConsumer(ImageConsumer ic) {
	    consumers.remove(ic);
	}

	public synchronized void requestTopDownLeftRightResend(ImageConsumer ic)
	{
	    if (pixels != null) {
		ic.setPixels(0, 0, width, height,
		    ColorModel.getRGBdefault(), pixels, 0, width);
		ic.imageComplete(ImageConsumer.STATICIMAGEDONE);
	    }
	}

	public synchronized void startProduction(ImageConsumer ic) {
	    addConsumer(ic);	// harmless if redundant
	    requestTopDownLeftRightResend(ic);
	}
    }

    public static void main(String[] args) throws Throwable {
	throw new RuntimeException("Raytracer must run in applet context");
	// there is too much applet context to run this outside of appletviewer
	//main(new Trace());
    }
}
