/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.example.service;

import java.io.*;
import java.lang.*;
import java.io.IOException;
import com.sun.jini.system.FileSystem;
import com.sun.jini.thread.StreamPlugThread;

public class RunTxnManager {


    private Process p1;
    private Runtime rt;
    private String  runCmd;
    private String  logDir;
	
    public RunTxnManager( String bindName,
			  String registryPort,
			  String jarTop,
			  String mahaloServer,
			  String urlTop,
			  String mahaloClient,
			  String policyFile,
			  String logDir,
			  String lookupGroups,
			  String txnOptions ) {


	this.logDir = logDir;

	runCmd = "java -jar ";
	runCmd = runCmd + "-Djava.security.policy=" + policyFile;

	if ( bindName.equals("rmi-registry") ) {
	  lookupGroups="none";
	  runCmd = runCmd + " -Dcom.sun.jini.use.registry=yes ";
	  runCmd = runCmd + " -Dcom.sun.jini.rmiregistryport=" + registryPort;
        }	   

	runCmd = runCmd +
		" -Dcom.sun.jini.mahalo.managerName=TransactionManager ";

	if ( jarTop.equals("") )
	    runCmd = runCmd + mahaloServer + " ";
	else
	    runCmd = runCmd + jarTop + File.separator + mahaloServer + " ";

	if ( urlTop.equals("") )
	    runCmd = runCmd + mahaloClient;
	else
	    runCmd = runCmd + urlTop + "/" + mahaloClient;

	runCmd = runCmd + " " + policyFile;
	runCmd = runCmd + " " + logDir;
	runCmd = runCmd + " " + lookupGroups;
	runCmd = runCmd + " " + txnOptions;

	try {
	    FileSystem.destroy((new File(logDir)),true);
	    System.err.println("found " + logDir + " ...removing it");
	} catch (IOException e) {
	    System.err.println("Could not delete " + logDir);
	}

        try {
            Runtime rt = Runtime.getRuntime();
            System.err.println("\n\nRunning " + runCmd );

	    p1 = rt.exec(runCmd);

          StreamPlugThread.plugTogether(p1.getInputStream(),System.out);
	  StreamPlugThread.plugTogether(p1.getErrorStream(),System.err);

        } catch (IOException e) {
            System.err.println("exec error:" + e);
          }
    }

    public void destroy() {
	if (p1 != null) {
	    p1.destroy();
	    System.err.println("Killing TxnManager Service");
	} else {
	    System.err.println("Txn Manager Not Running");
	}

	try {
	    FileSystem.destroy((new File(logDir)),true);
	    System.err.println("found " + logDir + " ...removing it");
	} catch (IOException e) {
	    System.err.println("Could not delete " + logDir);
	}

    }
}



