/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.example.spaceApplet;

import java.awt.*;
import java.awt.event.*;

/**
 * Execute an action and time how long it took.
 */
public abstract class TimedButton extends ActionButton {
    /**
     * @serial
     */
    private Timer		t = new Timer();

    /**
     * @serial
     */

    private Frame		myFrame;

    /**
     * @serial
     */
    protected SpaceApplet	mgr;

    public TimedButton(String label, SpaceApplet mgr) {
	super(label);
	this.mgr = mgr;
    }

    /**
     * The button has been pushed, time the event, calling <code>pushed</code>.
     * @see #pushed
     */
    public final void actionPerformed(ActionEvent e) {
	// "where" is the string label of the button (according to the API doc)
	waiting(true);
	long start = System.currentTimeMillis();
	pushed();
	long end = System.currentTimeMillis();
	waiting(false);
	mgr.msg(getLabel() + ": " + (end - start) + " ms");
    }

    /**
     * Do what you should do when this button are pushed.
     */
    public abstract void pushed();

    private void waiting(boolean wait) {
	int cnum = (wait ? Cursor.WAIT_CURSOR : Cursor.DEFAULT_CURSOR);
	mgr.setCursor(Cursor.getPredefinedCursor(cnum));
    }
}
