/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.example.spaceApplet;

/**
 * Provide a timer that can be started and stopped to meter execution
 * time.  This class is <code>final</code> to increase its speed
 * to the maximum possible.  Sorry.
 * @author Ken Arnold
 */
public final class Timer {
    private String	Name;
    private int		Count;
    private long	Total;
    private long	StartTime;

    /**
     * Create a time with no name.
     */
    public Timer() {
    }

    /**
     * Create a time with a given name.
     */
    public Timer(String name) {
	Name = name;
    }

    /**
     * Return the timer's name
     */
    public String name() {
	return Name;
    }

    /**
     * Return the number of times the timer has been stopped.
     */
    public int count() {
	return Count;
    }

    /**
     * Return the total time so far (not including any timing sequence
     * that has been started and not stopped).
     */
    public long total() {
	return Total;
    }

    /**
     * Start up the timer.  If two consecutive invactions are
     * made without an invocation of <code>stop</code>, the first
     * is forgotten.
     */
    public void start() {
	StartTime = System.currentTimeMillis();
    }

    /**
     * Stop the timer, and add the elapsed time
     * since the last <code>start</code> to the total.
     * A <code>stop</code> without a preceeding <code>start</code>
     * is ignored.
     */
    public void stop() {
	if (StartTime != 0) {
	    Total += (System.currentTimeMillis() - StartTime);
	    Count++;
	    StartTime = 0;
	}
    }

    public String toString() {
	String label = (Name == null ? "" : Name + ": ");
	return label + Count + " in " + Total + " ms ("
	    + (1000 * Count / (double) Total) + " per sec)";
    }

    /**
     * Two Timer's are equal if their names are the same.
     */
    public boolean equals(Object obj) {
	if (obj instanceof Timer)
	    return Name.equals(((Timer) obj).Name);
	return false;
    }

    /**
     * Return the name's <code>hashCode</code>
     */
    public int hashCode() {
	return Name.hashCode();
    }
}
