/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.example.spaceApplet;

import java.rmi.Remote;
import java.rmi.RemoteException;

import net.jini.core.transaction.Transaction;

/**
 * Interface for a commune of transaction sharers.  One entity in the
 * communue (usually simply the first to bind a give name into the
 * naming service) is the executive.  Everyone else adds themselves to
 * the commune by invoking <code>add</code> on the executive.  Add
 * returns a name that the member should use to prefix transaction
 * names.  When a member creates a transaction it wants to share, it
 * invokes the executive's <code>add</code> with that transaction's
 * name and the transaction itself, which results in <code>add</code>
 * being invoked on all members (including the originator).  Similarly
 * the executive's <code>delete</code> method is invoked when the
 * transaction is no longer active.
 *
 * @see com.sun.jini.example.books.RFBApplet
 */
public interface TxnCommune extends Remote {
    /**
     * Add a new member to the commune with the given name base.  The name
     * base is used by the executive to generate a transaction name prefix
     * to be used by the enrolled member when creating transactions.  For
     * example, if <code>nameBase</code> were <code>"Watcher"</code>, the
     * value returned by add might be <code>"Watcher 1"</code>.  All future
     * transactions created by that member should be named start with the
     * string <code>"Watcher 1"</code> followed by a suffix that is unique
     * relative to that member (e.g., <code>"Watcher 1 <i>X</i>"</code> where
     * <code><i>X</i></code> is a monotonically increasing number for each
     * new transaction.
     */
    String add(TxnCommune member, String nameBase) throws RemoteException;

    /**
     * Remove a member from the commune.
     */
    void delete(TxnCommune member) throws RemoteException;

    /**
     * Add a transaction to the commune's set under the given name.
     *
     * @see #add(TxnCommune,String)
     */
    void add(Transaction tr, String name) throws RemoteException;

    /**
     * Delete a transaction from the commune's set.
     */
    void delete(String name) throws RemoteException;
}
