/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.example.spaceBrowser;

import com.sun.jini.example.browser.EntryTreePanel;
import com.sun.jini.outrigger.JavaSpaceAdmin;
import com.sun.jini.outrigger.AdminIterator;
import net.jini.core.entry.Entry;
import net.jini.core.entry.UnusableEntryException;
import net.jini.core.transaction.TransactionException;
import java.rmi.RemoteException;
import java.awt.Toolkit;
import java.awt.Frame;
import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.util.List;
import javax.swing.JFrame;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;

/**
 * A browser utility to browse snapshotted entries in a specified space.
 *
 * @version 0.2 06/04/98
 * @author Haruo Fukuda
 */
public class SpaceBrowser extends JFrame {
  //private SpaceEntryPanel entryPanel;
  private final static int MINIMUM_WINDOW_WIDTH = 320;

  public SpaceBrowser(JavaSpaceAdmin admin, Frame parent) {
    super("SpaceBrowser");

    // init main components
    SpaceEntryPanel entryPanel = new SpaceEntryPanel(admin);

    // add menu and attr panel
    getContentPane().setLayout(new BorderLayout());
    getContentPane().add(new BrowserMenuBar(entryPanel), "North");
    getContentPane().add(entryPanel, "Center");

    validate();
    pack();
    setSize(((getSize().width < MINIMUM_WINDOW_WIDTH) ? MINIMUM_WINDOW_WIDTH : getSize().width),
	    getSize().height);

    // set frame location
    int xpos = 0;
    int ypos = 0;
    if(parent == null){
      // no parent frame is defined. set the center of screen
      Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
      Dimension dialogSize = getPreferredSize();
      xpos = (screenSize.width - dialogSize.width)/2;
      ypos = (screenSize.height - dialogSize.height)/2;
    } else {
      // centerize to parent frame
      Rectangle bounds = parent.getBounds();
      Dimension dialogSize = getPreferredSize();
      xpos = bounds.x + (bounds.width - dialogSize.width)/ 2;
      ypos = bounds.y + (bounds.height - dialogSize.height)/2;
    }
    setLocation((xpos < 0) ? 0 : xpos,
		(ypos < 0) ? 0 : ypos);
  }


  class BrowserMenuBar extends JMenuBar {
    private EntryTreePanel entryPanel;

    public BrowserMenuBar(EntryTreePanel entryPanel) {
      this.entryPanel = entryPanel;
      JMenuItem mitem;

      // "File" Menu
      JMenu fileMenu = (JMenu) add(new JMenu("File"));
      mitem = (JMenuItem) fileMenu.add(new JMenuItem("Refresh"));
      mitem.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent ev) {
	  BrowserMenuBar.this.entryPanel.refreshPanel();
	}
      });
      mitem = (JMenuItem) fileMenu.add(new JMenuItem("Close"));
      mitem.addActionListener(new ActionListener() {
	public void actionPerformed(ActionEvent ev) {
	  SpaceBrowser.this.setVisible(false);
	}
      });
    }
  }

  // Allocate only one empty array of entries
  private static final Entry[] emptyArray = new Entry[0];

  class SpaceEntryPanel extends EntryTreePanel {
    private JavaSpaceAdmin admin;

    public SpaceEntryPanel(JavaSpaceAdmin admin) {
      super(false);	// Entries are not editable.
      this.admin = admin;

      refreshPanel();
    }

    protected Entry[] getEntryArray() {
      try {
	AdminIterator iter = admin.contents(null, null, 128);
	List acc = new java.util.LinkedList();

	// Acculate entries in List
	Entry e;
	while ((e = iter.next()) != null) 
	    acc.add(e);	
	iter.close();

	// Construct array from List
	return (Entry[])acc.toArray(emptyArray);
      } catch(RemoteException e){
	e.printStackTrace();
      } catch(UnusableEntryException e) {
	e.printStackTrace();
      } catch(IllegalArgumentException e){
	System.out.println(e.getMessage());
	//e.printStackTrace();
      } catch(TransactionException e) {
	e.printStackTrace();
      }

      return null;
    }
  }
}
