/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */

package com.sun.jini.lease;

import net.jini.core.lease.*;

/**
 * @deprecated This class was experimental; as of 1.1Alpha, it has been
 *	replaced by {@link net.jini.lease.LeaseRenewalEvent}. This class  
 *	was needed by <tt>net.jini.lease.LeaseRenewalManager</tt>. This  
 *	class will be removed in a future release, so please update your
 *      code, and refer to the javadoc for 
 *      <tt>net.jini.lease.LeaseRenewalEvent</tt> for details on 
 *      <code>LeaseRenewalEvent</code>.
 * 
 * @see net.jini.lease.LeaseRenewalEvent
 * @see net.jini.lease.LeaseRenewalManager
 */
public class LeaseRenewalEvent extends java.util.EventObject {
    /**
     * The failed lease.
     *
     * @serial
     */
    private Lease lease;
    /**
     * The desired expiration of the failed lease.
     *
     * @serial
     */
    private long expiration;
    /**
     * The exception that caused the failure, if any.
     *
     * @serial
     */
    private Exception ex;

    /**
     * Construct an event.
     *
     * @param source the lease renewal manager that generates the event
     * @param lease the lease that failed to renew
     * @param expiration the desired absolute expiration for the lease
     * @param ex the exception, if any
     */
    public LeaseRenewalEvent(LeaseRenewalManager source,
			     Lease lease,
			     long expiration,
			     Exception ex) 
    {
	super(source);
	this.lease = lease;
	this.expiration = expiration;
	this.ex = ex;
    }

    private static final long serialVersionUID = -7536467680016825016L;

    /** Returns the lease that failed to renew. */
    public Lease getLease() {
	return lease;

    }


    /** Returns the desired absolute expiration for the lease. */
    public long getExpiration() {
	return expiration;
    }


    /**
     * Returns the exception, if any.  If the lease was denied, then
     * this will be a LeaseDeniedException.  If the lease expired,
     * this will be a LeaseUnknownException, a RemoteException, or null.
     */
    public Exception getException() {
	return ex;
    }
}
