/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.lease.landlord;

import net.jini.core.lease.Lease;

/**
 * <code>LandlordLeaseAccessor</code> provides access to the landlord,
 * expiration, and cookie information associated with a given landlord
 * lease.  This is broken out into a separate interface usually implemented
 * by a distinct object instead of added to landlord lease implementation
 * themselves so clients are not given direct access to these methods.
 * <p>
 * In general it is expected that this interface will be implemented by 
 * <code>LandlordLeaseFactory</code> objects so clients of the factory can
 * later retrieve the details/mutate the state of the leases created by that
 * factory.
 * <p>
 * @see LandlordLeaseFactory
 */
public interface LandlordLeaseAccessor {
   /** 
    * Return <code>true</code> if this object can access the private state
    * of the passed lease.
    * @param  lease The lease to check
    * @return <code>true</code> if this object can read/update the
    * state of lease, return <code>false</code> otherwise.
    */
   public boolean canAccess(Lease lease);

   /**
    * Update the expiration time of the passed lease to reflect a new
    * expiration time.
    * @param lease The lease to be updated
    * @param expiration The new expiration for the lease in milliseconds
    *                   since the beginning of the epoch
    * @throws IllegalArgumentException if <code>canAccess(lease)</code>
    *         would return <code>false</code>
    */
   public void setExpiration(Lease lease, long expiration);

   /**
    * Return the landlord associated with the passed lease.
    * @param lease The lease who's Landlord is to be returned.
    * @return The <code>Landlord</code> of the passed lease.
    * @throws IllegalArgumentException if <code>canAccess(lease)</code>
    *         would return <code>false</code>
    */
   public Landlord getLandlord(Lease lease);

   /**
    * Return the cookie associated with the passed lease.
    * @param lease The lease who's cookie is to be returned.
    * @return The cookie associated with lease.
    * @throws IllegalArgumentException if <code>canAccess(lease)</code>
    *         would return <code>false</code>
    */
   public Object getCookie(Lease lease);
}
