/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.lease.landlord;

import net.jini.core.lease.Lease;
import net.jini.core.lease.LeaseDeniedException;

/**
 * Factory interface for classes that create Lease objects that
 * follow the Landlord protocol.
 * @see Landlord
 */
public interface LandlordLeaseFactory {
    /**
     * Create a new lease that works with landlord protocol.
     * @param cookie Object that will be passed back to 
     *               <code>Landlord.renew()</code> 
     *               and <code>Landlord.cancel()</code>
     * @param landlord The landlord to callback when the lease is 
     *                 renewed or canceled.
     * @param expiration The initial expiration time of the lease in
     *                 milliseconds since the beginning of the
     *                 epoch. This must be the initial expiration time
     *                 of the lease, <code>newLease()</code> may not
     *                 set the lease to have a different initial
     *                 expiration time.
     * @exception LeaseDeniedException may be thrown if the lease can
     * not be allocated.  */
    public Lease newLease(Object cookie, Landlord landlord, long expiration)
         throws LeaseDeniedException;
}
