/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.lease.landlord;

/**
 * <code>LeaseManager</code> provides an interface for tracking lease
 * status.  Some implementations of <code>LeasePolicy</code>
 * (e.g. <code>LeaseDurationPolicy</code>) allow registration of a
 * <code>LeaseManager</code> that will be notified when a new Lease is
 * created or a Lease's duration changes.
 * <p>
 * The <code>LeaseManager</code> interface has methods indicating when
 * new leases have been granted and when existing leases have been
 * renewed, it does not have a method to indicate when a lease has
 * been cancelled.  Because of this, <code>LeaseManager</code>
 * implementations need to be careful not to leak memory/resources if
 * leases are cancelled before they expire.  In particular, they
 * should not hold on to <code>LeasedResource</code>s directly since
 * this would prevent garbage collection of resources when their
 * leases are cancelled before they expire.
 *
 * @see LeaseDurationPolicy
 * @see LeasedResource
 */
public interface LeaseManager {
    /**
     * Notifies the manager of a new lease being created.
     * @param resource   The resource associated with the new Lease.
     * @param duration   The duration of the lease
     */
    public void register(LeasedResource resource, long duration);

    /**
     * Notifies the manager of a lease being renewed.
     * @param resource      The resource associated with the new Lease.
     * @param duration      The duration of the lease
     * @param oldExpiration The expiration the resource had before it 
     *                      was renewed
     */
    public void renewed(LeasedResource resource, long duration, 
			long oldExpiration);
}



