/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.lease.landlord;

import net.jini.core.lease.Lease;
import net.jini.core.lease.LeaseDeniedException;
import net.jini.core.lease.UnknownLeaseException;

/**
 * Interface for classes that implement Lease grant and renewal policies. 
 */
public interface LeasePolicy {
    /**
     * Factory method that returns lease objects for the specified
     * resource. 
     *
     * @param resource The resource the lease is for. Will have its
     *                 expiration set.
     * @param requestedDuration
     *                 The length of lease the client wants.
     * @return A lease for the passed resource
     * @exception LeaseDeniedException thrown if the policy is not currently 
     *            granting leases.
     */
    public Lease leaseFor(LeasedResource resource, long requestedDuration) 
        throws LeaseDeniedException;

    /**
     * Renews the expiration of the passed resource.  
     * @param resource The resource having its lease renewed.  Will have its 
     *                 expiration set.  
     * @param requestedDuration 
     *                 The new duration the client wants
     * @return The new duration the lease should have.
     * @exception LeaseDeniedException thrown if the policy is unwilling 
     *            to renew this lease.
     * @exception UnknownLeaseException does not know of the lease associated
     *            with the passed resource.
     */
    public long renew(LeasedResource resource, long requestedDuration)
        throws LeaseDeniedException, UnknownLeaseException;

    /**
     * Returns <code>true</code> if the lease on the passed resource
     * has not expired yet.
     */
    public boolean ensureCurrent(LeasedResource resource);
}
