/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.lease.landlord;

import java.io.Serializable; 

/**
 * Leased resources must meet this interface if they are to be used
 * with <code>LeaseDurationPolicy</code> and/or
 * <code>LeaseManager</code>
 * <p>
 * @see LeaseDurationPolicy
 * @see LeaseManager 
 */
public interface LeasedResource {
    /**
     * Changes the expiration time of the leased resource.
     * @param newExpiration The new expiration time in milliseconds
     *                      since the beginning of the epoch
     */
    public void setExpiration(long newExpiration);

    /**
     * Returns the expiration time of the leased resource.
     * @return The expiration time in milliseconds since the beginning 
     *         of the epoch
     */
    public long getExpiration();

    /**
     * Returns a unique identifier that can be used by the grantor of
     * the resource to identify it in the context of a
     * <code>Landlord.renew()</code> or <code>Landlord.cancel()</code>
     * call.  In general this object will be passed between VMs using RMI
     * so care should be taken in not making this object bigger than it
     * needs to be.
     * @see Landlord#renew
     * @see Landlord#cancel
     */
    public Object getCookie();
}

