/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.lease.landlord;

import net.jini.core.lease.LeaseDeniedException;
import net.jini.core.lease.UnknownLeaseException;

/**
 * Interface that defines the basic landlord primitives (renew and
 * cancel) as local methods.  Classes already implementing 
 * <code>Landlord</code> should be able to implement this interface
 * simply by adding it the the <code>implements</code> clause of their
 * class declaration.
 * <p>
 * Some users may want to implement this interface using an adaptor
 * class. This could be useful in situations where the landlord's
 * cancel and renew methods don't do quite the right thing for batch
 * lease operations
 * <p>
 * This interface gives local landlord utilities access to the cancel
 * and renew methods.
 * @see net.jini.core.lease.Landlord 
 */
public interface LocalLandlord {
    /**
     * Renew the lease that is associated with the given cookie.
     *
     * @param cookie   An object that uniquely identifies a lease
     *                 granted by this <code>LocalLandlord</code>.
     * @param duration The duration in milliseconds the client
     *                 wants the lease renewed for.
     * @return The new duration the lease should have
     */
    public long renew(Object cookie, long duration)
	throws LeaseDeniedException, UnknownLeaseException;
       
    /**
     * Cancel the lease that is associated with the given cookie.
     *
     * @param cookie An object that uniquely identifies a lease
     *               granted by this <code>LocalLandlord</code>.
     */
    public void cancel(Object cookie) throws UnknownLeaseException;
}

    
