/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.mahalo;

import net.jini.core.transaction.*;
import net.jini.core.transaction.server.*;

import com.sun.jini.mahout.StorableObject;

import java.io.Serializable;
import java.rmi.RemoteException;


class ParticipantHandle implements Serializable, TransactionConstants {
    static final long serialVersionUID = -1776073824495304317L;

    /**
     * @serial
     */
    private StorableObject storedpart;

    /**
     * @serial
     */
    private long crashcount = 0;

    /**
     * @serial
     */
    private int prepstate;

    private static final boolean DEBUG = false;

    /**
     * Create a new handle, calculating the hash for the object.
     */
    ParticipantHandle(TransactionParticipant part)
	throws RemoteException {
	this(part, 0);
    }

    /**
     * Create a new node that is equivalent to that node
     */
    ParticipantHandle(TransactionParticipant part, long crashcount) 
	throws RemoteException {
	try {
	    storedpart = new StorableObject(part);
	    this.crashcount = crashcount;
	} catch (RemoteException re) {
	    System.out.println("ParticipantHandle: ParticipantHandle: " +
				"Cannot store the TransactionParticipant");
	    re.printStackTrace();
	}
	this.prepstate = ACTIVE;
    }

    long getCrashCount() {
	return crashcount;
    }

    TransactionParticipant getParticipant() {
	TransactionParticipant tmp = null;
	try {
	    tmp = (TransactionParticipant) storedpart.get();
	} catch (Exception re) {
	    System.out.println("ParticipantHandle: getParticipant: " +
				"problem unpacking participant");
	    re.printStackTrace();
	}
	return tmp;
    }

    StorableObject getStoredPart() {
	return storedpart;
    }

    synchronized void setPrepState(int state) {
	switch (state) {
	    case PREPARED:
	    case NOTCHANGED:
	    case COMMITTED:
	    case ABORTED:
		break;
	    default:
		throw new IllegalArgumentException("ParticipantHandle: " +
			    "setPrepState: cannot set to " + 
		    	    com.sun.jini.constants.TxnConstants.getName(state));
	}

	this.prepstate = state;
    }

    synchronized int getPrepState() {
	return prepstate;
    }


    boolean compareTo(ParticipantHandle other) {
        if (storedpart != null)
	    return storedpart.equals(other);
	else if (other == null)
	    return false;

	return false;
    }
}
