/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.mahalo;

import com.sun.jini.mahalo.log.*;

import com.sun.jini.mahalo.ParticipantHandle;
import net.jini.core.transaction.server.TransactionManager;
import net.jini.core.transaction.server.TransactionParticipant;
import java.io.Serializable;
/**
 * A <code>LogRecord</code> which encapsulates a generic
 * interaction with a participant.
 * 
 * @author Gary Holness
 */
class ParticipantModRecord implements TxnLogRecord {
    static final long serialVersionUID = 5542043673924560855L;

    private static final boolean DEBUG = false;

    /**
     * @serial
     */
    private ParticipantHandle part;

    /**
     * @serial
     */
    private int result;

    ParticipantModRecord(ParticipantHandle part, int result) {
	if (part == null)
	    throw new IllegalArgumentException("ParticipantModRecord: " +
			    "recover: non-null ParticipantHandle " +
						"recover attempted");

	this.part = part;
	this.result = result;
    }

    ParticipantHandle getPart() {
	return part;
    }

    int getResult() {
	return result;
    }

    public void recover(TxnManagerTransaction tmt)
	throws CannotRecoverException
    {
	if (tmt == null)
	    throw new NullPointerException("ParticipantModRecord: recover: " +
			    "non-null transaction must be specified");

        tmt.modifyParticipant(getPart(), getResult());

	if (getResult() == ABORTED)
	    tmt.modifyTxnState(ABORTED);

	if (DEBUG)
	    System.out.println("ParticipantModRecord: recover: recovered");
    }
}
