/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.mahalo;

import net.jini.core.transaction.*;
import net.jini.core.transaction.server.*;

import com.sun.jini.mahalo.*;
import com.sun.jini.mahalo.log.*;
import com.sun.jini.thread.*;

import java.util.*;

import java.rmi.RemoteException;
import java.rmi.NoSuchObjectException;

/**
 * A <code>SettlerTask</code> is scheduled task, which
 * causes an unsettled transaction to settle.
 *
 * @author Gary Holness
 */
public class SettlerTask extends RetryTask implements TransactionConstants {
    private long tid;
    private int attempt;
    private int maxtries = Integer.MAX_VALUE;
    private TransactionManager txnmgr;

    private static final boolean DEBUG = false;

    /**
     * Constructs a <code>SettlerTask</code>.
     *
     * @param manager <code>TaskManager</code> providing the threads
     *                of execution.
     *
     * @param txnmgr <code>TransactionManager</code> which owns the
     *               the transaction.
     * 
     * @param tid transaction ID
     */
    public SettlerTask(TaskManager manager,
			    TransactionManager txnmgr, long tid) {
	super(manager);

	if (txnmgr == null)
	    throw new IllegalArgumentException("SettlerTask: SettlerTask: " +
					    "txnmgr must be non-null");
	this.txnmgr = txnmgr;
	this.tid = tid;
    }

    /**
     * Inherit doc comment from supertype.
     *
     * @see com.sun.jini.thread.RetryTask
     */
    public boolean runAfter(List list, int max) {
        return false;
    }

    public boolean tryOnce() {
        try {
	    if (attempt >= maxtries)
		return true;

	    attempt++;

	    if(DEBUG)
		System.out.println("SettlerTask: tryOnce: settling " +
				    "Transaction(" + tid + ")");

	    int state = txnmgr.getState(tid);
	    switch(state) {
	      case ACTIVE:
	      case COMMITTED:
                txnmgr.commit(tid, Long.MAX_VALUE);
		break;
	    
	      case ABORTED:
		txnmgr.abort(tid, Long.MAX_VALUE);
		break;
	    }

	} catch (NoSuchObjectException nsoe) {
        } catch (TransactionException te) {
	    if (DEBUG) {
                System.out.println("SettlerTask: tryOnce: " +
                                            te.getMessage());
                System.out.println("SettlerTask: " +
                   "tryOnce: cannot " +
                   "settle recovered transaction: " +
                                                tid);
                te.printStackTrace();
	    }
        } catch (RemoteException re) {
	    //try again
	    return false;
	}

            if(DEBUG)
                System.err.println("SettlerTask: tryOnce: SETTLED " +
                                    "Transaction(" + tid + ")");

	return true;
    }
}
