/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.mahalo.log;

import net.jini.admin.Administrable;

/**
 * Represents the functions performed by a log manager.
 *
 * @author Gary Holness
 */
public interface LogManager extends Administrable {

    /**
     *  Factory method which returns a Log interface which
     *  can be used to create persistent records which
     *  collectively represent a system's state.
     *
     * @param cookie identifier representing the information
     *               being logged.
     */
    ClientLog logFor(long cookie) throws LogException;


    /**
     *  Consumes the log file and re-constructs a system's
     *  state.
     */
    void recover() throws LogException;

    /**
     * Returns an object that implements whatever administration interfaces
     * are appropriate for the <code>LogManager</code>. This method
     * overrides what is defined in the supertype.
     *
     * @return an object that implements whatever administration interfaces
     *         are appropriate for the particular service.
     *
     * @see net.jini.admin.Administrable
     */
    Object getAdmin();
}
