/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.mahout;

import com.sun.jini.mahout.PersistentServiceDesc;
import java.util.Enumeration;
import java.util.Properties;
import java.io.Serializable;

/**
 * Simple implementation of PersistentServiceDesc.  Its configuration
 * is fixed at creation.  Extends <code>PersistentServiceDesc</code>
 * with an additional method to return a lookup group list.
 */
public class BasicServiceDesc implements PersistentServiceDesc, Serializable
{
    final private String path;
    final private String classpath;
    final private String codebase;
    final private String policy;
    final private String jvmPath;
    final private String[] jvmArgs;
    final private String[] propvals;
    final private String[] propkeys;
    final private String[] groups;

    /**
     * Create a BasicServiceDesc that will return the given values
     * for getClassPath(), getCodebase(), getPolicyPath(), 
     * getPersistentsPath(), getGroups(), getJVMPath(), getJVMOptions(),
     * getProperties()
     */
    public BasicServiceDesc(String classpath, String codebase, String policy,
        String path, String[] groups, String jvmPath, String[] jvmArgs,
	Properties props)
    {
	this.classpath = classpath;
	this.codebase = codebase;
	this.policy = policy;
	this.path = path;
	this.groups = groups;
	this.jvmPath = jvmPath;
	this.jvmArgs = jvmArgs;


	String key = null;
	String value = null;

	if (props != null) {
	    propkeys = new String[props.size()];
	    propvals = new String[props.size()];

	    int i = 0;
	    for (Enumeration e = props.propertyNames();
			        e.hasMoreElements(); i++) {

	        key = (String) e.nextElement();
	        value = (String) props.getProperty(key);
	        propkeys[i] = key;
	        propvals[i] = value;
	    }
	} else {
	    //Satisfy the compiler
	    propkeys = null;
	    propvals = null;
	}
    }

    // purposefully inherit doc comment from supertype
    public String getPersistentsPath() {return path;}

    // purposefully inherit doc comment from supertype
    public String getClassPath() {return classpath;}

    // purposefully inherit doc comment from supertype
    public String getCodebase() {return codebase;}

    // purposefully inherit doc comment from supertype
    public String getPolicyPath() {return policy;}

    // purposefully inherit doc comment from supertype
    public Properties getProperties() {
	Properties props = null;
	
	if (propkeys != null ) {
	    props = new Properties();

	    for (int i = 0; i < propkeys.length; i++) {
	        props.setProperty(propkeys[i], propvals[i]);
	    }
	}

	return props;
    }

    // purposefully inherit doc comment from supertype
    public String getJVMPath() {return jvmPath;}

    // purposefully inherit doc comment from supertype
    public String[] getJVMOptions() {return jvmArgs;}
    
    /**
     * Lookup groups list, set when object was created.
     */
    public String[] getGroups() {return groups;}
}
