/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.mahout;

import java.util.Properties;

/**
 * Data required by a persistent service in the Jini system to set
 * itself up. If any method returns invalid data the service setup 
 * code will throw "IllegalArgumentException"
 */
public interface PersistentServiceDesc {
    /**
     * Absolute path to a directory where the service can persist its
     * data.  If the directory exists the builder of the
     * <code>PersistentServiceDesc</code> should ensure no service is
     * using it.  Not all services can successfully use existing log
     * files.
     */
    public String getPersistentsPath();
    
    /**
     * A list of file system paths separated by
     * <code>File.pathSeparator</code> This is where the service will
     * load its classes from
     */
    public String getClassPath();

    /**
     * A space separated list of URLs.  Classes loaded from the classpath
     * will be annotated with these URLs.
     */
    public String getCodebase();

    /**
     * Location of the security policy file.
     */
    public String getPolicyPath();

    /**
     * Properties for the VM the service will be run in.  May be
     * <code>null</code>
     */
    public Properties getProperties();

    /**
     * The name of the java executable, including the full path, or
     * null, meaning "use rmid's default". The named program must be
     * able to accept multiple -Dpropname=value options (as documented
     * for the "java" tool) 
     */
    public String getJVMPath();

    /**
     * Command line options for the VM in which the service will be
     * run.  May be <code>null</code>, which has the same affect as an
     * empty array.
     */
    public String[] getJVMOptions();
}

