/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.mahout;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.AccessException;
import java.rmi.RMISecurityManager;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.Remote;

/**
 * A <code>RegistryLocator</code> implements the
 * methods needed in order to locate a registry.
 *
 * @author Ken Arnold
 * @author Gary Holness
 *
 * @see Locator
 */

public class RegistryLocator extends Locator {
    private static boolean DEBUG = false;
    public RegistryLocator() {
    }

   /**
    *  This method returns null if nothing can
    *  be found.  The semantics are currently
    *  to return the registry running on the
    *  default port.
    */

    public Object locate() {
	Registry tmpobj = null;

	String locatehost = System.getProperty("outrigger.host");
	try {
	    Registry reg = null;
	    
	    if (DEBUG)
		System.out.println("RegistryLocator: locate: host = " + locatehost);

	    String portString = 
		System.getProperty("com.sun.jini.rmiRegistryPort");
	    int port;

	    reg = LocateRegistry.getRegistry(locatehost, calcPort());

            //Create and install the security manager
            try {
                System.setSecurityManager(new RMISecurityManager());
            } catch (SecurityException se) {
                 //swallow exception if security manager
                 //has already been installed
            }

	    tmpobj = reg;

	} catch (Exception e) {
	    System.out.println("RegistryLocator: find: " + e.getMessage());
	    e.printStackTrace();
	}

	if (DEBUG)
	    System.out.println("RegistryLocator: locate: registry = " + tmpobj);

	return tmpobj;
    }

    /**
     * Determine what port to look for the RMI registry on.
     */
    public static int calcPort() {
	String portString = 
	    System.getProperty("com.sun.jini.rmiRegistryPort");

	if (portString != null) {
	    Integer portObj = Integer.valueOf(portString);
	    return(portObj.intValue());
	} else
	    return(Registry.REGISTRY_PORT);
    }	
}
