/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.mahout.binder;

import java.io.IOException;
import net.jini.admin.JoinAdmin;

/**
 * Provides an abstraction for placing a service in a directory.
 * Different binders bind the service into different types of
 * directories (i.e. RMI registry v. Jini lookup).
 *
 * @see BinderSetup
 * @author John W.F. McClain 
 */
public interface Binder {
    /**
     * Return an object that implements DirectoryAdmin.  This object
     * is should be used by the services admin to implement the
     * <code>JoinAdmin</code> and <code>RegistryAdmin</code> interfaces.
     * @see net.jini.admin.JoinAdmin
     * @see com.sun.jini.mahout.RegistryAdmin
     */
    public DirectoryAdmin getDirectoryAdmin();

    /**
     * Has the binder register the service with the appropriate
     * directory service.  Only needs to be called once, though
     * calling it more than once is safe even if an exception is
     * thrown by a previous call
     */
    public void advertise() throws IOException;

    /**
     * Unregister with the directory service and unregister any
     * activatable objects.
     */
    public void destroy();
}
