/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.mahout.binder;

import java.io.IOException;
import java.rmi.RemoteException;
import net.jini.core.discovery.LookupLocator;
import net.jini.core.entry.Entry;
import net.jini.admin.JoinAdmin;

/**
 * Implementation of Binder that binds a service into a Jini lookup.
 * When this binder is initially activated or has its
 * <code>advertise()</code> method invoked it won't join any lookups
 * because its lookup locator and lookup group lists start empty.
 * <p>
 * This binder also supports binding into the RMI registry
 *
 * @see Binder
 *
 * @author John W.F. McClain 
 */
public class LookupBinder implements Binder {
    /** 
     * Remote implementation of <code>JoinAdmin</code> to be used by
     * the service's admin to implement JoinAdmin.
     */
    private RemoteJoinAdminImpl joinAdmin;

    /**
     * The key we use to lookup the RemoteJoinAdminImpl in the
     * LocalActivatableObjectTable
     */
    private LocalActivatableObjectTable.Findable key;

    /**
     * Used to create a Lookup binder for non-activatable services.
     * @param attributes Initial attribute set
     * @param service Service to be bound into lookup and/or RMI registry.
     */
    public LookupBinder(Entry[] attributes, Object service)
        throws IOException, RemoteException
    {		
	joinAdmin = new RemoteJoinAdminImpl(service, attributes);
	// Not strictly necessary since we should never use key if
	// joinAdmin is non-null
	key = joinAdmin;
    }

    /**
     * Used by the LookupBinderSetup's BinderFactory class.
     * @param key Remote reference to <code>RemoteJoinAdminImpl</code>
     */
    LookupBinder(LocalActivatableObjectTable.Findable key)
        throws RemoteException
    {		
	joinAdmin = 
	    (RemoteJoinAdminImpl)LocalActivatableObjectTable.lookup(key);
	this.key = key;
    }

    // Inherit doc comment from super interface
    public DirectoryAdmin getDirectoryAdmin() {
	return (DirectoryAdmin)key;
    }

    // Inherit doc comment from super interface
    public void advertise() throws IOException {
	joinAdmin.advertise();
    }

    // Inherit doc comment from super interface
    public void destroy() {
	joinAdmin.destroy();
    }
}
