/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.mercury;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.io.SyncFailedException;

/**
 * Class that implements the interface for a <tt>ControlLog</tt>. 
 * This class extends <tt>java.io.RandomAccessFile</tt> and relies on 
 * <code>LogStream</code>
 * interface for the actual reading/writing of control data. The only added
 * (convenience) method is <tt>sync</tt> which allows the user to 
 * force synchronization with the underlying device.
 *
 * @since Jini1.1
 */

class ControlLog extends RandomAccessFile implements LogStream {

    /** Key associated with this stream. */
    private final StreamKey key;

    /**
     * Constructor that takes a <tt>File</tt> and <tt>StreamKey</tt> argument. 
     * The <tt>file</tt> argument is passed through to the superclass 
     * and is always opened in read/write mode. The <tt>key</tt> argument 
     * is assigned to the appropriate internal field. 
     *
     * @exception IOException if an I/O error occurs
     */
    ControlLog(File file, StreamKey key) throws IOException {
        super(file, "rw");
        this.key = key;
    }

    /**
     * Forces system buffers to synchronize with the underlying device.
     *
     * @exception IOException if an I/O error occurs
     *
     * @exception SyncFailedException if the buffers cannot be guaranteed to
     *                                have synchronized with physical media.
     */
    void sync() throws IOException, SyncFailedException {
        getFD().sync();
    }

    //
    // LogStream Methods
    //

    // javadoc inherited from supertype
    public Object getKey() {
        return key;
    }

    // Note: close() method is already implemented by superclass.
}
