/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.mercury;

import java.io.IOException;
import java.util.NoSuchElementException;

import net.jini.core.event.RemoteEvent;

/** 
 * <code>EventLogIterator</code> provides an abstraction for accessing 
 * the events
 * stored on behalf of a particular registration.  The methods closely
 * resemble the <tt>java.util.iterator</tt> interface with the 
 * exception of the <code>throws</code> clauses and the additional 
 * <tt>destroy</tt> and <tt>add</tt> methods.
 *
 * The semantics for this iterator are the same as <tt>java.util.Iterator</tt>
 * in that <tt>next</tt> and <tt>remove</tt> are intended to be called
 * in pairs.
 *
 * @since Jini1.1
 */
public interface EventLogIterator {
    /** 
     * Store the given <tt>RemoteEvent</tt> event.
     * @throws IOException if an I/O errors ocurs
     */
    void add(RemoteEvent re) throws IOException;

    /**  
     * Return true if there are any events in the collection and 
     * false otherwise.
     * @throws IOException if an I/O errors ocurs
     */
    boolean hasNext() throws IOException;

    /** 
     * Return the next event in the collection. 
     * @throws IOException if an I/O errors ocurs
     * @throws NoSuchElementException if there are no available
     *    <code>RemoteEvent</code>s
     * @throws ClassNotFoundException if there was a problem deserializing
     *    the stored <code>RemoteEvent</code>
     */
    RemoteEvent next() throws IOException, NoSuchElementException,
			      ClassNotFoundException;

    /** 
     * Remove the event at the iterator's current cursor position.
     * It is expected that the cursor position will be updated to 
     * point to the next unread event object, if any, upon return 
     * from this method.
     * @throws IOException if an I/O errors ocurs
     */
    void remove() throws IOException;

    /** 
     * Destroy the collection of stored events. 
     * @throws IOException if an I/O errors ocurs
     */
    void destroy() throws IOException;
}

