/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.mercury;

import com.sun.jini.proxy.UUID;

import java.io.Serializable;

/**
 * This class works in conjunction with the <code>ListenerProxy</code>
 * class in order to facilitate smart proxy functionality. 
 * Prior to the Java 2 SDK, version 1.2.2, RMI semantics would attempt 
 * to substitute the associated
 * "stub" class for any Remote object being marshalled.  This behavior would
 * not allow the passing of state information (i.e. <code>registrationID</code>
 * for <code>notify</code> invocations.  
 *
 * The <tt>ListenerProxy</tt> class, instead, implements the
 * <tt>writeReplace</tt> method, which then substitutes an instance of this 
 * class into the marshalling stream. 
 * 
 * The <tt>ListenerProxyRep</tt>, in turn, implements the <tt>readResolve</tt>
 * method in order to reconstitute a <tt>ListenerProxy</tt> object when it is 
 * unmarshalled.
 *
 * This "trick" essentially allows us to pass a smart proxy through
 * third party objects while maintaining the RemoteEventListener interface
 * contract (which could not occur using default RMI semantics).
 *
 * @since Jini1.1
 */
class ListenerProxyRep implements Serializable {

    /**
     * The reference to the event mailbox service implementation
     * 
     * @serial
     */
    private final MailboxBackEnd server;

    /**
     * The proxy's <code>UUID</code>
     *
     * @serial
     */
    private final UUID registrationID;

    /** Simple constructor */
    ListenerProxyRep(MailboxBackEnd mb, UUID regID) {
	server = mb;
	registrationID = regID;
    }

    /** 
     * Called by <tt>ObjectInputStream</tt> to allow 
     * this class to substitute a <tt>ListenerProxy</tt> into
     * the serialization stream during unmarshalling.
     */
    private Object readResolve() {
        return new ListenerProxy(server, registrationID);
    }
}


