/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.mercury;

import com.sun.jini.lease.landlord.Landlord;
import com.sun.jini.mahout.KillVMAdmin;
import com.sun.jini.proxy.ThrowThis;
import com.sun.jini.proxy.UUID;

import java.rmi.Remote;
import java.rmi.RemoteException;

import net.jini.core.event.RemoteEventListener;
import net.jini.core.event.RemoteEvent;
import net.jini.core.event.UnknownEventException;
import net.jini.event.MailboxRegistration;
import net.jini.admin.Administrable;

/**
 * MailboxBackEnd defines the private protocol between the various client-side
 * proxies and the event mailbox server.
 * <p>
 * The declared methods are pretty straightforward mappings of the
 * <tt>EventMailbox</tt> and <tt>MailboxRegistration</tt> interfaces.
 * <p>
 * Note: The <tt>Landlord</tt> interface extends <tt>Remote</tt>, 
 * which implicitly makes this interface Remote as well. 
 *
 * @since Jini1.1
 */
interface MailboxBackEnd extends Landlord, Administrable, MailboxAdmin, 
				 KillVMAdmin 
{

    /** Property used for generating debugging information */
    String DEBUG_PROP = "com.sun.jini.mercury.debug";

    /**
     * Returns the unique identifier generated (or recovered) by the back-end
     * implementation of the event mailbox service when an instance of
     * that service is constructed. This ID is typically used to determine
     * equality between the proxies of any two instances of the event
     * mailbox service.
     * 
     * @return the unique ID that was generated (or recovered) by the
     *         back-end implementation of the event mailbox service
     *         at creation time
     * 
     * @throws java.rmi.RemoteException typically, this exception occurs when
     *         there is a communication failure between the client and the
     *         event mailbox service
     */
    public UUID getProxyID() throws RemoteException;

    /**
     * Register with the event mailbox service for the requested duration.
     *
     * @see net.jini.event.EventMailbox#register
     */
    MailboxRegistration register(long leaseDuration)
	throws RemoteException;

    /**
     * Enable delivery of events for the given registration
     * to the specified target
     *
     * @param uuid The unique registration identifier
     *
     * @param target The designated delivery target for event notifications
     *
     * @see net.jini.event.MailboxilboxRegistration#enableDelivery
     */
    public void enableDelivery(UUID registrationID, RemoteEventListener target) 
	throws RemoteException, ThrowThis;
	
    /**
     * Disable delivery of events for the given registration
     *
     * @param uuid The unique registration identifier
     *
     * @see net.jini.event.MailboxilboxRegistration#disableDelivery
     */
    public void disableDelivery(UUID registrationID) 
	throws RemoteException, ThrowThis;

    /**
     * Collect remote events for the associated registration.
     *
     * @param uuid The unique registration identifier
     *
     * @param theEvent The event to store and/or forward
     *
     * @see net.jini.core.event.RemoteEventListener#notify
     */
    public void notify(UUID registrationID, RemoteEvent theEvent) 
	throws UnknownEventException, RemoteException, ThrowThis;
}
