/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.mercury;

import com.sun.jini.proxy.UUID;

import java.io.Serializable;
import java.rmi.RemoteException;

import net.jini.admin.Administrable;
import net.jini.core.lease.Lease;
import net.jini.event.EventMailbox;
import net.jini.event.MailboxRegistration;

/**
 * A MailboxProxy is a proxy for the event mailbox service.  
 * This is the object passed to clients of this service.
 * It implements the <code>EventMailbox</code> and the 
 * <code>Administrable</code> interfaces.
 *
 * @since Jini1.1
 */
class MailboxProxy implements EventMailbox, Administrable, Serializable {

    /**
     * The reference to the event mailbox service implementation
     *
     * @serial
     */
    private final MailboxBackEnd server;

    /**
     * The proxy's <code>UUID</code>
     *
     * @serial
     */
    private final UUID proxyID;

    /** Convenience constructor. */
    public MailboxProxy(MailboxBackEnd server, UUID proxyID) {
	this.server = server;
	this.proxyID = proxyID;
    }

    // inherit javadoc from parent
    public MailboxRegistration register(long duration) throws RemoteException {
        // Check for a bad argument
        // Note that -1 (i.e. Lease.ANY) is a valid request
        if (duration < 1 && duration != Lease.ANY)
            throw new IllegalArgumentException(
                "Duration values must be positive");
        return server.register(duration);
    }

    // inherit javadoc from parent
    public Object getAdmin() throws RemoteException {
        return server.getAdmin();
    }

    /** Proxies for servers with the same proxyID have the same hash code. */
    public int hashCode() {
	return proxyID.hashCode();
    }

    /** 
     * Proxies for servers with the same <code>proxyID</code> are 
     * considered equal. 
     */
    public boolean equals(Object o) {
        if (this == o)
            return true;

        if (o == null)
            return false;

        if (o.getClass() != getClass())
            return false;

        MailboxProxy mp = (MailboxProxy)o;
	return proxyID.equals(mp.proxyID);
    }

}
