/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.mercury;

import java.io.IOException;
import com.sun.jini.mahout.ExecUtilBase;

/**
 * Utility class for starting a Mercury service instance
 */
public class MercuryExecUtil extends ExecUtilBase {

    /**
     * The path of the executable jar file for starting the Norm Service 
     */
    String mercuryJarFile = null;

    /** 
     * Change codebase, classpath, and/or policyFilePath based on the
     * appropriate <code>com.sun.jini.exec.mercury.*</code> properties that
     * have been set.
     */
    private void constructorUtil() {
	final String mercuryCodebase = 
	    System.getProperty("com.sun.jini.exec.mercury.codebase");

	if (codebase == null && mercuryCodebase == null)
	    throw new IllegalStateException("MercuryExecUtil:Must specify " +
	         "a codebase using com.sun.jini.exec.mercury.codebase or " +
		 "com.sun.jini.exec.codebase");
	
	if (mercuryCodebase != null) codebase = mercuryCodebase;


        mercuryJarFile = 
	    System.getProperty("com.sun.jini.exec.mercury.class.path");

	if (classpath == null && mercuryJarFile == null)
	    throw new IllegalStateException("MercuryExecUtil:Must specify " +
	         "a classpath using com.sun.jini.exec.mercury.class.path or " +
		 "com.sun.jini.exec.class.path.\n" +
		 "Note: currently the class path is really the name of the\n" +
		 "      executable jar file (mercury.jar)");
	
	if (mercuryJarFile == null) 
	    mercuryJarFile = classpath;

	final String mercuryPolicy = 
	    System.getProperty("com.sun.jini.exec.mercury.policy");

	if (policyFilePath == null && mercuryPolicy == null)
	    throw new IllegalStateException("MercuryExecUtil:Must specify " +
	         "a policy file using com.sun.jini.exec.mercury.policy or " +
		 "com.sun.jini.exec.policy");
	
	if (mercuryPolicy != null) policyFilePath = mercuryPolicy;
    }

    /**
     * Set <code>cmd</code>
     * @param groups      Lookup groups the service should bind into. Ignored
     *                    if <code>useRegistry</code> is <code>true</code>.
     */
    private void buildCmd(String groups)
    {
	cmd += "-Djava.security.policy=" + policyFilePath  + " ";
	cmd += "-Djava.class.path=" + classpath + " ";
	cmd += "-jar " + mercuryJarFile + " " + codebase + " " + 
	    policyFilePath + " " + persistencePath + " " + groups;
    }

    /**
     * Construct a MercuryExecUtil for an event mailbox that will bind itself
     * into a lookup service
     *
     * @param groups Lookup group list
     * @throws java.io.IOException if an I/O error occurs
     */
    public MercuryExecUtil(String groups) throws IOException 
    {
	super("Mercury");
	constructorUtil();
	buildCmd(groups);
    }
}
