/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.mercury;

/**
 * Class that serves the purpose of an enumeration type. It's used 
 * in conjunction with <tt>StreamKey</tt> in order to distinguish among 
 * different <tt>LogStream</tt> objects that can potentially act upon the
 * same underlying log file.
 *
 * @since Jini1.1
 */
class StreamType {

    /**
     * Simple constructor. Private protection is used in order to prevent
     * external instantiation of this class type.
     */
    private StreamType() {;}

    /** Event data output stream enumeration */
    static final StreamType OUTPUT = new StreamType();

    /** Event data input stream enumeration */
    static final StreamType INPUT = new StreamType();

    /** 
     * Control data stream enumeration. Control data will not use
     * separate input/output data streams and therefore no distinction
     * is made.
     */
    static final StreamType CONTROL = new StreamType();
}
