/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.norm;

import java.rmi.server.RMIClassLoader;
import net.jini.core.lease.Lease;


/**
 * Simple wraper for client lease objects that implements (primarly) equals
 * and hashcode.  We use objects of this class as keys in the leaseTable
 * so client leases are not passed to each other unless they are from the
 * same codebase.
 */
final class EqualBox {
    /** The lease we are fronting for */
    final private Lease clientLease;

    /** The annotation of clientLease */
    final String annotation;

    /**
     * Simple constructor
     * @param clientLease the lease this object should front for
     */
    EqualBox(Lease clientLease) {
	this.clientLease = clientLease;
	
	final Class clc = clientLease.getClass();
	annotation = RMIClassLoader.getClassAnnotation(clc);
    }

    /** 
     * Check the codebase of the lease wrapped by the passed
     * EqualBox to see if matches the codebase of the lease
     * wrapped by this EqualBox
     */
    final boolean isCodebaseMatch(EqualBox that) {
	return annotation.equals(that.annotation);
    }

    // Inherit java doc from super type
    public final boolean equals(Object other) {
	if (!(other instanceof EqualBox)) 
	    return false;

	final EqualBox that = (EqualBox)other;
	return isCodebaseMatch(that) && 
	    clientLease.equals(that.clientLease);
    }

    // Inherit java doc from super type
    public final int hashCode() {
	return clientLease.hashCode();
    }
}
    
