/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.norm;

import net.jini.core.lease.Lease;
import net.jini.lease.LeaseListener;
import net.jini.lease.LeaseRenewalEvent;

/**
 * Object that transfers events from the Lease Renewal Manager to the
 * rest of Norm Server.  Isolates the renewal manager from having to
 * block on the snap shot locks.
 */
class FailureNotifer extends Thread implements LeaseListener {
    /** Ref to the main server object which has all the top level methods */
    final private NormServerImpl server;

    /**
     * Queue we use to decouple the reception of events from the lease
     * renewal manager from the scheduling of the sending of remote
     * events and modifying our internal table s (which both require
     * obtaining serious locks.
     */
    final private Queue queue = new Queue();

    /**
     * Simple constructor
     * @param server Object that will make the will make the actual
     * internal updates and schedule the sending of remote events.
     */
    FailureNotifer(NormServerImpl server) {
	super("Failure Notifier");
	setDaemon(true);
	this.server = server;	
    }

    //////////////////////////////////////////////////
    // Methods required by the LeaseListener interface 

    // Inherit java doc from super type
    public void notify(LeaseRenewalEvent e) {	
	// Paranoia, check to make sure that lease is one of wrapped
	// client lease...if its not ignore the event
	final Lease l = e.getLease();
	if (l instanceof ClientLeaseWrapper) {
	    ClientLeaseWrapper clw = (ClientLeaseWrapper)l;
	    queue.enque(clw);
	}
    }

    public void run() {
	// Loop taking items off the queue and pass them to the server
	while (true) 
	    try {
		final ClientLeaseWrapper clw 
		    = (ClientLeaseWrapper) queue.deque();
		server.renewalFailure(clw);
	    } catch (InterruptedException e) {
		// Some wants this tread dead, just return
		return;
	    } catch (RuntimeException e) {
		System.err.println("Failure Notifier received the following " +
				   "exception:");
		e.printStackTrace();
		System.err.println("When processing a renewal failure, " +
				   "trying to go on");
	    } catch (Error e) {
		System.err.println("Failure Notifier received the following " +
				   "exception:");
		e.printStackTrace();
		System.err.println("When processing a renewal failure, " +
				   "trying to go on");
	    }
    }
}
