/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.norm;

import java.util.Map;
import com.sun.jini.proxy.UUID;

/**
 * Base class for logged operations that do not create or destroy a set
 */
abstract class LeaseSetOperation extends LoggedOperation {
    /**
     * Simple constructor
     * @param setID The <code>UUID</code> for the set this operation is on
     */
    protected LeaseSetOperation(UUID setID) {
	super(setID);
    }

    // Inherit java doc from super type
    void apply(Map setTable) throws StoreException {
	final LeaseSet set = (LeaseSet)setTable.get(setID);
	if (set != null) {
	    apply(set);
	} else {
	    throw new CorruptedStoreException("Asked to update set " +
	        setID + " but we have no record of that set");
	}
    }

    /**
     * Update the state of the passed <code>LeaseSet</code> to reflect the
     * state of the server after this operation was performed.
     * @throws StoreException if there is a problem applying the update
     */
    abstract void apply(LeaseSet set) throws StoreException;
}

