/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.norm;

import java.io.Serializable;
import java.util.Map;
import com.sun.jini.proxy.UUID;

/**
 * Base class for the objects Norm logs as delta for each state
 * changing operation.
 */
abstract class LoggedOperation implements Serializable {
    /** 
     * The <code>UUID</code> of the set this operation was on
     * @serial
     */
    protected UUID setID;

    /**
     * Simple constructor
     * @param setID The <code>UUID</code> for the set this operation is on
     */
    protected LoggedOperation(UUID setID) {
	this.setID = setID;
    }

    /**
     * Update state of the passed <code>Map</code> of
     * <code>LeaseSet</code>s to reflect the state of server after
     * this operation was performed.
     * @throws StoreException if there is a problem applying the update
     */
    abstract void apply(Map setTable) throws StoreException;
}


