/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.norm;

import java.rmi.Remote;
import java.rmi.RemoteException;
import net.jini.admin.JoinAdmin;
import com.sun.jini.admin.DestroyAdmin;

/**
 * Administrative interface for Norm servers
 */
public interface NormAdmin extends JoinAdmin, DestroyAdmin {
    /**
     * Change the weight factor applied by the Norm server to the 
     * snapshot size during the test to determine whether or not to
     * take a "snapshot" of the server's state.
     *
     * @param weight weight factor for snapshot size
     */
    public void setSnapshotWeight(float weight) throws RemoteException;

    /**
     * Retrieve the weight factor applied by the Norm server to the 
     * snapshot size during the test to determine whether or not to
     * take a "snapshot" of the server's state.
     */
    public float getSnapshotWeight() throws RemoteException;

    /**
     * Change the value of the size threshold of the snapshot; which
     * is employed by the Norm server in the test to determine whether
     * or not to take a "snapshot" of the server's state.
     *
     * @param threshold size threshold for taking a snapshot */
    public void setLogToSnapshotThreshold(int threshold) 
	throws RemoteException;

    /**
     * Retrieve the value of the size threshold of the snapshot; which is
     * employed by the Norm server in the test to determine whether
     * or not to take a "snapshot" of the server's state.
     */
    public int getLogToSnapshotThreshold() throws RemoteException;
}

