/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.norm;

import java.io.IOException;
import com.sun.jini.mahout.ExecUtilBase;

/**
 * Utility class for starting Norm Lease Renewal Servers
 */
public class NormExecUtil extends ExecUtilBase {

    /**
     * The path of the executable JAR file for starting the Norm service 
     */
    String normJarFile = null;

    /** 
     * Change codebase, classpath, and/or policyFilePath the
     * appropriate <code>com.sun.jini.exec.norm.*</code> property has
     * been set
     */
    private void constructorUtil() {
	final String normCodebase = 
	    System.getProperty("com.sun.jini.exec.norm.codebase");

	if (codebase == null && normCodebase == null)
	    throw new IllegalStateException("NormExecUtil:Must specify " +
	         "a codebase using com.sun.jini.exec.norm.codebase or " +
		 "com.sun.jini.exec.codebase");
	
	if (normCodebase != null) codebase = normCodebase;


	normJarFile = 
	    System.getProperty("com.sun.jini.exec.norm.class.path");

	if (classpath == null && normJarFile == null)
	    throw new IllegalStateException("NormExecUtil:Must specify " +
	         "a classpath using com.sun.jini.exec.norm.class.path or " +
		 "com.sun.jini.exec.class.path\n" +
		 "Note: currently the class path is really the name of the\n" +
		 "      executable jar file (mercury.jar)");
	
	if (normJarFile == null) 
	    normJarFile = classpath;

	final String normPolicy = 
	    System.getProperty("com.sun.jini.exec.norm.policy");

	if (policyFilePath == null && normPolicy == null)
	    throw new IllegalStateException("NormExecUtil:Must specify " +
	         "a policy file using com.sun.jini.exec.norm.policy or " +
		 "com.sun.jini.exec.policy");
	
	if (normPolicy != null) policyFilePath = normPolicy;
    }

    /**
     * Set <code>cmd</code>
     * @param groups      Lookup groups the service should bind into. Ignored
     *                    if <code>useRegistry</code> is <code>true</code>.
     */
    private void buildCmd(String groups)
    {
	cmd += "-Djava.security.policy=" + policyFilePath  + " ";
	cmd += "-Djava.class.path=" + classpath + " ";
	cmd += "-jar " + normJarFile + " " + codebase + " " + 
	    policyFilePath + " " + persistencePath + " " + groups;
    }

    /**
     * Build an NormExecUtil that will bind itself
     * into a lookup service
     *
     * @param groups Lookup group list
     */
    public NormExecUtil(String groups) throws IOException 
    {
	super("Norm");
	constructorUtil();
	buildCmd(groups);
    }
}
