/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.norm;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.MarshalledObject;

import net.jini.admin.Administrable;
import net.jini.core.lease.Lease;
import net.jini.core.event.EventRegistration;
import net.jini.core.event.RemoteEventListener;
import net.jini.lease.LeaseRenewalService;

import com.sun.jini.lease.landlord.Landlord;
import com.sun.jini.proxy.UUID;
import com.sun.jini.proxy.ThrowThis;

import com.sun.jini.mahout.KillVMAdmin;

/**
 * This interface is the private wire protocol to that the various
 * proxy objects created by a Norm server (the lease renewal service
 * itself, sets, leases, admins, etc.) use to communicate back to the
 * server.
 */
public interface NormServer extends Landlord, LeaseRenewalService, 
    Administrable, NormAdmin, KillVMAdmin, Remote 
{
    /**
     * If calling <code>setExpirationWarningListener</code> with a
     * <code>null</code> listener this is the value that should be passed 
     */
    final static long NO_LISTENER = -1;

    /**
     * Add a lease to a set.
     * @param id What set the lease should be added to.
     * @param leaseToRenew the lease to be added to the set
     * @param membershipDuration how long the lease should be in the set
     * @param renewDuration how long the lease should be renewed for
     *                      each time it is renewed
     * @throws ThrowThis when another exception has to be thrown by the proxy
     * @throws java.rmi.RemoteException */
    public void renewFor(UUID id, Lease leaseToRenew,
			 long membershipDuration, long renewDuration)
	throws RemoteException, ThrowThis;

    /**
     * Remove a lease from a set
     * @param id of set is being operated on.
     * @param leaseToRemove the lease to be added to the set
     * @throws ThrowThis when another exception has to be thrown by the proxy
     * @throws java.rmi.RemoteException 
     */
    public Lease remove(UUID id, Lease leaseToRemove) 
	throws RemoteException, ThrowThis;


    /**
     * Return all the leases in the set.  Returns <code>null</code> 
     * or a zero-length array if there are no leases in the set.
     * @param id of set is being operated on.
     * @return A array of MarsahlledObjects, on for each lease
     * @throws ThrowThis when another exception has to be thrown by the proxy
     * @throws java.rmi.RemoteException 
     */
    public MarshalledObject[] getLeases(UUID id) 
	throws RemoteException, ThrowThis;

    /**
     * Set the expiration warning listener for a set.  Also used to 
     * cancel a registration.
     * @param id of set is being operated on.
     * @param listener Listener to be notified when this event occurs.
     *                 Pass <code>null</code> to clear the registration.
     * @param minWarning How long be for the lease on the set expires
     *                   should the event be sent. Ignored if
     *                   <code>listener</code> is <code>null</code>
     * @param handback   An object to be handed back to the listener
     *                   when the warning event occurs. Ignored if
     *                   <code>listener</code> is <code>null</code>
     * @return An <code>EventRegistration</code> object for the new
     * registration if <code>listener</code> is non-<code>null</code>
     * and <code>null</code> otherwise.
     * @throws ThrowThis when another exception has to be thrown by the proxy
     * @throws java.rmi.RemoteException
     */
    public EventRegistration setExpirationWarningListener(
			         UUID                id,
	                         RemoteEventListener listener, 
				 long                minWarning, 
				 MarshalledObject    handback)
	throws RemoteException, ThrowThis;

    /**
     * Set the renewal failure listener for a set.  Also used to 
     * cancel a registration.
     * @param id of set is being operated on.
     * @param listener Listener to be notified when this event occurs.
     *                 Pass <code>null</code> to clear the registration.
     * @param handback   An object to be handed back to the listener
     *                   when the failure event occurs.  Ignored if
     *                   <code>listener</code> is <code>null</code>
     * @return An <code>EventRegistration</code> object for the new
     * registration if <code>listener</code> is non-<code>null</code>
     * and <code>null</code> otherwise.
     * @throws ThrowThis when another exception has to be thrown by the proxy
     * @throws java.rmi.RemoteException      
     */
    public EventRegistration setRenewalFailureListener(
			         UUID                id,
	                         RemoteEventListener listener, 
				 MarshalledObject    handback)
	throws RemoteException, ThrowThis;


}


