/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.norm;

import java.util.List;
import java.util.Collections;

/**
 * Simple thread safe FIFO sequence
 */
class Queue {
    /** List we actually store stuff in */
    final private List lst = 
	Collections.synchronizedList(new java.util.LinkedList());

    /**
     * Enqueue an item
     */
    synchronized void enque(Object o) {
	lst.add(o);
	notifyAll();
    }

    /**
     * Block until the queue is non-empty and return the first item.
     * @throws InterruptedException if the current thread is interrupted
     *         while the queue is waiting for an item to appear.
     */
    synchronized Object deque() throws InterruptedException {
	while (lst.isEmpty()) 
	    wait();

	return lst.remove(0);
    }
}

