/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.norm;

import java.io.PrintStream;
import java.io.PrintWriter;

/**
 * Base class for exceptions thrown by PersistantStore
 */
class StoreException extends Exception {
    /**
     * The exception (if any) that trigged the store exception.  This
     * may be <code>null</code>.
     *
     * @serial
     */
    final Throwable nestedException;

    /**
     * Create a store exception, forwarding a string to the superclass
     * constructor.
     * @param str a detailed message for the exception
     */
    StoreException(String str) {
	super(str);
	nestedException = null;
    }

    /**
     * Create an exception, forwarding a string to the
     * superclass constructor.
     * @param str a detailed message for the exception
     * @param ex root cause for the exception, may be <code>null</code>
     */
    StoreException(String str, Throwable ex) {
	super(str);
	nestedException = ex;
    }

    /**
     * Print the stack trace of this exception, plus that of the nested
     * exception, if any.
     */
    public void printStackTrace() {
	printStackTrace(System.err);
    }

    /**
     * Print the stack trace of this exception, plus that of the nested
     * exception, if any.
     */
    public void printStackTrace(PrintStream out) {
	super.printStackTrace(out);
	if (nestedException != null) {
	    out.println("nested exception:");
	    nestedException.printStackTrace(out);
	}
    }

    /**
     * Print the stack trace of this exception, plus that of the nested
     * exception, if any.
     */
    public void printStackTrace(PrintWriter out) {
	super.printStackTrace(out);
	if (nestedException != null) {
	    out.println("nested exception:");
	    nestedException.printStackTrace(out);
	}
    }
}

