/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.norm.event;

import net.jini.core.event.RemoteEvent;
import net.jini.core.event.RemoteEventListener;

/**
 * Object associated with an <code>EventType</code> so it can ensure
 * that the lease on an event notification is still valid and notify
 * the client when a exception occurs in the course of attempting to
 * send an event.
 *
 * @see EventType
 * @see EventTypeFactory 
 */
public interface SendMonitor {
    /**
     * Method called when an attempt to send the event associated with
     * this object results in a definite exception
     * (e.g. <code>java.rmi.NoSuchObjectException</code> or any other
     * <code>Throwable</code> that is not a subclass of
     * <code>java.rmi.RemoteException</code>.
     * <p>
     * The caller will own no locks when calling this method.
     * @param type The object that generated the event
     * @param listener The listener the event is intended for.
     * @param ev   The remote event that could not be sent.
     * @param registrationNumber of the event registration that
     *             generated the exception.  This can be used
     *             to call <code>EventTypeclearListenerIfSequenceMatch</code>
     *             ensure only the registration that cause the problem 
     *             gets cleared.
     * @param t    The definite exception that caused us to give up
     *             sending the event.  
     */
    public void definiteException(EventType           type,
                                  RemoteEventListener listener,
                                  RemoteEvent         ev,
				  long                registrationNumber,
                                  Throwable           t);
    /**
     * Should return <code>true</code> if the lease associated with this
     * event is still valid and <code>false</code> otherwise.
     * <p>
     * The caller will own no locks when calling this method.
     */
    public boolean isCurrent();
}
