/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.outrigger;

import java.rmi.*;
import net.jini.core.entry.*;

/**
 * Interface for the iterators returned by the <code>contents()</code>
 * method of <code>JavaSpaceAdmin</code>.  Note
 * <code>AdminIterator</code>s do not survive restarts of the
 * underlying space.
 * <p>
 * @see JavaSpaceAdmin 
 */
public interface AdminIterator {
    /**
     * Return the next entry in the sequence.  Returns <code>null</code>
     * if there are no more matching entries in the space.
     * <p>
     * This method is idempotent in the face of <code>RemoteException</code>s.
     *
     * @throws UnusableEntryException if the field of next entry in
     * sequence can't be deserialized (usually this is because the class
     * in question could not be loaded).
     */
    public Entry next()	throws UnusableEntryException, RemoteException;

    /**
     * The effect of this call depends on the most recent call to
     * <code>next()</code>:
     * <ul>
     * <li> If the last call to <code>next()</code> returned an
     * <code>Entry</code> that entry will be removed from the space.
     * <p>
     * <li> If the last call to <code>next()</code> threw a
     * <code>UnusableEntryException</code> the <code>Entry</code> that
     * could not be deserialized will be removed from the space.
     * <p>
     * <li> If the last call to <code>next()</code> returned
     * <code>null</code>, threw a <code>RemoteException</code>, or
     * <code>next()</code> has not yet been called on this iterator a
     * <code>IllegalStateException</code> will be thrown and no entry will
     * be removed.
     * <p>
     * </ul>
     * This method is idempotent in the face of
     * <code>RemoteException</code>.
     * <p>
     * @throws IllegalStateException if <code>next()</code> has not be
     * called on this iterator, or the last invocation of
     * <code>next()</code> returned <code>null</code> or threw
     * <code>RemoteException</code>.     
     */
    public void delete() throws RemoteException;

    /**
     * Tell the server that this iterator is no longer in use.  All
     * operations on a closed iterator  have undefined results, except  
     * the <code>close()</code> method.
     * <p> 
     * This method is idempotent in the face of <code>RemoteException</code>.
     */
    public void close() throws RemoteException;
}
