/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.outrigger;

import net.jini.space.*;

import java.util.Vector;

/**
 * This interface is implemented by classes that know how to hold 
 * entries in a <code>BasicSpace</code>.  <code>BasicSpace</code>
 * partitions its data into exact class matches, and uses a separate
 * <code>EntryHolder</code> object to manage each type.
 *
 * @see BasicSpace
 * @see JavaSpace
 */
interface EntryHolder extends java.io.Serializable {
    /**
     * Return an <code>EntryRep</code> object that matches the given
     * template, or <code>null</code> if none does.
     *
     * @param tmplRep	The template to match against
     * @param takeIt	<code>true</code> if <code>hasMatch</code> should
     *			remove the matching entry.
     * @param conflict	If not <code>null</code>, a previous call to
     *			<code>hasMatch</code> in the same search threw
     *			<code>TransactionConflictException</code>, so
     *			this invocation should add the conflicting
     *			transactions to this object, and throw
     *			<code>conflict</code> as the
     *			<code>TransactionConflictException</code> if no
     *			non-conflicting match is found.  If
     *			<code>conflict</code> is <code>null</code>,
     *			then <code>hasMatch</code> will create a new
     *			<code>TransactionConflictException</code>
     *			object to throw that includes all the
     *			conflicting transactions.
     *
     * @see #match
     * @see TransactionConflictException
     */
    EntryRep hasMatch(EntryRep tmplRep, TransactableMgr txnMgr, boolean takeIt,
		      TransactionConflictException conflict)
	throws TransactionConflictException;

    /**

     * Check to see if the given <code>EntryRep</code> from this
     * holder matches the given template. This will only succeed if
     * the match is valid, including if it matches the given
     * template. If it is valid, the <code>EntryRep</code> will be returned.
     * Otherwise the result is the same as <code>hasMatch</code>.  In
     * the case of a conflict, the conflicting transactions will be added
     * to <code>conflict</code> if it is not <code>null</code>, and then
     * <code>match</code> will return <code>null</code>.  If there is a
     * conflict and the <code>conflict</code> parameter is <code>null</code>,
     * a new <code>TransactionConflictException</code> will be created that
     * contains the conflicting transactions and then the exception will
     * be thrown.  This behavior reduces the number of thrown exceptions,
     * which on many systems is an expensive operation.
     *
     * @see #hasMatch
     * @see TransactionConflictException
     */
    EntryRep match(EntryRep template, EntryRep rep, TransactableMgr txnMgr,
	 	   boolean takeIt, long time,
		   TransactionConflictException conflict)
	throws TransactionConflictException;

    /**
     * Add new new entry to the holder, returning an object defined by the
     * specific holder type.
     */
    Object add(EntryRep rep, TransactableMgr txnMgr);

    /**
     * Return an enumerator over the contents of this space that are visible
     * from the given mgr.
     */
    RepEnum contents(TransactableMgr mgr);

    /**
     * Take the given resource out of this holder.  Return
     * <code>false</code> if the specified entry wasn't in this
     * holder.
     */
    boolean cancel(LeaseDesc desc);

    /**
     * Reap the underlying FastList
     */
    void reap();

}
