/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.outrigger;

import net.jini.core.lease.Lease;
import com.sun.jini.lease.landlord.LeasedResource;

import COM.odi.util.OSHashtable;

/**
 * A set of <code>EntryHolder</code> objects for a given space.
 *
 * @author Ken Arnold
 *
 * @see EntryHolder
 * @see BasicSpace
 */
class EntryHolderSet {
    protected OSHashtable holders = new OSHashtable();
    // Map of LeaseDescs indexed by the cookie of the underlying
    // LeasedResource.
    protected OSHashtable idMap = new OSHashtable();

    /**
     * Return the <code>EntryHolder</code> object for the exact class
     * of the <code>Entry</code> ojbect held in <code>bits</code>.  If
     * one doesn't yet exist, it will be created.
     *
     * @see #holderFor(java.lang.String)
     */
    EntryHolder holderFor(EntryRep rep) {
	return holderFor(rep.classFor());
    }

    /**
     * Return the <code>EntryHolder</code> object for the exact class
     * with the given ID. If one doesn't yet exist, it will be created.
     *
     * @see #holderFor(EntryRep)
     */
    synchronized EntryHolder holderFor(String className) {
	synchronized (holders) {
	    EntryHolder holder = (EntryHolder) holders.get(className);
	    if (holder == null) {
		holder = newHolder(idMap, className);
		holders.put(className, holder);
	    }
	    return holder;
	}
    }

    synchronized java.util.Enumeration allHolders() {
	return holders.elements();
    }

    EntryHolder newHolder(OSHashtable idMap, String className) {
	return new SimpleEntryHolder(idMap, className);
    }

    public boolean cancel(Object id) {
	LeaseDesc desc = (LeaseDesc) idMap.get(id);
	if (desc == null)
	    return false;
	EntryHolder holder = holderFor(desc.classFor());
	return holder.cancel(desc);
    }

    LeasedResource getLeasedResource(Object cookie) {
	final Object o = idMap.get(cookie);
	if (o == null)
	    return null;
	return ((LeaseDesc)o).getLeasedResource();
    }

    /**
     * Force all of the holders to reap their <code>FastList</code>s
     */
    void reap() {
	final java.util.Enumeration e = allHolders();
	while (e.hasMoreElements()) {
	    final EntryHolder h = (EntryHolder)e.nextElement();
	    h.reap();
	}	    
    }
}

