/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.outrigger;

import net.jini.space.*;
import net.jini.core.entry.*;
import net.jini.core.transaction.*;
import com.sun.jini.mahout.RegistryAdmin;
import com.sun.jini.admin.DestroyAdmin;
import net.jini.admin.JoinAdmin;

import java.rmi.*;

/**
 * This interface contains all the administrative methods that Outrigger
 * provides to control its JavaSpaces<sup><font size=-2>TM</font></sup> 
 * service.
 */
public interface JavaSpaceAdmin extends JoinAdmin, DestroyAdmin, RegistryAdmin {
    /**
     * Return the space that this administrative object governs.
     */
    JavaSpace space() throws RemoteException;

    /**
     * Return an <code>AdminIterator</code> that will iterate over all
     * the entries in the space that match the given template and are
     * visible under the given transaction.
     * <p>
     * The interactions between other operations on the space and
     * the returned iterator are undefined
     * <p>
     * @param tmpl The iterator should return only entries that match
     *             tmpl
     * @param txn  The iterator should return only entries that match
     *             this transaction
     */
    AdminIterator contents(Entry tmpl, Transaction txn)
	throws TransactionException, SecurityException, RemoteException;

    /**
     * Return an <code>AdminIterator</code> that will iterate over all
     * the entries in the space that match the given template and are
     * visible under the given transaction.
     * <p>
     * The interactions between other operations on the space and
     * the returned iterator are undefined
     * <p>
     * @param tmpl The iterator should return only entries that match
     *             tmpl
     * @param txn  The iterator should return only entries that match
     *             this transaction
     * @param fetchSize advice on how many entries to fetch when the iterator
     *             has to go to the server for more entries.
     * 
     */
    AdminIterator contents(Entry tmpl, Transaction txn, int fetchSize)
	throws TransactionException, SecurityException, RemoteException;
}
