/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.outrigger;

import net.jini.core.transaction.Transaction;
import net.jini.core.event.EventRegistration;
import com.sun.jini.mahout.StorableObject;

import java.io.*;
import java.util.Vector;
import java.rmi.MarshalledObject;

/**
 * Methods that log an operation.  This are used on both ends of the
 * log -- when writing to the log and by the driver that reads from
 * the log to signal the reading process which operation has occurred.
 */
public interface LogOps {
    /**
     * Log a server boot (first time start or any reactivation).
     */
    void bootOp(long time);

    /**
     * Log a <code>write</code> operation.
     */
    void writeOp(EntryRep entry, Long transactionID);

    /**
     * Log a <code>take</code> operation.
     */
    void takeOp(long id, Long transactionID);

    /**
     * Log a <code>notify</code> operation.  Notifications under transactions
     * are lost at the end of the transaction, so the only ones that are logged
     * are those that are under no transaction.
     */
    void registerOp(EntryRep tmpl, NotifyChit chit);

    /**
     * Log an increase in the sequence number.
     */
    void notifiedOp(long chitID, long newSequence);

    /**
     * Log a <code>renew</code> operation.  We use the expiration, not
     * the extension, because we don't want to calculate the expiration
     * relative to when we read the log -- we want to use the exact expiration
     * granted.
     */
    void renewOp(Long id, long expiration);

    /**
     * Log a <code>cancel</code> operation, including transaction aborts.
     */
    void cancelOp(Long id);

    /**
     * Log a transaction <code>prepare</code>.
     */
    void prepareOp(Long trID, StorableObject transactionID);

    /**
     * Log a transaction <code>commit</code>.
     */
    void commitOp(Long transactionID);
}
