/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.outrigger;

import net.jini.core.lookup.*;
import net.jini.discovery.*;
import net.jini.core.entry.*;
import net.jini.lookup.entry.Name;
import com.sun.jini.mahout.Locator;

import java.net.DatagramPacket;
import java.net.MulticastSocket;
import java.net.Socket;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.AccessException;
import java.util.Iterator;




/**
 * A <code>LookupFinder</code> implements the
 * methods needed locate a service in a Jini(tm) Lookup service.
 *
 * @author Ken Arnold
 * @author Gary Holness
 *
 * @see Finder
 */
public class LookupFinder extends Finder {
    private MulticastSocket sock;
    private DatagramPacket dgram;
    ServiceRegistrar lookup;
    private int retry;

    private static final boolean DEBUG = false;
    private static final int MAX_DISCOVERY_RETRY = 10;
    private static final byte PACKET_TTL = 100;

    //Allow us to easily choose whether or not we want
    //to use the supplied means of locating the lookup
    //within the djinn.

    /**
     * Create a new <code>LookupFinder</code> object
     */
    public LookupFinder() {
    }

    /**
     * Using the Jini lookup service returned by <code>locator</code>
     * find the service registered with a
     * <code>net.jini.lookup.entry.Name</code> attribute who's value
     * is <code>name</code>.  If no service is registered under the
     * specified name retry until such a service appears.  <p> This
     * method returns null if the lookup service can not be contacted.
     */
    public Object find (Locator locator, String name) {
	Object tmpobj = null;

	try {
	    if (!(locator instanceof DiscoveryLocator))
	        throw new ClassCastException("LookupFinder: find:  " +
	        			"locator must be a LookupLocator");

            //by casting to ServiceRegistrar here, we
            //are implicitly checking if what is eventually
            //returned by find() is really an instance of
            //ServiceRegistrar.

	    ServiceRegistrar registrar = (ServiceRegistrar)locator.locate();

	    Entry[] attrs = new Entry[1];
	    Name n = new Name();
	    n.name = name;
	    attrs[0] = n;

	    ServiceTemplate tmpl =
	    new ServiceTemplate(null, null, attrs);

	    if (DEBUG) {
	        System.out.println("LookupFinder: find: name = " + name);
	        System.out.println("LookupFinder: find: registrar = " +
							    registrar);
	        System.out.println("LookupFinder: find: tmpl = " + tmpl);
	    }

	    do {
	        tmpobj = registrar.lookup(tmpl);
		if (tmpobj == null) {
		    try {
			System.out.println("waiting for " + name);
			Thread.sleep(2000);
		    } catch (Exception te) {
		    }
		}

	    } while (tmpobj == null);
	} catch (Exception e) {
	    System.out.println("LookupLocator: find: " + e.getMessage());
	    e.printStackTrace();
	}

	System.out.println("found " + name + " = " + tmpobj);
	return tmpobj;
    }
}
