/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.outrigger;

import COM.odi.*;
import COM.odi.util.*;

/**
 * Hold the values in the database for an <code>ODIStore</code>.  The
 * values held here are the values that the space needs to store in the
 * DB.  The <code>refresh</code> method is used to fetch one that was
 * created by <code>setupDB</code>.  They are not created directly
 * because this class manages the values to ensure they are all
 * initialized properly.
 *
 * @see ODIStore
 */
class ODIStoreDB {
    private static final String	ROOT = "ODIStoreDB";	// the root we use

    private static final java.io.PrintWriter debug =
	BasicSpace.debug.getWriter("odi");

    String		name;
    EntryHolderSet	contents;
    TemplateHolderSet	templates;
    TypeTree		types;

    /** Should use <code>setupDB</code> and/or <code>refresh</code>.  */
    private ODIStoreDB() { }

    /**
     * Set up the database for the first time with the given
     * parameters.  If the root already has values for name
     * the ones passed here are ignored in favor of those already
     * stored persistently.
     */
    static void setupDB(String name, ODIStoreDB root) {
						// make the thing exist
	if (name == null)			// don't overwrite what's there
	    root.name = name;
    }

    /**
     * Return a properly-initialized <code>ODIStoreDB</code> object,
     * with all fields set to their current values in the DB.  If any
     * of the values are <code>null</code>, they are set to a new
     * object of the appropriate type.
     */
    static ODIStoreDB refresh(Database db) {
	ODIStoreDB pref;
	try {
	    pref = (ODIStoreDB) db.getRoot(ROOT);
	    if (debug != null)
		debug.println("ODIStoreDB:refresh:root " + ROOT + " found");
	} catch (COM.odi.DatabaseRootNotFoundException e) {
	    pref = new ODIStoreDB();
	    db.createRoot(ROOT, pref);
	    if (debug != null)
		debug.println("creating " + ROOT);
	}

	if (pref.contents  == null) {
	    if (debug != null) debug.println("Creating contents");
	    pref.contents  = new EntryHolderSet();
	}
	if (pref.templates == null) {
	    if (debug != null) debug.println("Creating templates");
	    pref.templates = new TemplateHolderSet();
	}
	if (pref.types     == null) {
	    if (debug != null) debug.println("Creating types");
	    pref.types     = new TypeTree();
	}

	// it's okay for name to be null
	return pref;
    }
}
