/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.outrigger;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.AccessException;
import java.rmi.RMISecurityManager;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.Remote;
import java.rmi.NotBoundException;
import com.sun.jini.mahout.Locator;
import com.sun.jini.mahout.RegistryLocator;
import com.sun.jini.mahout.binder.RefHolder;



/**
 * A <code>RegistryFinder</code> implements the methods needed to locate
 * a service in a RMI registry.
 *
 * @author Ken Arnold
 * @author Gary Holness
 *
 * @see Finder
 */
public class RegistryFinder extends Finder {
    /**
     * Using the RMI registry returned by <code>locator</code> find
     * the service registered under <code>name</code>.  If no service is
     * registered under the specified name retry until such a service
     * appears.
     * <p> This method returns null if the registry can not
     * be contacted.
     */
    public Object find(Locator locator,String name) {
	Object tmpobj = null;

	try {
	    if (!(locator instanceof RegistryLocator))
		throw new ClassCastException("RegistryFinder: find: " +
			    "locator must be a RegistryLocator");
	    Registry registry = (Registry)locator.locate();

            //Create and install the security manager
            try {
                System.setSecurityManager(new RMISecurityManager());
            } catch (SecurityException se) {
                 //swallow exception if security manager
                 //has already been installed
            }

	    do {
		try {
		    tmpobj = registry.lookup(name);
		} catch (java.rmi.NotBoundException e) {
		    tmpobj = null;
		}

		if (tmpobj == null) {
		    try {
			System.out.println("waiting for " + name);
			Thread.sleep(2000);
		    } catch (Exception te) {
		    }
		}

	    } while (tmpobj == null);

	    if (tmpobj instanceof RefHolder)
		tmpobj = ((RefHolder)tmpobj).proxy();

	} catch (Exception e) {
	    System.out.println("RegistryLocator: find: " + e.getMessage());
	    e.printStackTrace();
	    if (e instanceof RuntimeException)
		throw (RuntimeException) e;
	}

	System.out.println("found " + name + " = " + tmpobj);

	return tmpobj;
    }
}
