/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.outrigger;

import java.rmi.Remote;
import java.rmi.RemoteException;

/**
 * Interface for private interaction between an
 * <code>AdminIterator</code> and its server-side partner.
 *
 * @see OutriggerAdmin#contents
 */
interface RemoteIter extends Remote {
    /** 
     * Fetch up to <code>max</code> <code>EntryRep</code> objects.
     * 
     * @param max Advice on the number of entries to return
     * @param id  ID of the last entry received by the caller.  If this
     *            does not match the ID of the last entry sent by the iterator
     *            will re-send that last batch in place of a new batch.
     *            May be <code>null</code> in which case a new batch
     *            will be sent.  The first call to <code>next()</code>
     *            should have <code>id</code> set to <code>null</code>
     */
    EntryRep[] nextReps(int max, Object id) throws RemoteException;

    /** Delete the given entry. */
    void delete(Object id) throws RemoteException;

    /** Forget about this iterator. */
    void close() throws RemoteException;
}
