/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.outrigger;

import java.io.File;
import java.io.Serializable;
import java.io.IOException;
import com.sun.jini.mahout.binder.BinderFactory;

/**
 * A <code>SpaceState</code> encapsulates the state needed for
 * reconstructing the state of an Outrigger 
 * JavaSpaces<sup><font size=-2>TM</font></sup> service.
 *
 * @author Ken Arnold
 * @author Gary Holness
 *
 * @see BasicSpace
 * @see TransientSpace
 */
public class SpaceState implements Serializable {
    static final long serialVersionUID = -2638456008842101942L;

    /**
     * The name of the space.
     *
     * @serial
     */
    private String name;

    /**
     * The path to the persistent store.
     *
     * @serial
     */
    private String path;

    /**
     * The factor for binders for this space.
     *
     * @serial
     */
    private BinderFactory binderFactory;

    /**
     * Create a new <code>SpaceState</code> object.
     * @param name Name of the space
     * @param path Path to the space's persistent store
     */
    public SpaceState(String name, String path) throws IOException {
	this.name = name;
	this.path = path;
    }

    /**
     * Returns the space's name
     */
    public String getName() {
	return name;
    }

    /**
     * Returns the path to the space's persistent store
     */
    public String getPath() {
	return path;
    }

    /**
     * Return the
     * <code>com.sun.jini.mahout.binder.BinderFactory</code> this
     * space is using to obtain a
     * <code>com.sun.jini.mahout.binder.Binder</code>
     * 
     * @see com.sun.jini.mahout.binder.Binder
     */
    public BinderFactory getBinderFactory() {
	return binderFactory;
    }

    /**
     * Set the <code>com.sun.jini.mahout.binder.BinderFactory</code>
     * that will be used when the service is activated to obtain the
     * <code>com.sun.jini.mahout.binder.Binder</code> that will
     * be used bind the service into directory services.
     */
    public void setBinderFactory(BinderFactory bf) {
	binderFactory = bf;
    }
}
