/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.outrigger;

/**
 * This interface defines the methods that any BasicSpace store must
 * implement.
 *
 * @see BasicSpace#initialize
 */
interface Store {
    /**
     * Set up whatever store we're using.
     */
    public void setupStore();

    /**
     * Start up a new transaction for this space.
     */
    public void startTxn();

    /**
     * Commit the current transaction if <code>commit</code> is true,
     * otherwise abort it.  It is harmless to invoke this if no
     * transaction is in progress -- this allows you to put a
     * <code>commitTxn</code> or <code>abortTxn</code> in a
     * <code>finally</code> clause without worry that it may get
     * invoked before the <code>startTxn</code> in the <code>try</code>
     * block happens.
     *
     * @see BasicSpace#commitTxn
     * @see BasicSpace#abortTxn
     */
    public void endTxn(boolean commit);

    /**
     * Return a reference to a version of this object usable in
     * transient space, e.g., for a return value of a remote method.
     * The returned reference may be the same as the original if no
     * change is needed.
     */
    public Object transientOf(Object orig);

    /**
     * Destroy any persistent state associated with this store.
     * The object will not be used after this call.
     */
    public void destroy();
}
