/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.outrigger;

/**
 * This interface is implemented by objects that represent operations
 * undertaken under a transaction.  These objects are governed by
 * a <code>TransactableMgr</code> object that manages the overall
 * transaction's state in this space.  The set of <code>Transactable</code>
 * objets managed by a <code>TransactableMgr</code> object constitute
 * the set of operations made under the transaction.
 *
 * @see TransactableMgr
 */
interface Transactable
    extends net.jini.core.transaction.server.TransactionConstants
{
    /**
     * Prepare to commit this object's part of the transaction.  Return
     * the prepare's status.
     *
     * @see TransactableMgr#prepare
     */
    int prepare(TransactableMgr mgr, BasicSpace space);

    /**
     * Commit this object's part of the transaction.  The
     * <code>space</code> is the <code>BasicSpace</code> on which the
     * operation happens -- some commit operations have space-wide side
     * effects (for example, a commit of a <code>write</code> operation
     * can cause event notifications for clients registered under the
     * transaction's parent).
     */
    void commit(TransactableMgr mgr, BasicSpace space);

    /**
     * Abort this object's part of the transaction.
     */
    void abort(TransactableMgr mgr, BasicSpace space);
}
