/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.outrigger;

/**
 * This interface is implemented by entities in the system that manage
 * a <code>BasicSpace</code> object's transaction state for a particular
 * transaction.  A <code>TransactableMgr</code> object has a list of
 * <code>Transactable</code> objects which represent operations performed
 * on the space under this transaction.
 *
 * @see BasicSpace
 * @see Transactable
 */
interface TransactableMgr extends TransactableConstants {
    /**
     * Return the original ServerTransaction object which this manager
     * handles.
     */
    net.jini.core.transaction.server.ServerTransaction transaction(BasicSpace space);

    /**
     * Add a new <code>Transactable</code> object to the list of transactable
     * operations managed by this object.
     */
    Transactable add(Transactable t);

    /*
     * Block until this transaction is in a stable state.  A "stable state"
     * means that it is active, committed, or aborted, and not in any
     * intermediate state between.
     */
    void waitUntilStable();
}
