/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.outrigger;

import java.util.Set;

/**
 * Signal that <code>hasMatch</code> found a matching entry that was
 * being used in a conflicting way by a transaction, and found no other
 * matching entry.
 *
 * @see EntryHolder#hasMatch
 */
class TransactionConflictException extends Exception {
    static final long serialVersionUID = 7416410537935234031L;

    /**
     * The list of transactions causing the conflict.  This is transient
     * because it should not cross the wire -- it only has meaning in the
     * server.
     */
    private transient java.util.Set	txns;

    /**
     * Return the list of transactions, allocating it if necessary.
     */
    Set txns() {
	if (txns == null)
	    txns = new java.util.HashSet();
	return txns;
    }
}
