/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.outrigger;

import java.io.IOException;
import java.rmi.RemoteException;
import java.rmi.MarshalledObject;
import java.rmi.activation.ActivationID;

/**
 * A transient <code>BasicSpace</code>, where all the data is stored in
 * transient memory, and lost when the server goes down.  All methods
 * are implemented to work concurrently.  This class provides a
 * <code>main</code> that sets the right flags for
 * <code>BasicSpace</code>'s <code>main</code>.
 *
 * @author Ken Arnold
 * @author Gary Holness
 *
 * @see BasicSpace
 * @see TransientSpace
 */
public class TransientSpace extends BasicSpace {
    private static final boolean DEBUG = false;

    /**
     * Create a transient type of <code>BasicSpace</code>.
     * It will bind into a Jini Lookup service.
     */
    public TransientSpace() throws RemoteException, IOException
    {
	if (dbgStartup != null)
	    dbgStartup.println("TransientSpace::TransientSpace()");
	initialize(newStore());
    }

    /**
     * Activation constructor for <code>TransientSpace</code>.
     * This is just a pass-through constructor for FrontEndSpace.
     */
    public TransientSpace(ActivationID activationID, MarshalledObject data)
	throws RemoteException, IOException, ClassNotFoundException
    {
	super(activationID, data);
    }

    /**
     * Return a new <code>Store</code> object for this new object.
     * Subclasses can override to change the store.  The default
     * implementation returns a new <code>TransientStore</code>.
     * <p>
     * <b>NOTE:</b> This method is called by the
     * <code>TransientSpace</code> constructors, and so your
     * implementation should not rely on your fields being initialized --
     * they won't have been yet.
     */
    Store newStore() {
	return new TransientStore(this);
    }
}
