/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.outrigger;

/**
 * Store for <code>TransientSpace</code>.  Allocates everything
 * anew.
 *
 * @see TransientSpace
 */
class TransientStore implements Store {
    private BasicSpace	space;		// the space I'm a store for

    /**
     * Create a new <code>TransientStore</code> for the given space.
     */
    TransientStore(BasicSpace space) {
	this.space = space;
    }

    /**
     * Allocate all the data in memory.
     */
    public void setupStore() {
	space.contents = new EntryHolderSet();
	space.types = new TypeTree();
	space.templates = new TemplateHolderSet();

	// This takes a while the first time, so let's get it going while
	// the activation completes
	new Thread() {
	    public void run() {
		BasicSpace.nextID();
	    }
	}.start();
    }

    /**
     * No transaction needed, so do nothing.
     */
    public void startTxn() {
    }

    /**
     * No transaction needed, so do nothing.
     */
    public void endTxn(boolean abort) {
    }

    /**
     * Every object is transient, so just return <code>obj</code>.
     */
    public Object transientOf(Object obj) {
	return obj;
    }

    /**
     * Not much to do here. We could <code>null</code> everything
     * out to orphan stuff for the garbage sooner, but we're about
     * to exit anyway
     */
    public void destroy() {
	// Do nothing.
    }
}
