/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.reggie;

import net.jini.core.lease.*;

/** The base class for lease proxies. */
abstract class RegistrarLease extends com.sun.jini.lease.AbstractLease {

    private static final long serialVersionUID = 1286538697644640310L;

    /**
     * The registrar.
     *
     * @serial
     */
    protected final Registrar server;
    /**
     * The internal lease id.
     *
     * @serial
     */
    protected final long leaseID;

    /** Simple constructor. */
    protected RegistrarLease(Registrar server, long leaseID, long expiration) {
	super(expiration);
	this.server = server;
	this.leaseID = leaseID;
    }

    /** Create a lease map. */
    public LeaseMap createLeaseMap(long duration) {
	return new RegistrarLeaseMap(this, duration);
    }

    /**
     * Two leases can be batched if they are both RegistrarLeases and
     * have the same server.
     */
    public boolean canBatch(Lease lease) {
	return (lease instanceof RegistrarLease &&
		server.equals(((RegistrarLease)lease).server));
    }

    /** Return the registrar. */
    Registrar getRegistrar() {
	return server;
    }

    /** Returns the lease ID */
    long getLeaseID() {
	return leaseID;
    }

    /** Returns the service ID, or the event ID as a Long */
    abstract Object getRegID();

    /** Set the expiration. */
    void setExpiration(long expiration) {
	this.expiration = expiration;
    }
}
