/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.reggie;

import net.jini.core.lease.*;
import net.jini.core.lookup.ServiceID;
import java.rmi.RemoteException;
import java.util.HashMap;
import java.util.Map;
import java.util.Iterator;

/**
 * The LeaseMap implementation class for registrar leases.  Clients only see
 * instances via the LeaseMap interface.
 */
class RegistrarLeaseMap extends com.sun.jini.lease.AbstractLeaseMap {

    private static final long serialVersionUID = 1840099461579576431L;

    /**
     * The registrar
     *
     * @serial
     */
    private final Registrar server;

    /** Simple constructor */
    public RegistrarLeaseMap(RegistrarLease lease, long duration) {
	super(lease, duration);
	this.server = lease.getRegistrar();
    }

    /** Any RegistrarLease from the same server can be in the map */
    public boolean canContainKey(Object key) {
	return (key instanceof RegistrarLease &&
		server.equals(((RegistrarLease)key).getRegistrar()));
    }

    // This method's javadoc is inherited from an interface of this class
    public void renewAll() throws LeaseMapException, RemoteException {
	int size = map.size();
	if (size == 0)
	    return;
	Object[] regIDs = new Object[size];
	long[] leaseIDs = new long[size];
	long[] durations = new long[size];
	int i = 0;
	for (Iterator iter = map.entrySet().iterator(); iter.hasNext(); i++) {
	    Map.Entry e = (Map.Entry)iter.next();
	    RegistrarLease ls = (RegistrarLease)e.getKey();
	    regIDs[i] = ls.getRegID();
	    leaseIDs[i] = ls.getLeaseID();
	    durations[i] = ((Long)e.getValue()).longValue();
	}
	RenewResults results = server.renewLeases(regIDs, leaseIDs, durations);
	long now = System.currentTimeMillis();
	HashMap emap = (results.exceptions != null) ?
	    	       new HashMap(2 * results.exceptions.length + 1) : null;
	i = 0;
	int j = 0;
	for (Iterator iter = map.entrySet().iterator(); iter.hasNext(); i++) {
	    Map.Entry e = (Map.Entry)iter.next();
	    long duration = results.durations[i];
	    if (duration >= 0) {
		((RegistrarLease)e.getKey()).setExpiration(duration + now);
	    } else {
		emap.put(e.getKey(), results.exceptions[j++]);
		iter.remove();
	    }
	}
	if (emap != null)
	    throw new LeaseMapException("lease renewal failures", emap);
    }

    // This method's javadoc is inherited from an interface of this class
    public void cancelAll() throws LeaseMapException, RemoteException {
	int size = map.size();
	if (size == 0)
	    return;
	Object[] regIDs = new Object[size];
	long[] leaseIDs = new long[size];
	int i = 0;
	for (Iterator iter = map.keySet().iterator(); iter.hasNext(); i++) {
	    RegistrarLease ls = (RegistrarLease)iter.next();
	    regIDs[i] = ls.getRegID();
	    leaseIDs[i] = ls.getLeaseID();
	}
	Exception[] exceptions = server.cancelLeases(regIDs, leaseIDs);
	if (exceptions == null)
	    return;
	i = 0;
	HashMap emap = new HashMap(13);
	for (Iterator iter = map.keySet().iterator(); iter.hasNext(); i++) {
	    RegistrarLease ls = (RegistrarLease)iter.next();
	    Exception ex = exceptions[i];
	    if (ex != null) {
		emap.put(ls, ex);
		iter.remove();
	    }
	}
	throw new LeaseMapException("lease cancellation failures", emap);
    }
}
