/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.reggie;

import java.rmi.RemoteException;
import net.jini.core.entry.Entry;
import net.jini.core.lookup.*;
import net.jini.core.lease.*;

/** Implementation class for the ServiceRegistration interface. */
class Registration implements ServiceRegistration, java.io.Serializable
{
    private static final long serialVersionUID = -276552004282140614L;

    /**
     * The registrar
     *
     * @serial
     */
    private final Registrar server;
    /**
     * The service lease
     *
     * @serial
     */
    private final ServiceLease lease;

    /** Simple constructor */
    public Registration(Registrar server, ServiceLease lease) {
	this.server = server;
	this.lease = lease;
    }

    // This method's javadoc is inherited from an interface of this class
    public ServiceID getServiceID() {
	return lease.getServiceID();
    }

    // This method's javadoc is inherited from an interface of this class
    public Lease getLease() {
	return lease;
    }

    // This method's javadoc is inherited from an interface of this class
    public void addAttributes(Entry[] attrSets)
	throws UnknownLeaseException, RemoteException
    {
	server.addAttributes(lease.getServiceID(),
			     lease.getLeaseID(),
			     EntryRep.toEntryRep(attrSets, true));
    }

    // This method's javadoc is inherited from an interface of this class
    public void modifyAttributes(Entry[] attrSetTmpls, Entry[] attrSets)
	throws UnknownLeaseException, RemoteException
    {
	server.modifyAttributes(lease.getServiceID(),
				lease.getLeaseID(),
				EntryRep.toEntryRep(attrSetTmpls, false),
				EntryRep.toEntryRep(attrSets, false));
    }

    // This method's javadoc is inherited from an interface of this class
    public void setAttributes(Entry[] attrSets)
	throws UnknownLeaseException, RemoteException
    {
	server.setAttributes(lease.getServiceID(),
			     lease.getLeaseID(),
			     EntryRep.toEntryRep(attrSets, true));
    }
}
