/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.reggie;

import net.jini.core.lease.*;
import net.jini.core.lookup.ServiceID;
import java.rmi.RemoteException;

/**
 * A ServiceLease is a proxy for a service registration lease at a registrar.
 * Clients only see instances via the Lease interface.
 */
class ServiceLease extends RegistrarLease {

    private static final long serialVersionUID = 4366637533663829830L;

    /**
     * The service id assigned at registration.
     *
     * @serial
     */
    private final ServiceID serviceID;

    /** Simple constructor. */
    ServiceLease(Registrar server,
		 ServiceID serviceID,
		 long leaseID,
		 long expiration)
    {
	super(server, leaseID, expiration);
	this.serviceID = serviceID;
    }

    public void cancel() throws UnknownLeaseException, RemoteException {
	server.cancelServiceLease(serviceID, leaseID);
    }

    /** Do the actual renew. */
    protected long doRenew(long duration)
	throws UnknownLeaseException, RemoteException
    {
	return server.renewServiceLease(serviceID, leaseID, duration);
    }

    /** Returns the service ID */
    ServiceID getServiceID() {
	return serviceID;
    }

    Object getRegID() {
	return serviceID;
    }

    /** Returns a hash code based on content. */
    public int hashCode() {
	return server.hashCode() ^ serviceID.hashCode() ^ (int)leaseID;
    }

    /** Equal if for the same serviceID and leaseID at the same registrar. */
    public boolean equals(Object obj) {
	if (!(obj instanceof ServiceLease))
	    return false;
	ServiceLease ls = (ServiceLease)obj;
	return (server.equals(ls.server) &&
		serviceID.equals(ls.serviceID) &&
		leaseID == ls.leaseID);
    }
}
