/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.reggie;

import java.rmi.server.RMIClassLoader;
import java.io.IOException;

/** A ServiceType annotated with a codebase. */
class ServiceTypeBase implements java.io.Serializable {
    /**
     * The ServiceType.
     *
     * @serial
     */
    public final ServiceType type;
    /**
     * The codebase.
     *
     * @serial
     */
    public String codebase;

    private static final long serialVersionUID = -2930430739238582517L;

    /** Simple constructor */
    public ServiceTypeBase(ServiceType type, String codebase) {
	this.type = type;
	this.codebase = codebase;
    }

    /** Sets the codebase to the codebase of the given class. */
    public void setCodebase(Class cls) {
	codebase = RMIClassLoader.getClassAnnotation(cls);
    }

    /**
     * Converts an array of ServiceTypeBase to an array of Class.  If a
     * class cannot be loaded, it is left as null.
     */
    public static Class[] toClass(ServiceTypeBase[] stypes)
    {
	Class[] classes = null;
	if (stypes != null) {
	    classes = new Class[stypes.length];
	    for (int i = stypes.length; --i >= 0; ) {
		try {
		    ServiceTypeBase stype = stypes[i];
		    classes[i] = stype.type.toClass(stype.codebase);
		} catch (Throwable e) {
		    RegistrarProxy.handleException(e);
		}
	    }
	}
	return classes;
    }
}
