/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.reggie;

import net.jini.core.lookup.*;
import java.rmi.RemoteException;

/**
 * A Template contains the fields of a ServiceTemplate packaged up for
 * transmission between client-side proxies and the registrar server.
 * Instances are never visible to clients, they are private to the
 * communication between the proxies and the server.
 * <p>
 * This class only has a bare minimum of methods, to minimize
 * the amount of code downloaded into clients.
 */
class Template implements java.io.Serializable {

    private static final long serialVersionUID = 5719167991752760177L;

    /**
     * ServiceTemplate.serviceID
     *
     * @serial
     */
    public ServiceID serviceID;
    /**
     * ServiceTemplate.serviceTypes converted to ServiceTypes
     *
     * @serial
     */
    public ServiceType[] serviceTypes;
    /**
     * ServiceTemplate.attributeSetTemplates converted to EntryReps
     *
     * @serial
     */
    public EntryRep[] attributeSetTemplates;

    /**
     * Converts a ServiceTemplate to a Template.  Any exception that results
     * is bundled up into a MarshalException.
     */
    public Template(ServiceTemplate tmpl) throws RemoteException {
	serviceID = tmpl.serviceID;
	serviceTypes = ClassMapper.toServiceType(tmpl.serviceTypes);
	attributeSetTemplates =
	    EntryRep.toEntryRep(tmpl.attributeSetTemplates, false);
    }
}
