/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */

package com.sun.jini.reliableLog;

import java.io.IOException;

/** 
 * This class can be used to represent all exceptional conditions that
 * occur during any logging process. Whenever an exception is caught 
 * while information is being logged, the exception can be wrapped
 * in this class so as to indicate an unsuccessful log operation.
 */
public class LogException extends IOException {

    private static final long serialVersionUID = 1870528169848832111L;

    /** @serial */
    public Throwable detail;

    /**
     * Create a wrapper exception for exceptions that occur during a logging
     * operation.
     */
    public LogException() {}

    /**
     * For exceptions that occur during a logging operation, create a wrapper
     * exception with the specified description string.
     */
    public LogException(String s) {
	super(s);
    }

    /**
     * For exceptions that occur during a logging operation, create a wrapper
     * exception with the specified description string and the specified
     * nested exception.
     */
    public LogException(String s, Throwable ex) {
	super(s);
	detail = ex;
    }

    /**
     * Produce the message; including the message from the nested exception
     * if there is one.
     */
    public String getMessage() {
	if (detail == null) 
	    return super.getMessage();
	else
	    return super.getMessage() + 
		"; nested exception is: \n\t" +
		detail.toString();
    }
}
