/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.start;

import net.jini.core.discovery.LookupLocator;
import java.util.Properties;

/**
 * This class serves as a data structure, (and associated data accessor
 * methods), which can be used to store and return the values of the arguments
 * extracted from the command line when starting a client or service.
 */
public class ParsedArgs {
    /** The RMI codebase with which to annotate all remote classes that are
     * downloaded from the server side of the entity associated with the
     * arguments represented in this class.
     */
    private String codebase;
    /** The security policy file for the entity associated with the arguments
     *  represented in this class.
     */
    private String policyFile;
    /** The directory in which the state of the entity associated with the
     *  arguments represented in this class will be recorded for persistence.
     */
    private String logDir;
    /** If the entity associated with the arguments represented in this class
     *  is a lookup service, this field contains the names of the groups
     *  to which that lookup service is to be a member; otherwise, this
     *  field contains the names of the groups whose members are the lookup
     *  services the entity should discover and join
     */
    private String[] groups;
    /** If the entity associated with the arguments represented in this class
     *  is a lookup service, this field typically will have no effect on
     *  the state of that lookup service; otherwise, this field contains
     *  references to the specific lookup services the entity should
     *  discover and join
     */
    private LookupLocator[] locators;

    /** Specifies which JVM implementation should be used for the backend
     *  server of the entity, associated with the arguments of this class,
     *  that will be started as an activatable service. When not
     *  <code>null</code>, the value of this field is an absolute path
     *  to an executable and that accepts all the standard JVM command-line
     *  options (for example, -D, -classpath, and so on), and directly or
     *  indirectly start a JVM. The value contained in this field is useful
     *  for testing such entities on alternate JVM implementations. 
     */
    private String javaProgram;
    /** Specifies the system properties with which to start the entity,
     *  associated with the arguments of this class, that will be started
     *  as an activatable service.
     */
    private Properties properties;
    /** Specifies the command options (for example -cp <classpath>) to
     *  use when starting the entity, associated with the arguments of
     *  this class, as an activatable service.
     */
    private String[] options;

    /* Constructs an instance of this data structure with the appropriate
     * values extracted from the command line of the client or service being
     * started.
     */
    public ParsedArgs(String          codebase,
                      String          policyFile,
                      String          logDir,
                      String[]        groups,
                      LookupLocator[] locators,
                      String          javaProgram,
                      Properties      properties,
                      String[]        options)
    {
        this.codebase    = codebase;
        this.policyFile  = policyFile;
        this.logDir      = logDir;
        this.groups      = groups;
        this.locators    = locators;
        this.javaProgram = javaProgram;
        this.properties  = properties;
        this.options     = options;
    }//end constructor

    /** Returns the <code>codebase</code> field of this class */
    public String getCodebase() {
        return codebase;
    }

    /** Returns the <code>policyFile</code> field of this class */
    public String getPolicyFile() {
        return policyFile;
    }

    /** Returns the <code>logDir</code> field of this class */
    public String getLogDir() {
        return logDir;
    }

    /** Returns the <code>groups</code> field of this class */
    public String[] getGroups() {
        return groups;
    }

    /** Returns the <code>locators</code> field of this class */
    public LookupLocator[] getLocators() {
        return locators;
    }

    /** Returns the <code>javaProgram</code> field of this class */
    public String getJavaProgram() {
        return javaProgram;
    }

    /** Returns the <code>properties</code> field of this class */
    public Properties getProperties() {
        return properties;
    }

    /** Returns the <code>options</code> field of this class */
    public String[] getOptions() {
        return options;
    }
}//end class ParsedArgs
