/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */

package com.sun.jini.system;

import java.lang.Long;
import java.lang.String;
import java.io.File;
import java.util.ArrayList;
import java.util.Observable;
import java.util.Observer;

/**
 * Observer class that gets notified when a change has been noted,
 * that is a file has been found, by an Observable object. 
 * This class is normally passed into a class that extends 
 * <code>Observable</code>
 * 
 * @see java.util.Observable
 * @see java.util.Observer
 * @see com.sun.jini.system.FileWalker
 */
public class FileObserver implements Observer {
    private ArrayList fileList = (new ArrayList());
    private ArrayList timeList = (new ArrayList());
    private File currentFile = null;
    private int lcv = 0;
    private String suffix;

    /**
     * Constructor that allows the user to specify interest in a 
     * particular suffix. Only files with the provide suffix 
     * will be saved and returned to the user. All others will fall
     * through.
     * <p>
     * If suffix is an empty string then all files will be matched.
     */
    public FileObserver(String suffix) {
	this.suffix = suffix;
    }

    /**
     * Called when an Observer monitors a change.
     * When a change is noted by the item being observed,
     * in this case the given root build directory,
     * this method gets called.
     */
    public void update(Observable o,Object arg) {
	currentFile = (File)arg;
	if (currentFile.getName().endsWith(suffix)) {
	    //System.out.println("found file:"+currentFile.getName());
	    fileList.add(lcv,currentFile.getAbsolutePath());
	    timeList.add(lcv,new Long(currentFile.lastModified()));
	    lcv++;
	}
    }	

    /**
     * Get the complete list of found files. The
     * list contains String representations of the file names.
     * 
     * @return ArrayList - List of found files.
     */
    public ArrayList getFileList() {
	return fileList;
    }

    /**
     * Get the time stamps of the found files. There is a
     * 1-1 correspondance with the contents of getFileList.
     * The lit contains Long representations of the file time stamps.
     *
     * @see #getFileList
     * @return ArrayList - List of found files date stamps
     */
    public ArrayList getFileTimeList() {
	return timeList;
    }
}


