/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.system;

import java.lang.String;

import java.io.File;

import java.util.Observable;

/**
 * Class to recursively traverse a given directory. Each time
 * a file is found the Observer object added to this one will
 * be called.
 */
public class FileWalker extends Observable {

    /**
     * Method to call that starts the directory traversal.
     * Make sure that a FileObserver has been added prior
     * to invoking this method.
     * 
     * @param file  The top of the directory to traverse.
     * @param includeDirectories   If true it will recursively
     *                             traverse the given directory.
     *                             If false, just do the files 
     *                             currently found in the given
     *                             directory.
     * 
     * @see java.util.Observable
     * @see java.util.Observer
     * @see com.sun.jini.system.FileObserver
     */
    public void walk (File file, boolean includeDirectories) {
	if (file.isDirectory()) {
	    if (includeDirectories) {
		setChanged();
		notifyObservers(file);
	    }
	    String[] filenames = file.list();
	    if (filenames != null) {
		for (int i=0; i<filenames.length; i++) {
		    walk(new File(file,filenames[i]),includeDirectories);
		}
	    }
	} else {
	    setChanged();
	    notifyObservers(file);
	}
    }
}



