/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.system;

import java.io.File;
import java.io.FileFilter;

import java.lang.String;

/**
 * Implementation of FilenameFilter to allow us to just
 * look for java files by default, or other file extensions if
 * the user wishes to use a different extension.
 */
public class JavaSourceFilter implements FileFilter {
    /**
     * The extension that we we want to look for.
     */
    private String fileExtension;

    /**
     * Create a filter for *.java files.
     */
    public JavaSourceFilter() {
	this(".java");
    }

    /**
     * Create a filter for a user defined extension.
     * If an empty string is passed then all files
     * will be matched.
     *
     * @param fileExtension  The extension the user is looking for.
     */
    public JavaSourceFilter(String fileExtension) {
	this.fileExtension = fileExtension;
    }

    /**
     * Method required by FilenameFilter interface.
     * It is called by File.list() to get a subset of
     * the files in the given directory.
     *
     * @param filename The current file we are looking at.
     */
    public boolean accept(File filename){
	return (filename.toString().endsWith(fileExtension));
    }
}




