/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.thread;

import java.io.*;

/**
 * <code>StreamPlugThread</code> is a utility class that "plugs" two streams,
 * one input stream and one output stream, together by creating a thread that
 * repeatedly reads any data available from the input stream and writes it to
 * the output stream.
 *
 * @author Peter Jones
 */
public class StreamPlugThread extends Thread {

    private InputStream in;
    private OutputStream out;

    public StreamPlugThread(InputStream in, OutputStream out) {
	this.in  = in;
	this.out = out;
    }

    public void run() {
	byte[] buf = new byte[2048];
	int count;
	try {
	    while ((count = in.read(buf)) != -1) {
		out.write(buf, 0, count);
		out.flush();
	    }
	} catch (IOException e) {
	}
    }

    public static void plugTogether(InputStream in, OutputStream out) {
	(new StreamPlugThread(in, out)).start();
    }

    public static void plugTogether(OutputStream out, InputStream in) {
	(new StreamPlugThread(in, out)).start();
    }

    public static Process userProg(String cmd)
	throws IOException
    {
	Process proc = Runtime.getRuntime().exec(cmd);
	plugTogether(System.in,  proc.getOutputStream());
	plugTogether(System.out, proc.getInputStream());
	plugTogether(System.err, proc.getErrorStream());
	return proc;
    }

    public static void main(String[] args) {
	if (args.length != 1)
	    error("usage: StreamPlugThread \"command\"");

	try {
	    userProg(args[0]);
	} catch (IOException e) {
	    error("I/O Exception: " + e);
	}
    }

    public static void error(String err) {
	System.err.println("StreamPlugThread: " + err);
	System.exit(1); // non-zero argument means "not good"
    }
}
