/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package net.jini.discovery;

import java.net.InetAddress;
import java.net.UnknownHostException;

/**
 * A holder class for constants that pertain to the unicast and
 * multicast discovery protocols.
 * 
 * @author Bryan O'Sullivan
 */
public class Constants {
    /**
     * The address of the multicast group over which the multicast
     * request protocol takes place.
     */
    private static InetAddress requestAddress = null;

    /**
     * The address of the multicast group over which the multicast
     * announcement protocol takes place.
     */
    private static InetAddress announcementAddress = null;

    /**
     * The port for both unicast and multicast boot requests.
     */
    public static final short discoveryPort = 4160;

    /**
     * This class cannot be instantiated.
     */
    private Constants() {}
  
    /**
     * Return the address of the multicast group over which the
     * multicast request protocol takes place.
     */
    public static final InetAddress getRequestAddress()
	throws UnknownHostException
    {
	if (requestAddress == null)
	    requestAddress = InetAddress.getByName("224.0.1.85");
	return requestAddress;
    }
  

    /**
     * Return the address of the multicast group over which the
     * multicast announcement protocol takes place.
     */
    public static final InetAddress getAnnouncementAddress()
	throws UnknownHostException
    {
	if (announcementAddress == null)
	    announcementAddress = InetAddress.getByName("224.0.1.84");
	return announcementAddress;
    }
}
