/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package net.jini.discovery;

/**
 * This interface must be implemented by entities that wish to receive
 * notification of the following events:
 * <p><ul>
 *   <li> discovery of new lookup services
 *   <li> re-discovery of previously discovered but discarded lookup services
 *   <li> discard of previously discovered lookup services
 *   <li> changes in the member groups of previously discovered lookup services
 * </ul><p>
 *
 * @see net.jini.discovery.DiscoveryListener
 * @see net.jini.discovery.DiscoveryEvent
 * @see net.jini.discovery.LookupDiscovery
 */
public interface DiscoveryChangeListener extends DiscoveryListener {
    /**
     * Called when changes are detected in the discovery state of 
     * one or more of the previously discovered lookup services;
     * in particular, when changes occur in the member groups to
     * which those lookup services belong.
     * 
     * Note that implementations of this method should return quickly;
     * e.g., such implementations should never make remote invocations.
     *
     * @param e instance of <code>net.jini.discovery.DiscoveryEvent</code>
     *          representing the event that describes the lookup services
     *          whose discovery state has changed
     */
    void changed(DiscoveryEvent e);
}
