/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package net.jini.discovery;

import java.io.DataInputStream;
import java.io.InputStream;
import java.io.IOException;

/**
 * Encapsulate the details of unmarshaling an incoming unicast
 * request.  This class has no public methods because the request
 * portion of the current version of the unicast discovery protocol
 * does nothing other than indicate its version.
 *
 * @see OutgoingUnicastRequest
 * @author Bryan O'Sullivan
 */
public class IncomingUnicastRequest {
    /**
     * The current version of the unicast discovery protocol.
     */
    protected int protoVersion = 1;

    /**
     * Construct a new object, initialized by unmarshaling the
     * contents of an input stream.
     *
     * @param str the stream from which to unmarshal a request
     * @exception IOException the request could not be unmarshaled
     */
    public IncomingUnicastRequest(InputStream str) throws IOException {
	DataInputStream dstr = new DataInputStream(str);
	int proto = dstr.readInt();
	if (proto != protoVersion)
	    throw new IOException("unsupported protocol version: " + proto);
    }
}
