/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package net.jini.discovery;

import java.io.ObjectOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.rmi.MarshalledObject;
import net.jini.core.lookup.ServiceRegistrar;

/**
 * Encapsulate the details of marshaling a unicast response.
 *
 * @see IncomingUnicastResponse
 * @author Bryan O'Sullivan
 */
public class OutgoingUnicastResponse {
    /**
     * The current version of the unicast discovery protocol.
     */
    protected int protoVersion = 1;

    /**
     * Marshal a unicast response to the given output stream.  The
     * stream is flushed afterwards.
     *
     * @param str the stream to marshal to
     * @param reg the registrar object to marshal
     * @param groups the groups in which the registrar is currently a
     * member
     * @exception IOException a problem occurred during marshaling
     */
    public static void marshal(OutputStream str,
			       ServiceRegistrar reg,
			       String[] groups)
	throws IOException
    {
	ObjectOutputStream ostr = new ObjectOutputStream(str);
	ostr.writeObject(new MarshalledObject(reg));
	ostr.writeInt(groups.length);
	for (int i = 0; i < groups.length; i++) {
	    ostr.writeUTF(groups[i]);
	}
	ostr.flush();
    }
}
