/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package net.jini.event;

import java.rmi.RemoteException;

/**
 * The <code>EventMailbox</code> interface allows clients
 * to specify and use a third party for the purpose of
 * storing and retrieving events.
 *
 * @see MailboxRegistration
 * @author Gary Holness
 * @since Jini1.1
 */

public interface EventMailbox {
    /**
     * Defines the interface to the event mailbox service.
     * Event mailbox clients utilize this service by invoking
     * the <code>register</code> method to register themselves with
     * the service.
     *
     * @param leaseDuration the requested lease duration in milliseconds
     * @return A new <code>MailboxRegistration</code> 
     *
     * @throws <code>IllegalArgumentException</code> if 
     * <code>leaseDuration</code> is not positive or <code>Lease.ANY</code>.
     *
     * @throws <code>java.rmi.RemoteException</code> if there is
     *  a communication failure between the client and the service.
     */
    MailboxRegistration register(long leaseDuration) throws RemoteException;
}
