/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package net.jini.lease;

import java.rmi.MarshalledObject;
import net.jini.core.event.RemoteEvent;
import net.jini.core.lease.Lease;

/**
 * Event generated by a lease renewal set when its lease is about to expire.
 *
 * @see LeaseRenewalSet 
 */
public class ExpirationWarningEvent extends RemoteEvent {
    private static final long serialVersionUID = -2020487536756927350L;

    /**
     * Simple constructor.  Note event id is fixed to
     * <code>LeaseRenewalSet.EXPIRATION_WARNING_EVENT_ID</code>
     *
     * @param source the <code>LeaseRenewalSet</code> that generated the event
     * @param seqNum the sequence number of this event
     * @param handback the client handback
     * @param lease the lease which is about to expire 
     */
    public ExpirationWarningEvent(LeaseRenewalSet  source, 
                                  long             seqNum, 
                                  MarshalledObject handback)
    {
	super(source, LeaseRenewalSet.EXPIRATION_WARNING_EVENT_ID, seqNum,
	      handback);
    }

    /**
     * Convenience method to retrieve the <code>Lease</code>
     * associated with the source of this event. This is the
     * <code>Lease</code> which is about to expire.
     * <p>
     * The <code>Lease</code> object returned will be equivalent (in
     * the sense of <code>.equals()</code>) to other
     * <code>Lease</code> objects associated with the set, but may not
     * be the same object. One notable consequence of having two
     * different objects is that the <code>getExpiration</code> method
     * of the <code>Lease</code> object returned by this method may
     * return a different time than the <code>getExpiration</code>
     * methods of other <code>Lease</code> objects granted on the same
     * set.  
     * <p>
     * The expiration time associated with the <code>Lease</code>
     * object returned by this method will reflect the expiration the
     * lease had when the event occurred. Renewal calls may have
     * changed the expiration time of the underlying lease between the
     * time when the event was generated and when it was delivered.
     */
    public Lease getRenewalSetLease() {
	return ((LeaseRenewalSet)source).getRenewalSetLease();
    }
}


