/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package net.jini.lease;

import java.rmi.RemoteException;

/**
 * Interface to the lease renewal service.  The interface is not a
 * remote interface; each implementation of the renewal service
 * exports proxy objects that implement the
 * <code>LeaseRenewalService</code> interface that use an
 * implementation-specific protocol to communicate with the actual
 * remote server.  All of the proxy methods obey normal RMI remote
 * interface semantics.  Two proxy objects are equal if they are
 * proxies for the same renewal service.  Every method invocation (on
 * both a <code>LeaseRenewalService</code> and any
 * <code>LeaseRenewalSet</code> it has created) is atomic with respect
 * to other invocations.
 * 
 * @see LeaseRenewalSet 
 */

public interface LeaseRenewalService {
    /**
     * Create a new <code>LeaseRenewalSet</code> that the client can
     * populate with leases to be renewed. The initial duration of the
     * lease granted on this set will be less than or equal to
     * <code>leaseDuration</code>.
     * <p>
     * Two calls to this method should never return objects that are
     * equal.
     *
     * @param leaseDuration requested lease duration in milliseconds.
     * @return A new <code>LeaseRenewalSet</code> in the renewal service
     *
     * @throws IllegalArgumentException if <code>leaseDuration</code>
     * is not positive, <code>Lease.ANY</code>, or
     * <code>Lease.FOREVER</code>.
     *
     * @throws java.rmi.RemoteException 
     */
    public LeaseRenewalSet createLeaseRenewalSet(long leaseDuration) 
	throws RemoteException;
}

