/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package net.jini.lookup.entry;

import java.io.Serializable;
import net.jini.core.entry.Entry;

/**
 * A JavaBeans(TM) component that encapsulates a Comment object.
 *
 * @see Comment
 * @author Bryan O'Sullivan
 */
public class CommentBean implements EntryBean, Serializable {
    private static final long serialVersionUID = 5272583409036504625L;

    /**
     * The Entry object associated with this JavaBeans component.
     *
     * @serial
     */
    protected Comment assoc;

    /**
     * Construct a new JavaBeans component, linked to a new empty 
     * Comment object.
     */
    public CommentBean() {
	assoc = new Comment();
    }

    /**
     * Make a link to an Entry object.
     *
     * @param e the Entry object to link to
     * @exception java.lang.ClassCastException the Entry is not of the
     * correct type for this JavaBeans component
     */
    public void makeLink(Entry e) {
	assoc = (Comment) e;
    }

    /**
     * Return the Entry linked to by this JavaBeans component.
     */
    public Entry followLink() {
	return assoc;
    }

    /**
     * Return the value of the comment field in the Comment object linked to
     * by this JavaBeans component.
     */
    public String getComment() {
	return assoc.comment;
    }

    /**
     * Set the value of the comment field in the Comment object linked to by
     * this JavaBeans component.
     */
    public void setComment(String x) {
	assoc.comment = x;
    }
}
