/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package net.jini.lookup.entry;

import java.io.Serializable;
import net.jini.core.entry.Entry;

/**
 * A JavaBeans(TM) component that encapsulates a Status object.  Since the 
 * Status class is abstract, so too is this JavaBeans component.
 *
 * @see Status
 * @author Bryan O'Sullivan
 */
public abstract class StatusBean implements EntryBean, Serializable {
    private static final long serialVersionUID = -1975539395914887503L;

    /**
     * The Status object associated with this JavaBeans component.
     *
     * @serial
     */
    protected Status assoc;

    /**
     * Unlike other EntryBean constructors, this one does <i>not</i>
     * establish a link to a new object.
     */
    protected StatusBean() {
    }

    /**
     * Make a link to an Entry object.
     *
     * @param e the Entry object to link to
     * @exception java.lang.ClassCastException the Entry is not of the
     * correct type for this component
     */
    public void makeLink(Entry e) {
	assoc = (Status) e;
    }

    /**
     * Return the Entry linked to by this JavaBeans component.
     */
    public Entry followLink() {
	return assoc;
    }

    /**
     * Get the value of the severity field of the Status object to
     * which this JavaBeans component is linked.
     */
    public StatusType getSeverity() {
	return assoc.severity;
    }

    /**
     * Set the value of the severity field of the Status object to
     * which this JavaBeans component is linked.
     *
     * @param x the new value to put in place
     */
    public void setSeverity(StatusType x) {
	assoc.severity = x;
    }
}
