/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.mahalo;

import com.sun.jini.debug.Debug;
import com.sun.jini.mahout.ServiceConfig;
import com.sun.jini.start.ServiceStarter;

import net.jini.core.transaction.server.TransactionManager;

import java.io.IOException;
import java.io.PrintWriter;
import java.rmi.Remote;
import java.rmi.activation.ActivationException;

/**
 * This class contains the public methods and command line interface for
 * creating an activatable instance of the mahalo implementation of the
 * transaction manager service. This class empoloys the utility class 
 * <code>com.sun.jini.start.ServiceStarter</code> to start the desired
 * service. That utility class requires that this class define a
 * <code>public static create</code> method which performs all functions
 * that are specific to the desired service, and which can be invoked by the
 * the utility class.
 *
 * @see com.sun.jini.start.ServiceStarter
 */
public class StartMahalo {

    /**
     * Utility for outputting debug messages.
     */
    private static final Debug debug = new Debug(TxnManagerImpl.DEBUG_PROP);

    /**
     * Print writer for startup related messages.
     */
    private static final PrintWriter dbgStartup = debug.getWriter("startup");

    /** 
     * The fully-qualified path.classname of this class, which will be
     * passed to the <code>com.sun.jini.start.ServiceStarter</code>
     * utility class
     */
    private static String starterClass = StartMahalo.class.getName();

    /** 
     * The fully-qualified path.classname of the class that defines the
     * implementation of the backend server of the service that is to be
     * started
     */
    private static String implClass = TxnManagerImpl.class.getName();

    /** 
     * The name used to identify the particular resource bundle from which
     * usage/error message information may be retrieved for display
     * (helpful with internationalization)
     */
    private static String resourcename = "service";

    /**
     * Command line interface for starting an instance of the service
     * referenced in the <code>implClass</code> field
     * <p>
     *
     * @param args <code>String</code> array containing the command line
     *             arguments
     *
     * @see com.sun.jini.start.ServiceStarter
     */
    public static void main(String[] args) throws Throwable {
        TransactionManager txnMgr = (TransactionManager)
	    ServiceStarter.create(args, starterClass, implClass, resourcename);
	ServiceConfig.config(txnMgr, "Mahalo", 
	    TxnManager.TXNMGR_NAME_PROP, TxnManager.DEFAULT_NAME);
    }//end main

    /**
     * This method defines how the service referenced in the
     * <code>implClass</code> field of this class is to be started.
     * This method will be invoked through reflection by the utility
     * class <code>com.sun.jini.start.ServiceStarter</code>. This method
     * performs all functions that require knowledge of, or access to,
     * the service-specific information; information that the utility
     * class cannot easily obtain or exploit while attempting to start
     * the desired service.
     * <p>
     * This method returns a proxy that provides client-side access to
     * the backend server of the started service.
     *
     * @param serverStub a <code>Remote</code> reference to the backend server
     *                   of the started service
     * 
     * @return an instance of the public interface to the desired service
     *         referenced in the <code>implClass</code> field of this class;
     *         that is, a proxy that provides client-side access to the
     *         backend server of the started service
     * 
     * @throws java.rmi.activation.ActivationException this is a general
     *         exception thrown by the activation system to indicate a
     *         a number of exceptional conditions that prevent the
     *         activation system from fulfilling a particular request.
     * 
     * @throws java.io.IOException 
     *
     * @see com.sun.jini.start.ServiceStarter
     */
    public static TransactionManager create(Remote serverStub)
        throws ActivationException, IOException
    {
        if (dbgStartup != null) 
            dbgStartup.println("StartMahalo::create(stub=" + 
		serverStub + ")");
	return (TransactionManager)serverStub;
    }//end create

}
