/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.mercury;

import com.sun.jini.start.ServiceStarter;

import net.jini.event.EventMailbox;

import java.io.IOException;
import java.rmi.activation.ActivationException;
import java.rmi.Remote;

/**
 * This class contains the public methods and command-line interface for
 * creating an activatable instance of the Mercury implementation of the
 * event mailbox service. This class empoloys the utility class 
 * <code>com.sun.jini.start.ServiceStarter</code> to start the desired
 * service. That utility class requires that this class define a
 * <code>public static create</code> method which performs all functions
 * that are specific to the desired service, and which can be invoked by the
 * the utility class.
 *
 * @see com.sun.jini.start.ServiceStarter
 * @since Jini1.1
 */
public class StartMercury {
    /** 
     * The fully qualified name of this class, which will be
     * passed to the <code>com.sun.jini.start.ServiceStarter</code>
     * utility class
     */
    private static String starterClass = StartMercury.class.getName();

    /** 
     * The fully qualified name of the class that defines the
     * implementation of the service to be started
     */
    private static String implClass = MailboxImpl.class.getName();

    /** 
     * The name used to identify the particular resource bundle from which
     * usage and error message information may be retrieved for display
     * (helpful with internationalization)
     */
    private static String resourcename = "service";

    /**
     * Command-line interface for starting an event mailbox service instance.
     *
     * @param args <code>String</code> array containing the service's command 
     *             line  arguments
     *
     * @see com.sun.jini.start.ServiceStarter
     */
    public static void main(String[] args) {
        ServiceStarter.create(args, starterClass, implClass, resourcename);
    }//end main

    /**
     * This method defines how the event mailbox service is to be started.
     * This method will be invoked through reflection by the utility
     * class <code>com.sun.jini.start.ServiceStarter</code>. This method
     * performs all functions that require knowledge of, or access to,
     * the service-specific information; information that the utility
     * class cannot easily obtain or exploit while attempting to start
     * the desired service.
     * <p>
     * This method returns a proxy that provides client-side access to
     * the event mailbox service.
     *
     * @param serverStub a <code>Remote</code> reference to the event mailbox
     *                   service
     * 
     * @return a proxy object that implements the <code>EventMailbox</code>
     *         interface and provides client-side access to the
     *         actual service implementation.
     * 
     * @throws java.rmi.activation.ActivationException if there is a 
     *         problem activating the service
     * 
     * @throws java.io.IOException  if an I/O error occurs
     *
     * @see com.sun.jini.start.ServiceStarter
     */
    public static EventMailbox create(Remote serverStub)
                                      throws ActivationException, IOException
    {
	return new MailboxProxy((MailboxBackEnd)serverStub,
                                ((MailboxBackEnd)serverStub).getProxyID());
    }//end create

}//end class StartMercury
