/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.mercury;

import com.sun.jini.start.ServiceStarter;

import java.io.IOException;
import java.rmi.RMISecurityManager;
import java.rmi.server.RemoteObject;

import net.jini.event.EventMailbox;

/**
 * Public methods and command-line interface for creating a transient instance
 * of the Mercury implementation of the event mailbox service.
 *
 * @since Jini1.1
 */
public class TransientMailbox {

    /** 
     * The fully qualified classname of this class, which will be
     * passed to the <code>com.sun.jini.start.ServiceStarter</code>
     * utility class
     */
    private static String starterClass = TransientMailbox.class.getName();

    /** 
     * The name used to identify the particular resource bundle from which
     * usage/error message information may be retrieved for display
     * (helpful with internationalization)
     */
    private static String resourcename = "service";

    /**
     * Command line interface for starting an instance of the transient
     * version of the desired service
     * <p>
     *
     * @param args <code>String</code> array containing the command line
     *             arguments
     *
     * @see com.sun.jini.start.ServiceStarter
     */
    public static void main(String[] args) {
        ServiceStarter.createTransient (
            args, starterClass,
            System.getProperty("java.class.path"),
            System.getProperty("java.rmi.server.codebase"),
            resourcename);
    }//end main

    /**
     * This method defines how the desired service is to be started.
     * This method will be invoked through reflection by the utility
     * class <code>com.sun.jini.start.ServiceStarter</code>. This method
     * performs all functions that require knowledge of, or access to,
     * the service-specific information; information that the utility
     * class cannot easily obtain or exploit while attempting to start
     * the desired service.
     * <p>
     * This method returns a proxy that provides client-side access to
     * the backend server of the started service.
     *
     * @param logDirname <code>String</code> containing the full path and
     *                   directory name in which the state of the service to
     *                   be created will be recorded for persistence
     *
     * @return an instance of the public interface to the service that is
     *         to be started by this method; that is, a proxy that provides
     *         client-side access to the backend server of the started service
     *
     * @throws java.io.IOException
     *
     * @see com.sun.jini.start.ServiceStarter
     */
    public static EventMailbox create(String logDirname) throws IOException {
        if(System.getSecurityManager() == null) {
            System.setSecurityManager(new RMISecurityManager());
        }
        MailboxImpl mb = new MailboxImpl(logDirname);
        return new MailboxProxy((MailboxBackEnd)RemoteObject.toStub(mb), 
				mb.getProxyID());
    }//end create
}
