/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.norm;

import com.sun.jini.start.ServiceStarter;

import net.jini.lease.LeaseRenewalService;

import java.io.IOException;
import java.rmi.activation.ActivationException;
import java.rmi.Remote;

/**
 * This class contains the public methods and command line interface
 * for creating an activatable instance of the Norm implementation of
 * the lease renewal service. This class empoloys the utility class
 * <code>com.sun.jini.start.ServiceStarter</code> to start the desired
 * service. That utility class requires that this class define a
 * <code>public static create</code> method which performs all
 * functions that are specific to the desired service, and which can
 * be invoked by the the utility class.
 *
 * @see com.sun.jini.start.ServiceStarter
 */
public class StartNorm {
    /**
     * This method will be invoked through reflection by the utility
     * class <code>com.sun.jini.start.ServiceStarter</code>. This method
     * performs all functions that require knowledge of, or access to,
     * the Norm-specific information; information that the utility
     * class cannot easily obtain or exploit while attempting to start
     * a Norm server.
     * <p>
     * This method returns a proxy that provides client-side access to
     * a Norm server
     *
     * @param serverStub a <code>Remote</code> reference to a
     *         <code>NormServerImpl</code>
     * 
     * @return an instance of the public interface to a Norm server,
     *         a proxy that provides client-side access to the
     *         Norm server.
     *
     * @see com.sun.jini.start.ServiceStarter
     */
    public static LeaseRenewalService create(Remote serverStub) {
	return (NormServer)serverStub;
    }

    /**
     * Command line interface for creating a new Norm server
     * <p>
     * @param args <code>String</code> array containing the command line
     *             arguments. See <code>com.sun.jini.start.ServiceStarter</code>
     *             for details on how args will be parsed.
     *
     * @see com.sun.jini.start.ServiceStarter
     */
    public static void main(String[] args) {
        ServiceStarter.create(args, 
			      StartNorm.class.getName(),
			      NormServerImpl.class.getName(),
			      "service");
    }
}
