/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */
package com.sun.jini.reggie;

import com.sun.jini.start.ServiceStarter;

import net.jini.core.lookup.ServiceRegistrar;

import java.io.IOException;
import java.rmi.RMISecurityManager;
import java.rmi.server.RemoteObject;

/**
 * This class contains the public methods and command line interface for
 * creating a non-activatable, non-persistent (transient) instance of the 
 * reggie implementation of the lookup service. This class empoloys the
 * utility class <code>com.sun.jini.start.ServiceStarter</code> to start
 * the lookup service. That utility class requires that this class
 * define a <code>public static create</code> method which performs all
 * functions that are specific to the lookup service, and which can be
 * invoked by the the utility class.
 *
 * @see com.sun.jini.start.ServiceStarter
 */
public class TransientLookup {
    /** The fully-qualified path.classname of this class, which will be
     *  passed to the <code>com.sun.jini.start.ServiceStarter</code>
     *  utility class
     */
    private static String starterClass = TransientLookup.class.getName();
    /** The name used to identify the particular resource bundle from which
     *  usage/error message information may be retrieved for display
     *  (helpful with internationalization)
     */
    private static String resourcename = "lookup";
    /**
     * Command line interface for starting an instance of the transient
     * version of the lookup service
     * <p>
     *
     * @param args <code>String</code> array containing the command line
     *             arguments
     *
     * @see com.sun.jini.start.ServiceStarter
     */
    public static void main(String[] args) {
        ServiceStarter.createTransient
                               (args, starterClass, 
                                System.getProperty("java.class.path"),
                                System.getProperty("java.rmi.server.codebase"),
                                resourcename);
    }//end main

    /**
     * This method defines how the desired lookup service is to be started.
     * This method will be invoked through reflection by the utility
     * class <code>com.sun.jini.start.ServiceStarter</code>. This method
     * performs all functions that require knowledge of, or access to,
     * the service-specific information; information that the utility
     * class cannot easily obtain or exploit while attempting to start
     * the lookup service.
     * <p>
     * This method returns a proxy that provides client-side access to
     * the backend server of the started lookup service.
     *
     * @param logDirname <code>String</code> containing the full path and
     *                   directory name in which the state of the lookup
     *                   service to be created will be recorded for persistence
     * 
     * @return an instance of the public interface to the lookup service that
     *         is to be started by this method; that is, a proxy that provides
     *         client-side access to the backend server of the started lookup
     *         service
     * 
     * @throws java.io.IOException 
     *
     * @see com.sun.jini.start.ServiceStarter
     */
    public static ServiceRegistrar create(String logDirname)
                                                           throws IOException
    {
        if(System.getSecurityManager() == null) {
            System.setSecurityManager(new RMISecurityManager());
        }
        RegistrarImpl registrarImpl = new RegistrarImpl(logDirname);
        Registrar serverStub = (Registrar)RemoteObject.toStub(registrarImpl);
	return new RegistrarProxy(serverStub,serverStub.getServiceID());
    }//end create

}//end class TransientLookup
