/*
 * 
 * Copyright 2000 Sun Microsystems, Inc. All Rights Reserved.
 * 
 * The contents of this file are made available under and subject to the Research
 * Use License of the Sun Community Source License, Jini Technology Core Platform
 * Version 1.0 (the "License").  This file has not been tested and may not pass
 * the relevant Technology Compatibility Kit for Internal Deployment Use or
 * Commercial Use.  You must test this file and it must constitute Compliant
 * Covered Code in accordance with Your License prior to any Internal Deployment
 * Use or Commercial Use by You.
 * 
 * Contributor(s): Sun Microsystems, Inc.
 * 
 * Version 1.1Beta
 * 
 */

package net.jini.lookup;

import  net.jini.core.lookup.ServiceItem;

/**
 * The <code>ServiceItemFilter</code> interface defines the methods
 * used by an object such as the <code>ServiceDiscoveryManager</code> or
 * the<code>LookupCache</code> to apply additional matching criteria 
 * when searching for services in which an entity has registered 
 * interest. It is the responsibility of the entity requesting the
 * application of additional criteria to construct an implementation of
 * this interface that defines the additional criteria, and to pass the
 * resulting object (referred to as a <i>filter</i>) into the object
 * that will apply it.
 * <p> 
 * The filtering mechanism provided by implementations of this
 * interface is particularly useful to entities that wish to extend the
 * capabilities of the standard template matching scheme. For example,
 * because template matching does not allow one to search for services
 * based on a range of attribute values, this additional matching
 * mechanism can be exploited by the entity to ask the managing object
 * to find all registered printer services that have a resolution
 * attribute between say, 300 dpi and 1200 dpi.
 *
 * @see ServiceDiscoveryManager
 * @author Zane Pan
 */
public interface ServiceItemFilter {

    /**
     * The check method defines the implementation of the additional matching
     * criteria to apply to a <code>ServiceItem</code> object found through 
     * standard template matching. 
     * 
     * @param item the <code>ServiceItem</code> object to test against the 
     *             additional criteria.
     *
     * @return <code>true</code> if the input object satisfies the additional
     *         criteria, and <code>false</code> otherwise.
     * 
     */
    boolean  check(ServiceItem item);
}
